<?php
/**
 * File:
 * class.smarty.php
 *
 * Project:
 * DZIRCMS Content Management System
 *
 * Description:
 * New wrapper for Smarty lib in external folder
 *
 * @package     Smarty4
 * @version     1.0.0
 * @author      René Mansveld
 * @copyright   Spider IT Deutschland
 * @license     https://www.dzir.org/cms/license/LICENCE.txt
 * @link        https://www.dzir.org/cms/
 * @link        https://www.spider-it.de/
 * @since       file available since DZIRCMS release 2.2.0
 *
 * {@internal
 *   created    2022-01-04
 * }}
 */

defined('CON_FRAMEWORK') or die('Illegal call');

cInclude('external', 'smarty/libs/Smarty.class.php');

class Smarty extends Smarty4 {
    
    public function __construct() {
        global $cfg, $cfgClient, $client;
        
        switch (getcwd() . '/') {
            case $cfg['path']['contenido']:
                $this->setCompileDir($cfg['path']['contenido'] . 'data/cache/templates_c/');
                $this->setCacheDir($cfg['path']['contenido'] . 'data/cache/smarty_cache/');
                break;
            default:
                $this->setCompileDir($cfgClient[$client]['path']['frontend'] . 'cache/templates_c/');
                $this->setCacheDir($cfgClient[$client]['path']['frontend'] . 'cache/smarty_cache/');
                break;
        }
        
        parent::__construct();
    }

    public function fetch($template = null, $cache_id = null, $compile_id = null, $parent = null) {
        global $cfg, $cfgClient, $client, $cCurrentModuleName;

        $sModulePath = '';
        if ((getcwd() . '/') != $cfg['path']['contenido']) {
            if ((count(explode('/', $template)) == 1) && (isset($cCurrentModuleName))) {
                cInclude('includes', 'functions.upl.php');
                $sModulePath = uplCreateFriendlyName($cCurrentModuleName);
                $template = 'data/modules/' . $sModulePath . '/templates/' . $template;
            }
        }
        $sCode = parent::fetch($template, $cache_id, $compile_id, $parent);
        if ((getcwd() . '/') != $cfg['path']['contenido']) {
            # Replace css links if the files are in the module's folder
            $p1 = strpos($sCode, '<link');
            while ($p1 !== false) {
                $p2 = strpos($sCode, '>', $p1);
                $tmp = substr($sCode, $p1, (($p2 - $p1) + 1));
                if ((strpos(strtolower($tmp), 'rel="stylesheet"')) || (strpos(strtolower($tmp), "rel='stylesheet'"))) {
                    # Extract the path and filename
                    $url = '';
                    $p3 = strpos($tmp, 'href=');
                    if ($p3 !== false) { # shouldn't happen, an address is needed
                        $p3 += 6;
                        $char = substr($tmp, ($p3 - 1), 1);
                        $p4 = strpos($tmp, $char, $p3);
                        $url = substr($tmp, $p3, ($p4 - $p3));
                        if (count(explode('/', $url)) == 1) {
                            if (file_exists($cfgClient[$client]['path']['frontend'] . 'data/modules/' . $sModulePath . '/css/' . $url)) {
                                $sCode = str_replace($url, 'data/modules/' . $sModulePath . '/css/' . $url, $sCode);
                            }
                        }
                    }
                }
                $p1 = strpos($sCode, '<link', $p2);
            }
            # Replace js links if the files are in the module's folder
            $p1 = strpos($sCode, '<script');
            while ($p1 !== false) {
                $p2 = strpos($sCode, '>', $p1);
                # Check if this is included code, or code directly in the page
                if (substr($sCode, $p2, 10) == '></script>') {
                    # Script link
                    $tmp = substr($sCode, $p1, (($p2 - $p1) + 10));
                    # Check if this script tag has a type or language set
                    $p3 = strpos($tmp, 'type=');
                    if ($p3 !== false) {
                        $p3 += 6;
                        $char = substr($tmp, ($p3 - 1), 1);
                        $p4 = strpos($tmp, $char, $p3);
                        $type = substr($tmp, $p3, ($p4 - $p3));
                        # If type isn't "text/javascript", leave the tag alone
                        if (strtolower($type) != 'text/javascript') {
                            $p1 = strpos($sCode, '<script', $p2);
                            continue;
                        }
                    } else {
                        $p3 = strpos($tmp, 'language=');
                        if ($p3 !== false) {
                            $p3 += 10;
                            $char = substr($tmp, ($p3 - 1), 1);
                            $p4 = strpos($tmp, $char, $p3);
                            $lang = substr($tmp, $p3, ($p4 - $p3));
                            # If language isn't "javascript", leave the tag alone
                            if (strtolower($lang) != 'javascript') {
                                $p1 = strpos($sCode, '<script', $p2);
                                continue;
                            }
                        }
                    }
                    # We now definitely have a javascript link tag
                    $url = '';
                    $p3 = strpos($tmp, 'src=');
                    if ($p3 !== false) { # shouldn't happen, an address is needed
                        $p3 += 5;
                        $char = substr($tmp, ($p3 - 1), 1);
                        $p4 = strpos($tmp, $char, $p3);
                        $url = substr($tmp, $p3, ($p4 - $p3));
                        if (count(explode('/', $url)) == 1) {
                            if (file_exists($cfgClient[$client]['path']['frontend'] . 'data/modules/' . $sModulePath . '/js/' . $url)) {
                                $sCode = str_replace($url, 'data/modules/' . $sModulePath . '/js/' . $url, $sCode);
                            }
                        }
                    }
                }
                $p1 = strpos($sCode, '<script', ($p1 + 1));
            }
            # Replace image paths if the files are in the module's folder
            $p1 = strpos($sCode, '<img');
            while ($p1 !== false) {
                $p2 = strpos($sCode, '>', $p1);
                # Img tag
                $tmp = substr($sCode, $p1, (($p2 - $p1) + 1));
                $url = '';
                $p3 = strpos($tmp, 'src=');
                if ($p3 !== false) { # shouldn't happen, an address is needed
                    $p3 += 5;
                    $char = substr($tmp, ($p3 - 1), 1);
                    $p4 = strpos($tmp, $char, $p3);
                    $url = substr($tmp, $p3, ($p4 - $p3));
                    if (count(explode('/', $url)) == 1) {
                        if (file_exists($cfgClient[$client]['path']['frontend'] . 'data/modules/' . $sModulePath . '/images/' . $url)) {
                            $sCode = str_replace($url, 'data/modules/' . $sModulePath . '/images/' . $url, $sCode);
                        }
                    }
                }
                $p1 = strpos($sCode, '<img', ($p1 + 1));
            }
            # Replace image paths inside style parameters if the files are in the module's folder
            $p1 = strpos($sCode, ' style=');
            while ($p1 !== false) {
                $p1 += 8;
                $c = substr($sCode, ($p1 - 1), 1);
                $p2 = strpos($sCode, $c, $p1);
                $tmp = substr($sCode, $p1, (($p2 - $p1)));
                $p3 = strpos($tmp, 'url(');
                while ($p3 !== false) {
                    $p3 += 5;
                    $char = substr($tmp, ($p3 - 1), 1);
                    $p4 = strpos($tmp, $char, $p3);
                    $url = substr($tmp, $p3, ($p4 - $p3));
                    if (count(explode('/', $url)) == 1) {
                        if (file_exists($cfgClient[$client]['path']['frontend'] . 'data/modules/' . $sModulePath . '/images/' . $url)) {
                            $sCode = str_replace($url, 'data/modules/' . $sModulePath . '/images/' . $url, $sCode);
                        }
                    }
                    $p3 = strpos($tmp, 'url(', $p4);
                }
                $p1 = strpos($sCode, ' style=', ($p1 + 1));
            }
        }
        return $sCode;
    }

    public function display($template = null, $cache_id = null, $compile_id = null, $parent = null) {
        $sCode = $this->fetch($template, $cache_id, $compile_id, $parent);
        echo $sCode;
        return (strlen($sCode) > 0);
    }
}