<?php
/**
 * Project:
 * Contenido Content Management System
 *
 * Description:
 * Layout class
 *
 * Requirements:
 * @con_php_req 5.0
 *
 *
 * @package    Contenido Backend classes
 * @version    1.1
 * @author     Timo Hummel
 * @copyright  four for business AG <www.4fb.de>
 * @license    http://www.contenido.org/license/LIZENZ.txt
 * @link       http://www.4fb.de
 * @link       http://www.contenido.org
 *
 * {@internal
 *   created  2004-08-07
 *   modified 2011-03-15, Murat Purc, adapted to new GenericDB, partly ported to PHP 5, formatting
 *   modified 2018-04-05, René Mansveld, implemented editing in files
 *
 *   $Id$:
 * }}
 *
 */

if (!defined('CON_FRAMEWORK')) {
    die('Illegal call');
}


class cApiLayoutCollection extends ItemCollection
{
    public function __construct()
    {
        global $cfg;
        parent::__construct($cfg["tab"]["lay"], "idlay");
        $this->_setItemClass("cApiLayout");
    }

    /** @deprecated  [2011-03-15] Old constructor function for downwards compatibility */
    public function cApiLayoutCollection()
    {
        cWarning(__FILE__, __LINE__, "Deprecated method call, use __construct()");
        $this->__construct();
    }

    public function create($title)
    {
        global $client;
        $item = parent::create();
        $item->set("name", $title);
        $item->set("idclient", $client);
        $item->store();
        return ($item);
    }

    /**
     * Deletes an item in the table, deletes also existing cache entries and
     * properties of the item.
     *
     * @param   mixed  $mId  Id of entry to delete
     * @return  bool
     */
    protected function _delete($mId) {
        global $cfgClient, $client;

        $oDb = $this->_getSecondDBInstance();

        # Delete layout files
        $sql = 'SELECT `name` FROM `' . $this->table . '` WHERE (`' . $this->primaryKey . '`=' . $mId . ')';
        $oDb->query($sql);
        if ($oDb->next_record()) {
            $sName = $oDb->f('name');
            cInclude('includes', 'functions.upl.php');
            $sLayoutPath = uplCreateFriendlyName($sName);
            if (file_exists($cfgClient[$client]['path']['frontend'] . 'data/layouts/' . $sLayoutPath . '.php')) {
                unlink($cfgClient[$client]['path']['frontend'] . 'data/layouts/' . $sLayoutPath . '.php');
            }
        }

        // delete db entry
        $sql = "DELETE FROM `%s` WHERE %s = '%s'";
        $oDb->query($sql, $this->table, $this->primaryKey, $mId);

        // delete cache entry
        self::$_oCache->removeItem($mId);

        // delete the property values
        $oProperties = $this->_getPropertiesCollectionInstance();
        $oProperties->deleteProperties($this->primaryKey, $mId);

        return ($oDb->affected_rows() == 0) ? false : true;
    }
}


class cApiLayout extends Item
{
    protected $_error;

    protected $_bCodeFromFile = false;
    
    private $_aLayFileEditConf = array(        
        'folder_name' => 'layouts',
        'full_path' => ''
    );

    private $_bNoted = false;

    /**
     * Constructor Function
     * @param  mixed  $mId  Specifies the ID of item to load
     */
    public function __construct($mId = false)
    {
        global $cfg, $cfgClient, $client;
        parent::__construct($cfg["tab"]["lay"], "idlay");
        
        $this->setFilters(array(), array());
        
        $this->_aLayFileEditConf['full_path'] = $cfgClient[$client]["path"]["frontend"] . 'data/' . $this->_aLayFileEditConf['folder_name'];
        if (!is_dir($this->_aLayFileEditConf['full_path'])) {
            mkdir($this->_aLayFileEditConf['full_path'], 0755, true);
        }
        $this->_aLayFileEditConf['full_path'] .= ((substr($this->_aLayFileEditConf['full_path'], -1) == '/') ? '' : '/');

        if ($mId !== false) {
            $this->loadByPrimaryKey($mId);

            # Remember the name from the database for the sync all modules functionality
            $this->set('name-orig', $this->get('name'), true);

            # Mark the non-existing field as not modified in order to keep it from trying to be saved
            unset($this->modifiedValues['name-orig']);
        }
    }

    /** @deprecated  [2011-03-15] Old constructor function for downwards compatibility */
    public function cApiLayout($mId = false)
    {
        cWarning(__FILE__, __LINE__, "Deprecated method call, use __construct()");
        $this->__construct($mId);
    }

    public function getField($field)
    {
        $value = parent::getField($field);

        switch ($field) {
            case "name":
                if ($value == "") {
                    $value = i18n("-- Unnamed Layout --");
                }
        }
        return ($value);
    }

    public function store($bJustStore = false)
    {
        if ($bJustStore) {
            // Just store changes, e.g. if specifying the lay package
            parent::store();
        } else {
            cInclude("includes", "functions.con.php");

            $sOldName = $this->get('name-orig');

            $code = $this->get('code');
            if ($this->get('code-orig') != $this->get('code') && ($_POST['action'] == 'lay_edit')) {
                if ((strpos($code, "\\r") !== false) || (strpos($code, "\\n") !== false) || (strpos($code, "\\'") !== false) || (strpos($code, '\\"') !== false) || (strpos($code, "\\\\\\") !== false)) {
                    $code = stripslashes($code);
                    $this->set('code', $code);
                }
                $this->modifiedValues['code'] = true;
            }

            parent::store();

            # Was the layout renamed? -> rename the file
            if ($this->get('name') != $sOldName) {
                @rename($this->_aLayFileEditConf['full_path'] . uplCreateFriendlyName($sOldName) . '.php', $this->_aLayFileEditConf['full_path'] . uplCreateFriendlyName($this->get('name')) . '.php');
            }

            if ($this->get('code-orig') != $this->get('code')) {

                # Rewrite the code file (if this was edited, it will be rewritten with the very same code)
                if (($_GET['synclay'] != 1) && ($_REQUEST['fromSync'] != 1)) {
                    $fp = fopen($this->_aLayFileEditConf['full_path'] . uplCreateFriendlyName($this->get('name')) . '.php', 'w');
                    fputs($fp, $code);
                    fclose($fp);
                }
            }

            conGenerateCodeForAllartsUsingLayout($this->get("idlay"));
        }
    }
    
    public function getLayFileEditConf() {
        return $this->_aLayFileEditConf;
    }

    /**
     * Overridden parent method for hooking editing in file
     * 
     * @return void
     */
    protected function _onLoad() {
        $this->_setCodeFromPhpFile();
    }

    /**
     * Use a PHP-file, if present, for layout code
     * 
     * @return boolean
     */
    private function _setCodeFromPhpFile () {
        cInclude('includes', 'functions.upl.php');
        $sCodeFile = $this->_aLayFileEditConf['full_path'] . uplCreateFriendlyName($this->get('name')) . '.php';
        return $this->_setFieldFromFile($sCodeFile, 'code');
    }
    
    /**
     * read file and set an object field
     * 
     * @param string $sFile 
     * @param string $Field
     * @return boolean
     */
    private function _setFieldFromFile($sFile, $sField) {

        if ((is_file($sFile)) && (is_readable($sFile))) {
            $iFileSize = (int) filesize($sFile);
            if (($iFileSize) && ($fh = fopen($sFile, 'r'))) {
                $db = new DB();
                $sql = 'SELECT `' . $sField . '`
                        FROM `' . $this->table . '`
                        WHERE (`idlay`=' . $this->get('idlay') . ')';
                $db->query($sql);
                $db->next_record();
                $this->set($sField . '-orig', $db->f($sField), true);
                $db->disconnect();
                unset($this->modifiedValues[$sField . '-orig'], $db);
                $this->set($sField, fread($fh, $iFileSize), false);
                fclose($fh);
            } else {
                $this->set($sField, '');
            }
            switch($sField) {
                case "code":
                    $this->_bCodeFromFile = true;
                    break;
            }
            return true;
        }        
        return false;
    }
    
    public function isLoadedFromFile($sWhat) {
        switch($sWhat) {
            case "code":
                return $this->_bCodeFromFile;
                break;
            
            default:
                return false;
        }
    }
    
    public function getFullPath() {
        global $cfgClient, $client;
        
        return str_replace(dirname($cfgClient[$client]["path"]["frontend"]), '', $this->_aLayFileEditConf['full_path']) . uplCreateFriendlyName($this->get('name')) . '.php';
    }
}

?>