<?php
/**
 * Project: 
 * Contenido Content Management System
 * 
 * Description: 
 * Edit form for layout
 *
 * @package    Contenido Backend includes
 * @version    $Rev$
 * @author     Olaf Niemann
 * @author     Ortwin Pinke
 * @copyright  four for business AG <www.4fb.de>
 * @license    http://www.contenido.org/license/LIZENZ.txt
 * @link       http://www.4fb.de
 * @link       http://www.contenido.org
 * @since      file available since contenido release <= 4.6
 *
 * {@internal
 *   modified 2018-04-05, René Mansveld, implemented editing in files
 *
 *   $Id$:
 * }}
 *
 */

if(!defined('CON_FRAMEWORK')) {
    die('Illegal call');
}

cInclude("includes","functions.upl.php");
cInclude("external", "edit_area/class.edit_area.php");

$noti = '';

if (!isset($idlay)) $idlay = 0;

if ((isset($refreshtemplates)) && ($refreshtemplates != "")) {
    
    # Update all templates for containers with mode fixed and mandatory
    $sql = 'SELECT `idtpl` FROM `' . $cfg['tab']['tpl'] . '` WHERE (`idlay`="' . intval($idlay) . '")';
    $db->query($sql);
    while ($db->next_record()) {
        tplAutoFillModules($db->f("idtpl"));
    }
}

if ($action == "lay_delete") {
    if (!$perm->have_perm_area_action_anyitem($area, $action)) {
        $notification->displayNotification("error", i18n("Permission denied"));    
    } else {
        $errno = layDeleteLayout($idlay);
        $layout->virgin = true;
        $sMsg = $notification->returnNotification("info", i18n("Layout deleted"));
    }
}
if (($action == "lay_new") && (!$perm->have_perm_area_action_anyitem($area, $action))) {
    $notification->displayNotification("error", i18n("No permission"));
} else {
    if ($action == "lay_new") {
        $layouts = new cApiLayoutCollection;    
        $layout = $layouts->create(i18n("-- New Layout --"));
        $layout->store();
        $idlay = $layout->get('idlay');

        # Export the layout into files
        $sLayRoot = $cfgClient[$client]['path']['frontend'] . 'data/layouts/';
        if (!is_dir($sLayRoot)) {
            if (!mkdir($sLayRoot, 0755, true)) {
                $noti .= i18n("Directory not writable") . ': ' . $sLayRoot;
            }
        }
        $sFile = $sLayRoot . uplCreateFriendlyName($layout->get('name')) . '.php';
        $fp = @fopen($sFile, 'w');
        @fwrite($fp, $layout->get('code'));
        @fclose($fp);
        @chmod($sFile, 0644);
    }
    $layout = new cApiLayout($idlay);

    $oInUse = new InUseCollection;
    list($bInUse, $message) = $oInUse->checkAndMark("idlay", $idlay, true, i18n("Layout is in use by %s (%s)"), true, "main.php?area=$area&frame=$frame&idlay=$idlay");
    unset ($oInUse);

    if ($bInUse) {
        $message .= "<br>";
        $disabled = 'disabled';
    } else {
        $disabled = "";
    }

    $page = new cPage;
    $form = new UI_Table_Form("lay_edit");
    $form->setVar("area", $area);
    $form->setVar("frame", $frame);
    $form->setVar("idlay", $idlay);

    if (!$bInUse) {
        $form->setVar("action", "lay_edit");
    }
    $form->setWidth("100%");

    $form->addHeader(i18n("Edit Layout"));

    $name = new cHTMLTextbox("layname", $layout->get("name"), 60);
    $descr = new cHTMLTextarea("descr", htmlspecialchars($layout->get("description"), ENT_COMPAT, $encoding[$lang]), 100, 5);
    $descr->setStyle("width: 100%; font-family: monospace;");

    // Get layout code; if specified, prepare row fields
    $sCode = $layout->get("code");

    $layoutcheck = getSystemProperty("system", "layoutcheck");
        
    $codeok = true;
    if ($layoutcheck !== "false") {
        # Search for duplicate containers
        tplPreparseLayout($idlay, $sCode);
        $ret = tplBrowseLayoutForContainers($idlay, $sCode);

        if (strlen($ret)) {
            $containers = explode("&", $ret);

            $types = array();

            foreach ($containers as $value) {
                if (strlen($value)) {
                    $container[$value] = 0;

                    # Search for old-style CMS_CONTAINER[x]
                    $container[$value] += substr_count($sCode, "CMS_CONTAINER[$value]");

                    # Search for the new-style containers
                    $count = preg_match_all("/<container( +)id=\\\\\"$value\\\\\"(.*)>(.*)<\/container>/i", addslashes($sCode), $matches);

                    $container[$value] += $count;

                    if (is_array(tplGetContainerTypes($idlay, $value))) {
                        $types = array_merge($types, tplGetContainerTypes($idlay, $value));
                    }
                }
            }

            $types = array_unique($types);
            $layout->setProperty("layout", "used-types", implode(';', $types));

            $msg = "";

            foreach ($container as $key => $value) {
                if ($value > 1) {
                    $msg .= sprintf(i18n("Container %s was defined %s times"), $key, $value)."<br>";     
                }    
            }
        }

        $layoutcheck = (getEffectiveSetting("layout", "htmlvalidator", "true") == "true");

        # Try to validate html
        if ((($layoutcheck) || ($_GET['sync'] == 1)) && ($sCode !== "")) {
            $v = new cHTMLValidator;
            $v->validate($sCode);

            if ((!$v->tagExists("body")) && (!$v->tagExists("BODY"))) {
                $msg .= sprintf(i18n("The body tag does not exist in the layout. This is a requirement for the in-site editing."));
                $msg .= "<br>";    
            }

            if ((!$v->tagExists("head")) && (!$v->tagExists("HEAD"))) {
                $msg .= sprintf(i18n("The head tag does not exist in the layout. This is a requirement for the in-site editing."));
                $msg .= "<br>";    
            }

            $codeok = (count($v->missingNodes) == 0);

            foreach ($v->missingNodes as $value) {
                $idqualifier = "";

                $attr = array();

                if (strlen($value["name"])) {
                    $attr["name"] = "name '" . $value["name"] . "'";
                }

                if (strlen($value["id"])) {
                    $attr["id"] = "id '" . $value["id"] . "'";
                }

                $idqualifier = implode(", ", $attr);

                if (strlen($idqualifier)) {
                    $idqualifier = "($idqualifier)";    
                }
                $msg .= sprintf(i18n("Tag '%s' %s has no end tag (start tag is on line %s char %s)"), $value["tag"], $idqualifier, $value["line"],$value["char"]);
                $msg .= "<br>";
            }
        }
        if (!$codeok) {
            $codeled = '<img align="right" src="images/but_online_no.gif" alt="'.$msg.'" title="'.$msg.'">';
        } else {
            $okMessage = i18n("Layout successfully compiled");
            $codeled = '<img align="right" src="images/but_online.gif" alt="'.$okMessage.'" title="'.$okMessage.'">';
        }
        if ($msg != "") {
            $notification->displayNotification("warning", $msg);
        }
            
        // Store error information in the database (to avoid re-eval for layout overview/menu)
        if ($codeok) {
            $sStatus = "none";
        } else {
            $sStatus = "code";
        }
        
        // If status has been changed, store and show in overview
        $sPrevStatus = $layout->get("error");
        if ($sPrevStatus !== $sStatus) {
            $layout->set("error", $sStatus);
        }
        if (($sPrevStatus !== $sStatus) || ($_GET['sync'] == 1) || ($action == 'lay_edit')) {
            $layout->store();
            $page->setReload();
        }
    }
    $code = new cHTMLTextarea("code", htmlspecialchars($sCode, ENT_COMPAT, $encoding[$lang]), 100, 40, 'code');
    $code->updateAttributes(array("wrap" => "off"));
    $code->setStyle("width: 100%; font-family: monospace;");

    $form->add(i18n('Name'), $name->render());
    $form->add(i18n("Description"), $descr->render());
    $form->add('<table width="100%" border="0" cellspacing="0" cellpadding="0"><tr><td style="vertical-align: top;">'.i18n("Code").'</td><td style="vertical-align: top;">'.$codeled.'</td></tr></table>', $code->render());

    $cb_refresh = new cHTMLCheckbox("refreshtemplates", i18n("On save, apply default modules to new containers"));
    $form->add(i18n("Options"), $cb_refresh->render());

    if ($idlay != 0) {
        $sScript = '<script>
    if (document.getElementById(\'scroll\')) {
        document.getElementById(\'scroll\').onmousedown = triggerClickOn;
        document.getElementById(\'scroll\').onmouseup = triggerClickOff;
        document.getElementById(\'scroll\').style.paddingTop=\'4px\';
        document.getElementById(\'scroll\').style.paddingBottom=\'5px\';
        
    }
</script>';
        $page->setContent($noti.$message.$form->render().$sScript);
    }

    $page->setSubnav("idlay=$idlay", "lay");

    if ($action) {
        if (stripslashes($idlay > 0)) {
            $sReloadScript = "<script>
        var left_bottom = parent.parent.frames['left'].frames['left_bottom'];
        if (left_bottom) {
            var href = left_bottom.location.href;
            href = href.replace(/&idlay[^&]*/, '');
            left_bottom.location.href = href+'&idlay=".$idlay."';
        }
    </script>";
        } else {
            $sReloadScript = "";
        }

        // Only reload overview/menu page, if something may have changed
        $page->addScript('reload', $sReloadScript);
    }

    $oEditArea = new EditArea('code', 'html', substr(strtolower($belang), 0, 2), true, $cfg, !$disabled);
    $page->addScript('editarea', $oEditArea->renderScript());
    if ($_GET['sync'] == 1) {

        # Save the layout
        $page->addScript('sync', '<script>
                setTimeout(function() {
                    var f = document.forms.lay_edit;
                    var c = document.createElement("input");
                    c.type = "hidden";
                    c.name = "fromSync";
                    c.value = 1;
                    f.appendChild(c);
                    f.submit();
                }, 100);
            </script>');
    }
}
$page->render();
?>