<?php
/**
 * Project:
 * Contenido Content Management System
 *
 * Description:
 * List layouts in database
 *
 * Requirements:
 * @con_php_req 5.0
 *
 *
 * @package    Contenido Backend includes
 * @version    1.0.1
 * @author     Olaf Niemann
 * @copyright  four for business AG <www.4fb.de>
 * @license    http://www.contenido.org/license/LIZENZ.txt
 * @link       http://www.4fb.de
 * @link       http://www.contenido.org
 * @since      file available since contenido release <= 4.6
 *
 * {@internal
 *   created 2003-03-27
 *   modified 2008-06-27, Frederic Schneider, add security fix
 *   modified 2010-08-18, Munkh-Ulzii Balidar, add a functionality to show the used info
 *   modified 2018-04-05, René Mansveld, implemented editing in files
 *
 *   $Id$:
 * }}
 *
 */

if (!defined('CON_FRAMEWORK')) {
    die('Illegal call');
}

if (!(int) $client > 0) {
    #if there is no client selected, display empty page
    $oPage = new cPage;
    $oPage->render();
    return;
}

############################
# Now build bottom with list
############################
$oApiLayoutCollection = new cApiLayoutCollection();
$oPage = new cPage;

// no value found in request for items per page -> get form db or set default
$oUser = new cApiUser($auth->auth["uid"]);
if ((!isset($_REQUEST["elemperpage"])) || (!is_numeric($_REQUEST['elemperpage'])) || ($_REQUEST['elemperpage'] < 0)) {
    $_REQUEST["elemperpage"] = $oUser->getProperty("itemsperpage", $area);
}
if (!is_numeric($_REQUEST["elemperpage"])) {
    $_REQUEST["elemperpage"] = 0;
}
if ($_REQUEST["elemperpage"] > 0) {
    // -- All -- will not be stored, as it may be impossible to change this back to something more useful
    $oUser->setProperty("itemsperpage", $area, $_REQUEST["elemperpage"]);
}
unset ($oUser);

if ((!isset($_REQUEST["page"])) || (!is_numeric($_REQUEST['page'])) || ($_REQUEST['page'] <= 0) || ($_REQUEST["elemperpage"] == 0)) {
    $_REQUEST["page"] = 1;
}

// Sort by requested
if ((isset($_REQUEST["sortby"])) && (strlen($_REQUEST["sortby"]))) {
    $oApiLayoutCollection->setOrder($_REQUEST["sortby"]." ".$_REQUEST["sortorder"]);
} 
else {
    $oApiLayoutCollection->setOrder("name asc");
}

// Search filter requested
if ((isset($_REQUEST["filter"])) && ($_REQUEST["filter"] != '')  && ($_REQUEST["searchin"] == '')) {
    $oApiLayoutCollection->setWhereGroup("default", "name", "%".$_REQUEST["filter"]."%", "LIKE");
    $oApiLayoutCollection->setWhereGroup("default", "description", "%".$_REQUEST["filter"]."%", "LIKE");
    $oApiLayoutCollection->setWhereGroup("default", "code", "%".$_REQUEST["filter"]."%", "LIKE");
    $oApiLayoutCollection->setInnerGroupCondition("default", "OR");
} else if (isset ($_REQUEST["filter"]) && $_REQUEST["filter"] != ""  && $_REQUEST["searchin"] == 'name') {
    $oApiLayoutCollection->setWhereGroup("default", "name", "%".$_REQUEST["filter"]."%", "LIKE");
    $oApiLayoutCollection->setInnerGroupCondition("default", "OR");
} else if (isset ($_REQUEST["filter"]) && $_REQUEST["filter"] != ""  && $_REQUEST["searchin"] == 'description') {
    $oApiLayoutCollection->setWhereGroup("default", "description", "%".$_REQUEST["filter"]."%", "LIKE");
    $oApiLayoutCollection->setInnerGroupCondition("default", "OR");
} else if (isset ($_REQUEST["filter"]) && $_REQUEST["filter"] != ""  && $_REQUEST["searchin"] == 'code') {
    $oApiLayoutCollection->setWhereGroup("default", "code", "%".$_REQUEST["filter"]."%", "LIKE");
    $oApiLayoutCollection->setInnerGroupCondition("default", "OR");
}

$oApiLayoutCollection->setWhere("idclient", $client);

if ($_REQUEST["elemperpage"] > 0) {
    $oApiLayoutCollection->query();
    $iItemCount = $oApiLayoutCollection->count();

    if ($iItemCount < (($_REQUEST["page"] - 1) * $_REQUEST["elemperpage"])) {
        $_REQUEST["page"] = 1;
    }
    
    if ($_REQUEST["elemperpage"]*($_REQUEST["page"]) >= $iItemCount+$_REQUEST["elemperpage"] && $_REQUEST["page"]  != 1) {
        $_REQUEST["page"]--;
    }

    $oApiLayoutCollection->setLimit(($_REQUEST["elemperpage"] * ($_REQUEST["page"] -1)), $_REQUEST["elemperpage"]);
} 
else {
    $iItemCount = 0;
}

// Build list for left_bottom considering filter values
$mlist = new UI_Menu;
$iMenu = 0;

# Include the client's config.php in order to know the $force state
include_once($cfgClient[$client]['path']['frontend'] . 'config.php');

$oApiLayoutCollection->query();
while ($oApiLayout = $oApiLayoutCollection->next()) {
    $idlay = $oApiLayout->get("idlay");
    if ($perm->have_perm_item($area, $idlay)) {
        
        $link = new cHTMLLink;
        $link->setMultiLink("lay", "", "lay_edit", "");
        $link->setCustom("idlay", $idlay);
        $link->updateAttributes(array("alt" => $oApiLayout->get("description")));
        $link->updateAttributes(array("title" => $oApiLayout->get("description")));

        $sName = $oApiLayout->get("name");
        $colName = $sName;
/* CHECK LAYOUT FOR ERRORS???
        if ($force) {
            // Check layout and force check, or client's $force has been enabled - check layout (surprisingly...)
            $bInputOK = modTestModule($oApiLayout->get("input"), $idlay."i", false);
            $bOutputOK = modTestModule($oApiLayout->get("output"), $idlay."o", true);
            if (($bInputOK) && ($bOutputOK)) {
                $oApiLayout->set('error', 'none');
                $colName = $sName;              // The set default color: none :)
            }
            elseif ($bOutputOK) {
                $oApiLayout->set('error', 'input');
                $colName = '<font color="#B1AC58">'.$sName.'</font>';
            }
            elseif ($bInputOK) {
                $oApiLayout->set('error', 'output');
                $colName = '<font color="#B1AC58">'.$sName.'</font>';
            }
            else {
                $oApiLayout->set('error', 'both');
                $colName = '<font color="red">'.$sName.'</font>';
            }
            $oApiLayout->store();
            $oApiLayout->loadByPrimaryKey($idlay);
        }
        else {
            // Do not check layouts (or don't force it) - so, let's take a look into the database 
            $sModuleError = $oApiLayout->get("error");
            
            if ($sModuleError == "none") {
                $colName = $sName;
            } 
            else if ($sModuleError == "input" || $sModuleError == "output") {
                $colName = '<font color="#B1AC58">'.$sName.'</font>';
            } 
            else {
                $colName = '<font color="red">'.$sName.'</font>';
            }
        }
*/

        $iMenu ++;

        $mlist->setTitle($iMenu, $colName);
        if ($perm->have_perm_area_action_item("lay_edit", "lay_edit", $idlay) || $perm->have_perm_area_action_item("mod_translate", "mod_translation_save", $idlay)) {
            $mlist->setLink($iMenu, $link);
        }
        
        $todo = new TODOLink("idlay", $idlay, "Layout: $sName", "");

        $inUse = $classlayout->layoutInUse($idlay);
        $deletebutton = "";
        if ($inUse) {
            $inUseString = i18n("Click for more information about usage");
            $deletebutton = '<a href="javascript:;" rel="' . $idlay . '" class="in_used_lay"><img src="'.$cfg['path']['images'].'exclamation.gif" border="0" title="'.$inUseString.'" alt="'.$inUseString.'" style="padding-left: 2px;"></a>';
        } else {
            if ($perm->have_perm_area_action_item("lay", "lay_delete", $idlay)) {
                $delTitle = i18n("Delete layout");
                $delDescr = sprintf(i18n("Do you really want to delete the following layout:<br><br>%s<br>"), $sName);
                $deletebutton = '<a title="'.$delTitle.'" href="javascript://" onclick="box.confirm(\''.$delTitle.'\', \''.$delDescr.'\', \'deleteLayout('.$idlay.')\')"><img src="'.$cfg['path']['images'].'delete.gif" border="0" title="'.$delTitle.'" alt="'.$delTitle.'" style="padding-left: 2px;"></a>';                
            } else {
                $delDescription = i18n("No permission");    
            }
        }

        if ($oApiLayout->get('code-orig') != $oApiLayout->get('code')) {
            $syncbutton = '<a href="javascript:conMultiLink(\'right_top\',\'main.php?area=lay&frame=3&action=&idlay=' . $oApiLayout->get('idlay') . '&contenido=' . $contenido . '\',\'right_bottom\',\'main.php?area=lay_edit&frame=4&action=&sync=1&idlay=' . $oApiLayout->get('idlay') . '&contenido=' . $contenido . '\');"><img src="images/sync_from_files.png" alt="' . i18n("Layout needs synchronization") . '" title="' . i18n("Layout needs synchronization") . '" style="padding-left: 2px;"></a>';
        }
        else {
            $syncbutton = '<img src="images/synchronized.png" alt="' . i18n("Layout is synchronized") . '" title="' . i18n("Layout is synchronized") . '" style="padding-left: 2px;">';
        }
        
        $mlist->setActions($iMenu, "todo", $todo->render());
        $mlist->setActions($iMenu, "delete", $deletebutton);
        $mlist->setActions($iMenu, "sync", $syncbutton);
        
        if (isset($_GET['idlay']) && $_GET['idlay'] == $idlay) {
            $mlist->setExtra($iMenu, 'id="marked" ');
        }     
    }
}

$deleteScript = '
    <script type="text/javascript">

        var sid = "'.$sess->id.'";
        box = new messageBox("", "", "", 0, 0);

        function deleteLayout(idlay) {

            form = document.getElementById("filter");

            url  = \'main.php?area=lay_edit\';
            url += \'&action=lay_delete\';
            url += \'&frame=4\';
            url += \'&idlay=\' + idlay;
            url += \'&contenido=\' + sid;
            url += get_registered_parameters();
            url += \'&sortby=\' + parent.frames[1].document.filter.sortby;
                    url += \'&sortorder=\' + parent.frames[1].document.filter.sortorder;
                    url += \'&filter=\' + parent.frames[1].document.filtertype;
                    url += \'&elemperpage=\' + parent.frames[1].document.filter.elemperpage;
                    url += \'&page=\' + parent.frames[1].document.filter.page;
            parent.parent.right.right_bottom.location.href = url;
        }

    </script>';

$sShowUsedInfo = ' 
    <script type="text/javascript">       
        $(document).ready(function() {
            
            $(".in_used_lay").on("click", function() {
                var iId = $(this).attr("rel");
                if (iId) {
                    var layName = $(this).parents().filter("td[align=left]").next().text();
                    $.post(
                        "' . $cfg['path']['contenido_fullhtml'] . 'ajaxmain.php' . '", 
                        { area: "' . $area . '", ajax: "inused_layout", id: iId, contenido: sid }, 
                        function(data) {
                            var inUseTitle = "' . i18n("The layout '%s' is used for following templates") . '";
                            inUseTitle = inUseTitle.replace(/%s/, layName);    
                            box.notify(inUseTitle, data);
                        } 
                    );
                }    
            });
        });
    </script>';

$sMarkRow = '
    <script language="javascript">    
        if (document.getElementById(\'marked\')) {
            row.click(document.getElementById(\'marked\'));
        }
    </script>';
    
$oPage->setMargin(0);
$oPage->addScript('messagebox', '<script type="text/javascript" src="scripts/messageBox.js.php?contenido='.$sess->id.'"></script>');
$oPage->addScript('jquery', '<script type="text/javascript" src="scripts/jquery/jquery.js"></script>');
$oPage->addScript('delete', $deleteScript);
$oPage->addScript('showUsedInfo', $sShowUsedInfo);
$oPage->addScript('cfoldingrow.js', '<script language="JavaScript" src="scripts/cfoldingrow.js"></script>');
$oPage->addScript('parameterCollector.js', '<script language="JavaScript" src="scripts/parameterCollector.js"></script>');
$oPage->setContent($mlist->render(false).$sMarkRow);

//generate current content for Object Pager
$oPagerLink = new cHTMLLink;
$pagerl="pagerlink";
$oPagerLink->setTargetFrame('left_bottom');
$oPagerLink->setLink("main.php");
$oPagerLink->setCustom("elemperpage", $_REQUEST["elemperpage"]);
if(isset($_REQUEST["filter"])) $oPagerLink->setCustom("filter", stripslashes($_REQUEST["filter"]));
if(isset($_REQUEST["sortby"])) $oPagerLink->setCustom("sortby", $_REQUEST["sortby"]);
if(isset($_REQUEST["sortorder"])) $oPagerLink->setCustom("sortorder", $_REQUEST["sortorder"]);
$oPagerLink->setCustom("frame", $frame);
$oPagerLink->setCustom("area", $area);
$oPagerLink->enableAutomaticParameterAppend();
$oPagerLink->setCustom("contenido", $sess->id);
$oPager = new cObjectPager("02420d6b-a77e-4a97-9395-7f6be480f497", $iItemCount, $_REQUEST["elemperpage"], $_REQUEST["page"], $oPagerLink, "page", $pagerl);

//add slashes, to insert in javascript
$sPagerContent = $oPager->render(1);
$sPagerContent = str_replace('\\', '\\\\', $sPagerContent);
$sPagerContent = str_replace('\'', '\\\'', $sPagerContent);

//send new object pager to left_top
$sRefreshPager = '
    <script type="text/javascript">
        var sNavigation = \''.$sPagerContent.'\';
        var left_top = parent.left_top;
        if (left_top.document) {
            var oPager = left_top.document.getElementById(\'02420d6b-a77e-4a97-9395-7f6be480f497\');
            if (oPager) {
                oInsert = oPager.firstChild;
                oInsert.innerHTML = sNavigation;
                left_top.toggle_pager(\'02420d6b-a77e-4a97-9395-7f6be480f497\');
            }
        }
    </script>';
            
$oPage->addScript('refreshpager', $sRefreshPager); 

$oPage->render();


/*
$oApiLayoutCollection->select("idclient = '$client'", '', 'name ASC');

$tpl->reset();

$tpl->set('s', 'SID', $sess->id);

$darkrow = false;
while ($layout = $oApiLayoutCollection->next()) {

    if (!$perm->have_perm_area_action_item('lay_edit', 'lay_edit', $layout->get('idlay'))) {
        continue;
    }

    $name  = $layout->get('name');
    $descr = $layout->get('description');
    $idlay = $layout->get('idlay');

    if (strlen($descr) > 64) {
        $descr = substr($descr, 0, 64);
        $descr .= '&hellip;';
    }

    $tmp_mstr = '<a href="javascript:conMultiLink(\'%s\', \'%s\', \'%s\', \'%s\')" title="%s" alt="%s">%s</a>';
    $area = 'lay';
    $mstr = sprintf($tmp_mstr, 'right_top',
                               $sess->url("main.php?area=$area&frame=3&idlay=$idlay"),
                               'right_bottom',
                               $sess->url("main.php?area=lay_edit&frame=4&idlay=$idlay"),
                               $descr, $descr, $name);

    $tpl->set('d', 'NAME', $mstr);

    $inUse = $classlayout->layoutInUse($idlay);

    $bgColor = ($darkrow) ? $cfg['color']['table_dark'] : $cfg['color']['table_light'];
    $darkrow = !$darkrow;
    $tpl->set('d', 'BGCOLOR', $bgColor);

    if ((!$perm->have_perm_area_action_item('lay', 'lay_delete', $idlay)) && 
        (!$perm->have_perm_area_action('lay', 'lay_delete'))) {
        $delDescr = i18n("No permission");
    }

    if ($inUse) {
        $delDescr   = i18n("Layout is in use, cannot delete");
        $inUseDescr = i18n("Click for more information about usage");
        $inUseLink = '&nbsp;<a href="javascript:;" rel="' . $idlay . '" class="in_used_lay">'
                   . '<img src="'.$cfg['path']['images'].'exclamation.gif" border="0" title="'.$inUseDescr.'" alt="'.$inUseDescr.'"></a>';
        $tpl->set('d', 'INUSE', $inUseLink);
    } else {
        $tpl->set('d', 'INUSE', '');
    }

    if ($perm->have_perm_area_action_item('lay', 'lay_delete', $idlay) && !$inUse) {
        $delTitle = i18n("Delete layout");
        $delDescr = sprintf(i18n("Do you really want to delete the following layout:<br><br>%s<br>"), htmlspecialchars($name));
        $delLink  = '&nbsp;<a title="'.$delTitle.'" href="javascript://" onclick="box.confirm(\''.$delTitle.'\', \''.$delDescr.'\', \'deleteLayout('.$idlay.')\')">'
                  . '<img src="'.$cfg['path']['images'].'delete.gif" border="0" title="'.$delTitle.'" alt="'.$delTitle.'"></a>';
        $tpl->set('d', 'DELETE', $delLink);
        $tpl->set('d', 'TODO', '');
    } else {
        $tpl->set('d', 'DELETE','');
        $todo = new TODOLink('idlay', $idlay, i18n("Layout") . ': ' . $name, '');
        $tpl->set('d', 'TODO', '&nbsp;' . $todo->render());
    }

    if ($layout->isLoadedFromFile('code')) {
        $usesFilesString = i18n("Layout uses a file. Editing and Saving may not be possible in backend.");
        $tpl->set('d', 'IN_FILE', '&nbsp;<img src="'.$cfg['path']['images'].'uses_files.gif" border="0" title="'.$usesFilesString.'" alt="'.$usesFilesString.'" style="margin-left: 2px;">');
    } else {
        $tpl->set('d', 'IN_FILE', '');
    }

    if (stripslashes($_REQUEST['idlay']) == $idlay) {
        $tpl->set('d', 'ID', 'marked');
    } else {
        $tpl->set('d', 'ID', '');
    }

    $tpl->next();
}

//datas for show of used info per ajax
$tpl->set('s', 'AREA', $area);
$tpl->set('s', 'SESSION', $contenido);
$tpl->set('s', 'AJAXURL', $cfg['path']['contenido_fullhtml'].'ajaxmain.php');
$tpl->set('s', 'BOX_TITLE', i18n("The layout '%s' is used for following templates") . ":");

$tpl->generate($cfg['path']['templates'] . $cfg['templates']['lay_overview']);
*/
?>