<?php
/**
 * Project: 
 * DZIRCMS Content Management System
 * 
 * Description: 
 * Edit file
 * 
 * Requirements: 
 * @con_php_req 7.0
 * 
 *
 * @package    DZIRCMS Backend includes
 * @version    1.0.0
 * @author     René Mansveld
 * @copyright  Spider IT Deutschland
 * @license    https://www.dzir.org/license/LIZENZ.txt
 * @link       https://www.spider-it.de
 * @link       https://www.dzir.org
 * @since      file available since DZIRCMS 2.2.0
 * 
 * {@internal 
 *   created 2022-10-19
 * }
 */

if (!defined('CON_FRAMEWORK')) {
    die('Illegal call');
}

cInclude("external", "edit_area/class.edit_area.php");
cInclude('includes', 'functions.upl.php');

$action = $_REQUEST['action'];
$mode = $_REQUEST['mode'];

$page = new cPage;
$page->setEncoding("utf-8");

if (!$perm->have_perm_area_action($area, $action)) {
    $notification->displayNotification("error", i18n("Permission denied"));
}
elseif ((int) $client == 0) {

    #if there is no client selected, display empty page
    $page->render();
}
else {
    $page->addScript('jQuery', '<script src="scripts/jquery/jquery.js"></script>');

    $idmod = (int) $_REQUEST['idmod'];
    $oModule = new cApiModule($idmod);
    $sName = uplCreateFriendlyName($oModule->get("name"));
    $sPath = $cfgClient[$client]['path']['frontend'] . 'data/modules/' . $sName . '/';

    if ($action == "mod_importexport_module") {
        if ($mode == i18n("Export to ZIP file")) {
            
            if (strlen($sName)) {

                # Export the module into an XML file
                $sXml = $oModule->export($sName . '.xml', true);
                $fp = fopen($sPath . $sName . '.xml', 'w');
                fputs($fp, $sXml);
                fclose($fp);

                # Export the module translations into XML files
                $oMTC = new cApiModuleTranslationCollection();
                $oLC = new cApiLanguageCollection();
                $aClientLangs = $oLC->getClientLanguages($client, true);
                $aLangFiles = array();
                foreach ($aClientLangs as $aClientLang) {
                    $sXml = $oMTC->export($idmod, $aClientLang['idlang'], '', true);
                    $fn = $sName . '_' . uplCreateFriendlyName(strtolower($aClientLang['name'])) . '.xml';
                    $fp = fopen($sPath . $fn, 'w');
                    fputs($fp, $sXml);
                    fclose($fp);
                    $aLangFiles[] = array('langcode' => getProperty($client, 'idlang', $aClientLang['idlang'], 'language', 'code') . '_' . strtoupper(getProperty($client, 'idlang', $aClientLang['idlang'], 'country', 'code')), 'filename' => $fn);
                }

                # Create a module.xml file
                $smarty = new Smarty();
                $smarty->assign('name', $oModule->get("name"));
                $aDesc = explode("\n", $oModule->get('description'));
                $smarty->assign('desc_en_US', trim($aDesc[0]));
                $smarty->assign('desc_de_DE', trim($aDesc[0]));
                $smarty->assign('desc_nl_NL', trim($aDesc[0]));
                $smarty->assign('tag', strtolower($oModule->get('name')));
                $sAuthor = $sVersion = $sCopyright = $sEmail = $sWebsite = '';
                foreach ($aDesc as $sDesc) {
                    if ((strtolower(substr($sDesc, 0, 7)) == 'author:') || (strtolower(substr($sDesc, 0, 6)) == 'autor:')) {
                        $sAuthor = trim(substr($sDesc, (strpos($sDesc, ':') + 1)));
                    }
                    elseif (strtolower(substr($sDesc, 0, 8)) == 'version:') {
                        $sVersion = trim(substr($sDesc, (strpos($sDesc, ':') + 1)));
                    }
                    elseif (strtolower(substr($sDesc, 0, 10)) == 'copyright:') {
                        $sCopyright = trim(substr($sDesc, (strpos($sDesc, ':') + 1)));
                    }
                    elseif ((strtolower(substr($sDesc, 0, 7)) == 'e-mail:') || (strtolower(substr($sDesc, 0, 6)) == 'email:')) {
                        $sEmail = trim(substr($sDesc, (strpos($sDesc, ':') + 1)));
                    }
                    elseif (strtolower(substr($sDesc, 0, 8)) == 'website:') {
                        $sWebsite = trim(substr($sDesc, (strpos($sDesc, ':') + 1)));
                    }
                }
                $smarty->assign('author', $sAuthor);
                $smarty->assign('version', $sVersion);
                $smarty->assign('date', date('Y-m-d'));
                $smarty->assign('copyright', ((strlen($sCopyright)) ? $sCopyright : '© ' . date('Y')));
                $smarty->assign('mail', $sEmail);
                $smarty->assign('website', $sWebsite);
                $smarty->assign('php', PHP_MAJOR_VERSION . '.' . PHP_MINOR_VERSION . '.0');
                $aVersion = explode('.', $cfg['version']);
                $smarty->assign('dzircms', $aVersion[0] . '.' . $aVersion[1] . '.' . $aVersion[2]);
                $smarty->assign('filename', $sName . '.xml');
                $smarty->assign('langfiles', $aLangFiles);
                $sXml = $smarty->fetch($cfg['path']['templates'] . 'module.xml');
                $fp = fopen($sPath . 'module.xml', 'w');
                fputs($fp, $sXml);
                fclose($fp);

                # ZIP the entire module folder
                if (file_exists($sPath . $sName . '.zip')) {
                    unlink($sPath . $sName . '.zip');
                }
                $zip = new ZipArchive();
                $zip->open($sPath . $sName . '.zip', ZipArchive::CREATE | ZipArchive::OVERWRITE);
                $files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($sPath), RecursiveIteratorIterator::LEAVES_ONLY);
                foreach ($files as $name => $file) {
                    $filepath = $file->getRealPath();
                    if (is_dir($file)) {
                        $zip->addEmptyDir(substr($filepath, strlen($sPath)));
                    }
                    else {
                        $zip->addFile($filepath, substr($filepath, strlen($sPath)));
                    }
                }
                $zip->close();

                # Offer the ZIP file as download
                header('Cache-control: private');
                header('Content-Type: application/octet-stream');
                header('Content-Length: ' . filesize($sPath . $sName . '.zip'));
                header('Content-Disposition: filename="' . $sName . '.zip"');
                print(file_get_contents($sPath . $sName . '.zip'));

                # Delete all files we just created
                unlink($sPath . $sName . '.xml');
                unlink($sPath . $sName . '.zip');
                unlink($sPath . 'module.xml');
                foreach ($aLangFiles as $aLangFile) {
                    unlink($sPath . $aLangFile['filename']);
                }

                die();
            }
        }
        elseif ($mode == i18n("Import from ZIP file")) {
            if (file_exists($_FILES["file"]["tmp_name"])) {

                # Save the zip file into the module's directory
                if (move_uploaded_file($_FILES['file']['tmp_name'], $sPath . $_FILES['file']['name'])) {

                    # Delete previous files for the old module
                    removeDir($sPath . 'css/', true);
                    removeDir($sPath . 'images/', true);
                    removeDir($sPath . 'js/', true);
                    removeDir($sPath . 'php/', true);
                    removeDir($sPath . 'templates/', true);

                    # Extract the zip file
                    $zip = new ZipArchive();
                    if ($zip->open($sPath . $_FILES['file']['name']) === true) {
                        $zip->extractTo($sPath);
                        $zip->close();
                    }

                    # Read the module.xml file to check for problems (e.g a module with the new module's name already exists)
                    $xml = simplexml_load_file($sPath . 'module.xml');
                    $sNewName = $xml->general->name;
                    $iInstance = 0;
                    $oMC = new cApiModuleCollection();
                    $oMC->setWhere('idclient', $client);
                    $oMC->setWhere('name', $sNewName);
                    $oMC->query();
                    while ($oMC->count()) {
                        $iInstance ++;
                        $oMC = new cApiModuleCollection();
                        $oMC->setWhere('idclient', $client);
                        $oMC->setWhere('name', $sNewName . ' (' . $iInstance . ')');
                        $oMC->query();
                    }
                    if ($iInstance) {
                        $sNewName .= ' (' . $iInstance . ')';
                    }

                    # If we have an instance number, we must rename the module to it BEFORE importing
                    $cnt = file_get_contents($sPath . $xml->files->file[0]);
                    $cnt = str_replace('<name>' . $xml->general->name . '</name>', '<name>' . $sNewName . '</name>', $cnt);
                    file_put_contents($sPath . $xml->files->file[0], $cnt);

                    # Now that we have a name for the new module, let's install it and eventually rename it and it's the PHP files
                    if (!$oModule->import($sPath . $xml->files->file[0])) {
                        $noti .= sprintf(i18n("Error while importing XML file: %s"), $oModule->_error). "<br>";    
                    } else {

                        # Rename the module's folder and php files
                        rename($sPath, str_replace($sName, uplCreateFriendlyName($sNewName), $sPath));
                        $sPath = str_replace($sName, uplCreateFriendlyName($sNewName), $sPath);
                        rename($sPath . 'php/' . uplCreateFriendlyName($xml->general->name) . '-input.php', $sPath . 'php/' . uplCreateFriendlyName($sNewName) . '-input.php');
                        rename($sPath . 'php/' . uplCreateFriendlyName($xml->general->name) . '-output.php', $sPath . 'php/' . uplCreateFriendlyName($sNewName) . '-output.php');

                        # Load the item again (clearing slashes from import)
                        $oModule->loadByPrimaryKey($oModule->get($oModule->primaryKey));

                        # Install module translations
                        if (count($xml->files->file) > 1) {
                            $aLangs = array();
                            $oCLC = new cApiClientLanguageCollection();
                            $oCLC->setWhere('idclient', $client);
                            $oCLC->query();

                            while ($oItem = $oCLC->next()) {
                                $oLang = new cApiLanguage($oItem->get("idlang"));
                                $aLangs[$oItem->get("idlang")] = array('name' => $oLang->get('name'), 'code' => $oLang->getProperty('language', 'code') . '_' . strtoupper($oLang->getProperty('country', 'code')));
                            }
                            unset($oCLC, $oItem, $oLang);

                            $aInstallLangs = array();
                            foreach ($xml->files->file as $file) {
                                if (isset($file['lang'])) {
                                    $aInstallLangs[(string) $file['lang']] = (string) $file;
                                }
                            }
                            $oTranslations = new cApiModuleTranslationCollection();
                            foreach ($aInstallLangs as $sLangCode => $sLangFile) {

                                # Install the translations only if this language is present
                                foreach ($aLangs as $idlang => $aLang) {
                                    if ($aLang['code'] == $sLangCode) {
                                        $oTranslations->import($oModule->get('idmod'), $idlang, $sPath . $sLangFile);
                                    }
                                }
                            }
                            unset($oTranslations, $aInstallLangs, $aLangs);
                        }

                        # Delete installation files
                        foreach ($xml->files->file as $file) {
                            unlink($sPath . $file);
                        }
                        unlink($sPath . 'module.xml');
                        unlink($sPath . $_FILES['file']['name']);

                        # Reload the left frame and select the module there (reloading the right top and bottom frames)
                        $page->addScript('jQuery', '<script src="scripts/general.js"></script>');
                        $page->addScript('', '<script>
                                //var target = parent.parent.left.left_bottom;
                                //target.location.href = target.location.href + "&idmod=' . $oModule->get('idmod') . '";
                                conMultiLink("left_bottom", "main.php?area=mod&frame=2&idmod=' . $oModule->get('idmod') . '&contenido=' . $contenido . '", "right_top", "main.php?area=mod&frame=3&action=&idmod=' . $oModule->get('idmod') . '&contenido=' . $contenido . '", "right_bottom", "main.php?area=mod_edit&frame=4&action=&idmod=' . $oModule->get('idmod') . '&contenido=' . $contenido . '");
                            </script>');
                    }
                }
                else {
                    $noti .= i18n('Error while importing ZIP file');
                }
            }
        }
        elseif ($mode == i18n("Import from XML file")) {
            if (file_exists($_FILES["upload"]["tmp_name"])) {            
                if (!$oModule->import($_FILES["upload"]["tmp_name"])) {
                    $noti .= sprintf(i18n("Error while importing XML file: %s"), $oModule->_error). "<br>";    
                } else {
                    // Load the item again (clearing slashes from import)
                    $oModule->loadByPrimaryKey($oModule->get($oModule->primaryKey));
                }
            }
        }
    }

    $oForm = new UI_Table_Form('choose_file');
    $oForm->addHeader(i18n("Import/Export"));
    $oForm->setWidth("300");
    $oForm->setVar("area", $area);
    $oForm->setVar("action", 'mod_importexport_module');
    $oForm->setVar('idmod', $idmod);
    $oForm->setVar("frame", $frame);
    $oForm->setVar("status", 'send');

    $oMode1 = new cHTMLRadiobutton('mode', i18n("Export to ZIP file"), '', ((strlen($oModule->get('input'))) || (strlen($oModule->get('output')))), (!(strlen($oModule->get('input') . $oModule->get('output')))));
    $oMode2 = new cHTMLRadiobutton('mode', i18n("Import from ZIP file"), '', (!(strlen($oModule->get('input') . $oModule->get('output')))));
    $oMode3 = new cHTMLRadiobutton('mode', i18n("Import from XML file"), '');
    $oForm->add(i18n("Mode"), $oMode1->render() . '<br>' . $oMode2->render() . '<br>' . $oMode3->render());
    $sModeScript = '<script>
        $(document).ready(function() {
            $("[name=\'mode\']").click(function() {
                $(this).parents("table").find("tr#file").css("display", (($(this).val() == "' . i18n("Export to ZIP file") . '") ? "none" : "table-row"));
            });
            $("[name=\'mode\']:checked").trigger("click");
        });
    </script>';
    $page->addScript('mode', $sModeScript);

    $oForm->add(i18n("File"), '<input type="file" name="file">', 'file', 'display: none;');

    $page->setContent($oForm->render());

    $page->render();
}
?>