<?php
/**
 * Project: 
 * DZIRCMS Content Management System
 * 
 * Description: 
 * Edit modules
 * 
 * Requirements: 
 * @con_php_req 5.0
 * 
 *
 * @package    DZIRCMS Backend includes
 * @version    2.1.0
 * @author     René Mansveld
 * @copyright  Spider IT Deutschland
 * @license    https://www.dzir.org/cms/license/LIZENZ.txt
 * @link       https://www.spider-it.de/
 * @link       https://www.dzir.org/cms/
 * @since      file available since DZIRCMS release 2.1.1
 * 
 * {@internal 
 *   created 2021-12-16
 * }}
 * 
 */

if (!defined('CON_FRAMEWORK')) {
	die('Illegal call');
}

if (!isset($_GET['mode'])) {
    $_GET['mode'] = 'empties';
}

if (($action == "mod_translations") && (!$perm->have_perm_area_action_anyitem($area, $action))) {
    $notification->displayNotification("error", i18n("No permission"));
}
else {
    $iSavedMod = 0;
    $sSavedString = '';

    # Was one of the forms sent?
    if (isset($_REQUEST['operation'])) {
        switch ($_REQUEST['operation']) {
            case 'search':
                $_GET['mode'] = 'all';
                break;
            case 'edit':
                $iMod = (int) $_POST['editMod'];
                $sString = urldecode($_POST['editString']);
                
                # Get the languages for the client
                $aLangIDs = array();
                $sql = 'SELECT l.`idlang`
                        FROM `' . $cfg['tab']['lang'] . '` l INNER JOIN `' . $cfg['tab']['clients_lang'] . '` cl ON l.`idlang` = cl.`idlang`
                        WHERE (cl.`idclient`=' . $client . ')
                        ORDER BY l.`name` ASC';
                $db->query($sql);
                while ($db->next_record()) {
                    $aLangIDs[] = $db->f('idlang');
                }
                
                # Insert the translations for the destination module into the destination string
                $oTranslations = new cApiModuleTranslationCollection();
                foreach ($aLangIDs as $iLang) {
                    $oTranslations->select('((idmod=' . $iMod . ') AND (idlang=' . $iLang . '))');
                    $bOK = false;
                    while ($oTrans = $oTranslations->next()) {
                        if ($oTrans->get('original') == $sString) {
                            #echo '<pre>Save translation for language ' . $iLang . ': ' . trim($_POST['translations'][$iLang]) . '</pre>';
                            $oTrans->set('translation', stripslashes(trim($_POST['translations'][$iLang])));
                            $oTrans->store();
                            $bOK = true;
                            break;
                        }
                    }
                    if (!$bOK) {
                        #echo '<pre>Create translation for language ' . $iLang . ': ' . trim($_POST['translations'][$iLang]) . '</pre>';
                        $oTranslations->create($iMod, $iLang, $sString, trim($_POST['translations'][$iLang]));
                    }
                }
                
                if ($_GET['mode'] == 'all') {
                    $iSavedMod = $iMod;
                    $sSavedString = $sString;
                }
                break;
            case 'copy':
                $iFromMod = (int) $_POST['copyMod'];
                $sFromString = urldecode($_POST['copyString']);
                $iToMod = (int) $_POST['targetMod'];
                $sToString = urldecode($_POST['targetString']);
                
                # Get the languages for the client
                $aLangIDs = array();
                $sql = 'SELECT l.`idlang`
                        FROM `' . $cfg['tab']['lang'] . '` l INNER JOIN `' . $cfg['tab']['clients_lang'] . '` cl ON l.`idlang` = cl.`idlang`
                        WHERE (cl.`idclient`=' . $client . ')
                        ORDER BY l.`name` ASC';
                $db->query($sql);
                while ($db->next_record()) {
                    $aLangIDs[] = $db->f('idlang');
                }
                
                # Get the translations from the source module and source string
                $aTranslations = array();
                $oTranslations = new cApiModuleTranslationCollection();
                foreach ($aLangIDs as $iLang) {
                    $oTranslations->select('((idmod=' . $iFromMod . ') AND (idlang=' . $iLang . '))');
                    while ($oTrans = $oTranslations->next()) {
                        if ($oTrans->get('original') == $sFromString) {
                            $aTranslations[$iLang] = $oTrans->get('translation');
                            break;
                        }
                    }
                }
                
                # Insert the translations for the destination module into the destination string
                foreach ($aLangIDs as $iLang) {
                    $oTranslations->select('((idmod=' . $iToMod . ') AND (idlang=' . $iLang . '))');
                    while ($oTrans = $oTranslations->next()) {
                        if ($oTrans->get('original') == $sToString) {
                            $oTrans->set('translation', $aTranslations[$iLang]);
                            $oTrans->store();
                            break;
                        }
                    }
                }
                
                $iSavedMod = $iToMod;
                $sSavedString = $sToString;
                break;
            case 'hide':
                $sql = 'UPDATE ' . $cfg['tab']['mod'] . '
                        SET `hide_empty_translations` = 1
                        WHERE (`idmod`=' . (int) $_GET['idmod'] . ')';
                $db->query($sql);
                break;
            case 'unhide':
                $sql = 'UPDATE ' . $cfg['tab']['mod'] . '
                        SET `hide_empty_translations` = 0
                        WHERE (`idmod`=' . (int) $_GET['idmod'] . ')';
                $db->query($sql);
                break;
        }
    }

    $smarty = new Smarty();
    
    $smarty->assign('title', 'Translations');
    $smarty->assign('form_action_clean', 'main.php?area=translations&mode=' . $_GET['mode'] . '&frame=4&contenido=' . $contenido);
    if ((isset($_REQUEST['searchFor'])) && (strlen($_REQUEST['searchFor'])) && (is_array($_REQUEST['searchIn'])) && (count($_REQUEST['searchIn']))) {
        $sSearchIn = '';
        foreach ($_REQUEST['searchIn'] as $iLang) {
            $sSearchIn .= '&searchIn[]=' . $iLang;
        }
        $smarty->assign('form_action', 'main.php?area=translations&mode=' . $_GET['mode'] . '&searchFor=' . urlencode($_REQUEST['searchFor']) . $sSearchIn . '&frame=4&contenido=' . $contenido);
    }
    else {
        $smarty->assign('form_action', 'main.php?area=translations&mode=' . $_GET['mode'] . '&frame=4&contenido=' . $contenido);
    }
    
    # List the languages for the client
    $aLanguages = array();
    $aLangIDs = array();
    $aLangNames = array();
    $sql = 'SELECT l.`idlang`, l.`name`
            FROM `' . $cfg['tab']['lang'] . '` l INNER JOIN `' . $cfg['tab']['clients_lang'] . '` cl ON l.`idlang` = cl.`idlang`
            WHERE (cl.`idclient`=' . $client . ')
            ORDER BY l.`name` ASC';
    $db->query($sql);
    while ($db->next_record()) {
        $aLanguages[] = array('id' => $db->f('idlang'), 'name' => $db->f('name'), 'checked' => (((is_array($_REQUEST['searchIn'])) && (in_array($db->f('idlang'), $_REQUEST['searchIn']))) ? 'checked' : ''));
        $aLangIDs[] = $db->f('idlang');
        $aLangNames[] = $db->f('name');
    }
    $smarty->assign('languages', $aLanguages);
    $smarty->assign('langids', $aLangIDs);
    $smarty->assign('langs', $aLangNames);
    
    $smarty->assign('showSearch', (($_GET['mode'] == 'empties') ? 0 : 1));
    $smarty->assign('textSearch', i18n("Search"));
    $smarty->assign('textSearchFor', i18n("Search for"));
    $smarty->assign('textSearchIn', i18n("Search in"));
    $smarty->assign('original', i18n("Original"));
    $smarty->assign('textSave', i18n("Go"));
    if (isset($_POST['scrollTop'])) {
        $smarty->assign('scrollTop', $_POST['scrollTop']);
    }
    $smarty->assign('colspan', (count($aLangIDs) + 1));
    
    $smarty->assign('searchIn0checked', (((is_array($_REQUEST['searchIn'])) && (in_array(0, $_REQUEST['searchIn']))) ? 'checked' : ''));
    if (is_array($_REQUEST['searchIn'])) {
        $smarty->assign('searchFor', $_REQUEST['searchFor']);
    }
    
    # List the modules for the client
    $aModules = array();
    $sql = 'SELECT `idmod`, `name`, `hide_empty_translations`
            FROM `' . $cfg['tab']['mod'] . '`
            WHERE ((`idclient`=' . $client . ')' . (($_GET['mode'] == 'empties') ? '
               AND (`hide_empty_translations`=0)' : '') . ')
            ORDER BY `name` ASC';
    $db->query($sql);
    while ($db->next_record()) {
        $aModules[] = array(
            'idmod' => $db->f('idmod'),
            'name' => $db->f('name'),
            'hide' => $db->f('hide_empty_translations'),
            'strings' => array()
        );
    }
    $sActions = '<a class="edit" href="#" data-action="edit" data-idmod="{IDMOD}" data-string="{STRING}"><img src="images/but_edittext.gif" alt="' . i18n("Edit") . '" title="' . i18n("Edit") . '"></a><a class="save" href="#" data-action="save" data-idmod="{IDMOD}" data-string="{STRING}"><img src="images/but_speichern.gif" alt="' . i18n("Save") . '" title="' . i18n("Save") . '"></a><a class="copy" href="#" data-action="copy" data-idmod="{IDMOD}" data-string="{STRING}"><img src="images/but_copy.gif" alt="' . i18n("Copy") . '" title="' . i18n("Copy") . '"></a><a class="paste" href="#" data-action="paste" data-idmod="{IDMOD}" data-string="{STRING}"><img src="images/importieren.gif" alt="' . i18n("Paste") . '" title="' . i18n("Paste") . '"></a><a class="cancel" href="#"><img src="images/but_cancel.gif" alt="' . i18n("Cancel") . '" title="' . i18n("Cancel") . '"></a>';
    for ($i = (count($aModules) - 1); $i >= 0; $i --) {
        $oTranslations = new cApiModuleTranslationCollection();
        
        # Find new strings in this module
        $oMod = new cApiModule($aModules[$i]['idmod']);
        $aStrings = $oMod->parseModuleForStrings();
        
        # Insert new strings into the DB
        foreach ($aLangIDs as $iLang) {
            foreach ($aStrings as $sString) {
                $oTranslations->create($aModules[$i]['idmod'], $iLang, $sString);
            }
        }
        unset($oMod);
        
        # Delete all old (not used anymore) translation entries in the DB
        foreach ($aLangIDs as $iLang) {
            $oTranslations->select('((idmod=' . $aModules[$i]['idmod'] . ') AND (idlang=' . $iLang . '))');
            while ($d_modtrans = $oTranslations->next()) {
                if (!in_array($d_modtrans->get("original"), $aStrings)) {
                    $oTranslations->delete($d_modtrans->get("idmodtranslation"));
                }
            }
        }
        
        # Get all strings for this module with it's translations
        foreach ($aLangIDs as $iLang) {
            $oTranslations->select('((idmod=' . $aModules[$i]['idmod'] . ') AND (idlang=' . $iLang . '))');
            while ($oTrans = $oTranslations->next()) {
                $sOriginal = $oTrans->get('original');
                $aModules[$i]['strings'][$sOriginal]['buttons'] = str_replace(array('{IDMOD}', '{STRING}'), array($aModules[$i]['idmod'], urlencode($sOriginal)), $sActions);
                $aModules[$i]['strings'][$sOriginal]['orig'] = htmlentities($sOriginal);
                $aModules[$i]['strings'][$sOriginal][$iLang] = htmlentities($oTrans->get('translation'));
                $aModules[$i]['strings'][$sOriginal]['saved'] = ((($iSavedMod == $aModules[$i]['idmod']) && ($sSavedString == $sOriginal)) ? ' class="saved"' : '');
            }
        }
        if ($_GET['mode'] == 'empties') {
            
            # Delete all string rows where all translations are present
            foreach ($aModules[$i]['strings'] as $sOrig => $aStrings) {
                foreach ($aLangIDs as $iLang) {
                    $bOK = true;
                    if (strlen(trim($aStrings[$iLang])) == 0) {
                        $bOK = false;
                        break;
                    }
                }
                if ($bOK) {
                    unset($aModules[$i]['strings'][$sOrig]);
                }
            }
        }
        elseif ((isset($_REQUEST['searchFor'])) && (strlen($_REQUEST['searchFor'])) && (is_array($_REQUEST['searchIn'])) && (count($_REQUEST['searchIn']))) {
            
            # Delete all string rows where the searched string is not present for the selected languages
            foreach ($aModules[$i]['strings'] as $sOrig => $aStrings) {
                $bOK = false;
                if ((in_array('0', $_REQUEST['searchIn'])) && (strpos(strtolower($aModules[$i]['strings'][$sOrig]['orig']), strtolower($_REQUEST['searchFor'])) !== false)) {
                    $bOK = true;
                }
                foreach ($aLangIDs as $iLang) {
                    if ((in_array($iLang, $_REQUEST['searchIn'])) && (strpos(strtolower($aModules[$i]['strings'][$sOrig][$iLang]), strtolower($_REQUEST['searchFor'])) !== false)) {
                        $bOK = true;
                    }
                }
                if (!$bOK) {
                    unset($aModules[$i]['strings'][$sOrig]);
                }
            }
        }

        # Set the show/hide switch where applicable
        if ($_GET['mode'] == 'empties') {
            $aModules[$i]['switchShowHide'] = '<a href="main.php?area=translations&mode=' . $_GET['mode'] . '&frame=4&operation=hide&idmod=' . $aModules[$i]['idmod'] . '&contenido=' . $contenido . '">' . i18n("Hide") . '</a>';
        }
        elseif ($aModules[$i]['hide']) {
            $aModules[$i]['switchShowHide'] = '<a href="main.php?area=translations&mode=' . $_GET['mode'] . '&frame=4&operation=unhide&idmod=' . $aModules[$i]['idmod'] . '&contenido=' . $contenido . '">' . i18n("Unhide") . '</a>';
        }
        
        # Delete all modules where there are no translations (left)
        if (count($aModules[$i]['strings']) == 0) {
            unset($aModules[$i]);
        }
    }
    $smarty->assign('all', $aModules);
    
    $smarty->display('templates/standard/template.mod_translations.html');
}
?>