<?php
/**
 * Project:
 * DZIRCMS Content Management System
 *
 * Description:
 * Design Shop
 *
 * Requirements:
 * @con_php_req 7.0
 *
 *
 * @package    DZIRCMS Frontend Designs
 * @version    1.0
 * @author     René Mansveld
 * @copyright  Spider IT Deutschland
 * @license    https://www.dzir.org/cms/license/LICENCE.txt
 * @link       https://www.dzir.org/cms/
 * @link       https://www.spider-it.de/
 * @since      file available since DZIRCMS release 2.2.0
 *
 * {@internal
 *   created  2022-01-15
 * }}
 *
 */

if (!defined('CON_FRAMEWORK')) {
    die('Illegal call');
}

function copyr($src, $dst) {
    $src .= ((substr($src, -1) == '/') ? '' : '/');
    $dst .= ((substr($dst, -1) == '/') ? '' : '/');
    
    $bOK = true;
    $dir = opendir($src);
    @mkdir($dst);
    while (($file = readdir($dir)) !== false) {
        if (($file != '.') && ($file != '..')) {
            if (is_dir($src . '/' . $file)) {
                if (!copyr($src . $file, $dst . $file)) {
                    $bOK = false;
                }
            }
            else {
                if (!copy($src . $file, $dst . $file)) {
                    $bOK = false;
                }
            }
        }
    }
    closedir($dir);
    return $bOK;
}

$plugin_name = "ds";

$iLWidth = 1920;
$iLHeight = 1080;
$iPWidth = 360;
$iPHeight = 680;
$bShowImagesWhenDone = false;
$sCurrentDemo = 'Demo_2.2.0';

# Make the pages show faster, even with slow backend internet connections
@ini_set('zlib.output_compression', 0);
@ini_set('implicit_flush', 1);
ob_implicit_flush(1);
ob_end_flush();

if (isset($_POST['registerCode'])) {
    
    # Send the code to the repository for registration
    $errno = 0;
    $errmsg = '';
    $ret = getRemoteContent($_POST['url'] . '?Design=' . $_POST['design'] . '&Action=RegisterCode&Code=' . $_POST['registerCode'], $errno, $errmsg, getProperty(0, 'design_repository', $_POST['repository'], 'server', 'login'), getProperty(0, 'design_repository', $_POST['repository'], 'server', 'password'));
    if (substr($ret, 0, 3) == '+OK') {
        
        # Enter the code into the database for later use (no client, this will be changed while installing the design)
        $i = 0;
        $aProps = getPropertiesByItemtype('designs', 'itemid');
        if (count($aProps)) {
            $i = ($aProps[(count($aProps) - 1)]['itemid'] + 1);
        }
        setProperty(0, 'designs', $i, 'design', 'key', Contenido_Security::escapeDB($_POST['design'], $db));
        setProperty(0, 'designs', $i, 'design', 'url', Contenido_Security::escapeDB($_POST['url'], $db));
        setProperty(0, 'designs', $i, 'design', 'code', Contenido_Security::escapeDB($_POST['registerCode'], $db));
    }
    die();
}

$smarty = new Smarty();
$smarty->assign('contenido', $contenido);
switch ($_GET['operation']) {
    case 'clients':
        if (isset($_GET['client'])) {
            $iClient = (int) $_GET['client'];
            switch ($_POST['execute']) {
                case 'install':
                    ob_end_flush();
                    echo '<!DOCTYPE html>
                    <html lang="de">
                        <head>
                            <title>Design Shop</title>
                            <meta charset="utf-8">
                            <meta http-equiv="expires" content="0">
                            <meta http-equiv="cache-control" content="no-cache">
                            <meta http-equiv="pragma" content="no-cache">
                            <link rel="stylesheet" type="text/css" href="styles/admin.css" />
                            <link rel="stylesheet" type="text/css" href="plugins/ds/templates/default/css/ds.css">
                            <script src="scripts/jquery/jquery-3.6.0.min.js"></script>
                            <script type="text/javascript">
                                setTimeout(function() {
                                    $("html, body").animate({scrollTop: $(document).height() + "px"}, 2000);
                                }, 100);
                            </script>
                        </head>
                        <body>';
                    flush();
                    $sReloadScript = '
                        <p>' . i18n("(auto redirect in 5 seconds)") . '</p>
                        <script>';
                    if (!$client) {
                        $sReloadScript .= '
                            var tmr;
                            tmr = setInterval(function() {
                                checkLoadInit();
                            }, 1);
                            function checkLoadInit() {
                                var nav = top.header.document.getElementById("main_6");
                                if (nav == null) {
                                    clearInterval(tmr);
                                    tmr = setInterval(function() {
                                        checkLoadReady();
                                    }, 10);
                                }
                            }
                            function checkLoadReady() {
                                var nav = top.header.document.getElementById("main_6");
                                if (nav != null) {
                                    clearInterval(tmr);
                                    $("#sub_ds", top.header.document).removeClass("sub").addClass("activemenu");
                                    top.header.window.show("sub_6", "main_6");
                                    setTimeout(function() {
                                        var menu = top.header.window.ContenidoRegistry.get("headerMenu");
                                        menu.setActiveSubMenu("sub_6");
                                        menu.setActiveMenu("main_6");
                                    }, 10);
                                }
                            }
                            top.header.document.location.href = "' . $cfg['path']['contenido_fullhtml'] . 'header.php?changeclient=&changelang=&contenido=' . $contenido . '";';
                    }
                    $sReloadScript .= '
                            setTimeout(function() {
                                parent.parent.left.left_bottom.location.href = parent.parent.left.left_bottom.location.href + \'&client=|1\';
                            }, 5000);
                        </script>';
                    
                    $sDesign = Contenido_Security::escapeDB($_POST['design'], $db);
                    $sUrl = Contenido_Security::escapeDB($_POST['url'], $db);
                    $sCode = Contenido_Security::escapeDB($_POST['code'], $db);
                    $sRepository = $_POST['repository'];
                    $bNew = ($iClient == 0);
                    
                    # Get the domain for this installation
                    if ($bNew) {
                        $sName = $_POST['clientname'];
                        $sFrontendPath = $_POST['frontendpath'] . ((substr($_POST['frontendpath'], -1) == '/') ? '' : '/');
                        $sHtmlPath = $_POST['htmlpath'] . ((substr($_POST['htmlpath'], -1) == '/') ? '' : '/');
                        $sDomain = $sHtmlPath;
                    }
                    else {
                        $sDomain = $cfgClient[$iClient]['path']['htmlpath'];
                    }
                    $sDomain = str_replace(array('https://', 'http://', 'www.'), '', $sDomain);
                    $p = strpos($sDomain, '/');
                    if ($p !== false) {
                        $sDomain = substr($sDomain, 0, $p);
                    }
                    
                    # Check (if a code was submitted) if the code is valid
                    $errno = 0;
                    $errmsg = '';
                    $ret = getRemoteContent($sUrl . '?Design=' . urlencode($sDesign) . '&Action=CheckCode&Code=' . urlencode($sCode) . '&Domain=' . urlencode($sDomain), $errno, $errmsg, getProperty(0, 'design_repository', $sRepository, 'server', 'login'), getProperty(0, 'design_repository', $sRepository, 'server', 'password'));
                    if (substr($ret, 0, 3) == '+OK') {
                        plugin_include($plugin_name, 'includes/include.functions.php');
                        
                        # Do we install a new client? If so, create it first.
                        if ($bNew) {
                            $iClient = $db->nextid($cfg['tab']['clients']);
                            $sql = 'INSERT INTO ' . $cfg['tab']['clients'] . '
                                    SET name = "' . Contenido_Security::escapeDB($sName, $db) . '",
                                        frontendpath = "' . Contenido_Security::escapeDB($sFrontendPath, $db) . '",
                                        htmlpath = "' . Contenido_Security::escapeDB($sHtmlPath, $db) . '",
                                        idclient = ' . intval($iClient);
                            $db->query($sql);
                            rereadClients();
                            
                            # Copy the client template to the new location
                            $destPath = $sFrontendPath;
                            $sourcePath = $cfg['path']['contenido'] . $cfg['path']['frontendtemplate'];
                            if (!file_exists($destPath)) {
                                recursive_copy($sourcePath, $destPath);
                                $res = fopen($destPath . 'config.php', 'rb+');
                                $res2 = fopen($destPath . 'config.php.new', 'ab+');
                                if (($res) && ($res2)) {
                                    while (!feof($res)) {
                                        $buffer = fgets($res, 4096);
                                        $buffer = str_replace(array('!CLIENT!','!PATH!' ), array($iClient, $cfg['path']['contenido']), $buffer);
                                        fwrite($res2, $buffer);
                                    }
                                } else {
                                    $note = $notification->returnNotification("error",i18n("Couldn't write the file config.php.")) . '<br />';
                                }
                                fclose($res);
                                fclose($res2);
                                unlink($destPath . 'config.php');
                                rename($destPath . 'config.php.new', $destPath . 'config.php');
                                echo '<p>' . i18n("Client created") . '</p>';
                                flush();
                            } else {
                                $message = sprintf(i18n("The directory %s already exists. The client was created, but you have to copy the frontend-template yourself"),$destPath);
                                $note = $notification->returnNotification("warning", $message) . '<br />';
                            }
                        }
                        
                        # First stick the design to the client if a code was specified
                        if (strlen($sCode)) {
                            
                            # Find the code in the database (registered before through here)
                            $sql = 'SELECT `itemid`
                                    FROM `' . $cfg['tab']['properties'] . '`
                                    WHERE (((`idclient`=0) OR (`idclient`=' . $iClient . '))
                                       AND (`itemtype`="designs")
                                       AND (`type`="design")
                                       AND (`name`="code")
                                       AND (`value`="' . $sCode . '"))';
                            $db->query($sql);
                            if ($db->next_record()) {
                                
                                # Update it to bind it to the client (otherwise you can't buy it for another client)
                                $sql = 'UPDATE `' . $cfg['tab']['properties'] . '`
                                        SET `idclient` = ' . $iClient . '
                                        WHERE ((`itemtype`="designs")
                                           AND (`itemid`="' . $db->f('itemid') . '"))';
                                $db->query($sql);
                            }
                            else {
                                
                                # Bought in the online shop, register it for the client
                                $i = 0;
                                $aProps = getPropertiesByItemtype('designs', 'itemid');
                                if (count($aProps)) {
                                    $i = ($aProps[(count($aProps) - 1)]['itemid'] + 1);
                                }
                                setProperty($iClient, 'designs', $i, 'design', 'key', $sDesign);
                                setProperty($iClient, 'designs', $i, 'design', 'url', $sUrl);
                                setProperty($iClient, 'designs', $i, 'design', 'code', $sCode);
                            }
                        }
                        
                        # For an existing client, delete the entire data and then reset the sequence table
                        if (!$bNew) {
                            
                            # Delete the client data from the database, except for the client table record
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['lang'],
                                array(
                                    'idlang' => 'SELECT `idlang` FROM `' . $cfg['tab']['clients_lang'] . '` WHERE (`idclient`=' . $iClient . ')'
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['clients_lang'],
                                array(
                                    'idclient' => $iClient
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['cat_art'],
                                array(
                                    'idcat' => 'SELECT `idcat` FROM `' . $cfg['tab']['cat'] . '` WHERE (`idclient`=' . $iClient . ')'
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['cat_lang'],
                                array(
                                    'idcat' => 'SELECT `idcat` FROM `' . $cfg['tab']['cat'] . '` WHERE (`idclient`=' . $iClient . ')'
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['cat_tree'],
                                array(
                                    'idcat' => 'SELECT `idcat` FROM `' . $cfg['tab']['cat'] . '` WHERE (`idclient`=' . $iClient . ')'
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['cat'],
                                array(
                                    'idclient' => $iClient
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['content'],
                                array(
                                    'idartlang' => 'SELECT al.`idartlang` FROM `' . $cfg['tab']['art_lang'] . '` al INNER JOIN `' . $cfg['tab']['art'] . '` a ON al.`idart` = a.`idart` WHERE (a.`idclient`=' . $iClient . ')'
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['meta_tag'],
                                array(
                                    'idartlang' => 'SELECT al.`idartlang` FROM `' . $cfg['tab']['art_lang'] . '` al INNER JOIN `' . $cfg['tab']['art'] . '` a ON al.`idart` = a.`idart` WHERE (a.`idclient`=' . $iClient . ')'
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['art_lang'],
                                array(
                                    'idart' => 'SELECT `idart` FROM `' . $cfg['tab']['art'] . '` WHERE (`idclient`=' . $iClient . ')'
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['art'],
                                array(
                                    'idclient' => $iClient
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['art_spec'],
                                array(
                                    'client' => $iClient
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['lay'],
                                array(
                                    'idclient' => $iClient
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['mod_translations'],
                                array(
                                    'idmod' => 'SELECT `idmod` FROM `' . $cfg['tab']['mod'] . '` WHERE (`idclient`=' . $iClient . ')'
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['mod'],
                                array(
                                    'idclient' => $iClient
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['container_conf'],
                                array(
                                    'idtplcfg' => 'SELECT `tc`.`idtplcfg` FROM `' . $cfg['tab']['tpl_conf'] . '` tc INNER JOIN `' . $cfg['tab']['tpl'] . '` t ON tc.`idtpl` = t.`idtpl` WHERE (`t`.`idclient`=' . $iClient . ')'
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['container'],
                                array(
                                    'idtpl' => 'SELECT `idtpl` FROM `' . $cfg['tab']['tpl'] . '` WHERE (`idclient`=' . $iClient . ')'
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['tpl_conf'],
                                array(
                                    'idtpl' => 'SELECT `idtpl` FROM `' . $cfg['tab']['tpl'] . '` WHERE (`idclient`=' . $iClient . ')'
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['tpl'],
                                array(
                                    'idclient' => $iClient
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['properties'],
                                array(
                                    'idclient' => $iClient,
                                    'itemtype' => array('clientsetting', 'idclient', 'idlang', 'idlay')
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['upl_meta'],
                                array(
                                    'idupl' => 'SELECT `idupl` FROM `' . $cfg['tab']['upl'] . '` WHERE (`idclient`=' . $iClient . ')'
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['upl'],
                                array(
                                    'idclient' => $iClient
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['dbfs'],
                                array(
                                    'idclient' => $iClient
                                )
                            );
                            $ret = deleteTableData(
                                $db,
                                $cfg['tab']['file_information'],
                                array(
                                    'idclient' => $iClient
                                )
                            );
                            echo '<p>' . i18n("Client data removed from the database") . '</p>';
                            flush();
                            
                            # Delete additional folders and files for the client
                            removeDir($cfgClient[$iClient]['path']['frontend'] . 'css', true);
                            removeDir($cfgClient[$iClient]['path']['frontend'] . 'images', true);
                            removeDir($cfgClient[$iClient]['path']['frontend'] . 'includes', true);
                            removeDir($cfgClient[$iClient]['path']['frontend'] . 'js', true);
                            removeDir($cfgClient[$iClient]['path']['frontend'] . 'templates', true);
                            removeDir($cfgClient[$iClient]['path']['frontend'] . 'upload', true);
                            $aDirs = getSubdirs($cfgClient[$iClient]['path']['frontend']);
                            foreach ($aDirs as $sDirOther) {
                                $sDirOther = str_replace($cfgClient[$iClient]['path']['frontend'], '', $sDirOther);
                                if (!in_array(substr($sDirOther, 0, -1), array('cache', 'css', 'images', 'includes', 'js', 'logs', 'templates', 'upload', 'version'))) {
                                    removeDir($cfgClient[$iClient]['path']['frontend'] . $sDirOther, true);
                                }
                            }
                            $aFiles = getFilesInDirectory($cfgClient[$iClient]['path']['frontend']);
                            foreach ($aFiles as $sFile) {
                                if (!in_array($sFile, array('.htaccess', 'config.php', 'dbfs.php', 'error.php', 'front_content.php', 'front_crcloginform.inc.php', 'index.php', 'robots.txt'))) {
                                    unlink($cfgClient[$iClient]['path']['frontend'] . $sFile);
                                }
                            }
                            if (is_dir($cfgClient[$iClient]['path']['frontend'] . 'data/modules_c')) {
                                removeDir($cfgClient[$iClient]['path']['frontend'] . 'data/modules_c', true);
                            }
                            echo '<p>' . i18n("Client files removed") . '</p>';
                            flush();
                            
                            # Do the cleanup
                            $oPurge = new Purge($db, $cfg, $cfgClient);
                            $oPurge->resetClientConCode($iClient);
                            $oPurge->clearClientCache($iClient);
                            $oPurge->clearClientHistory($iClient, false, 0);
                            $oPurge->clearClientLog($iClient);
                            cInclude('includes', 'functions.database.php');
                            $oPurge->updateConSequence();
                            unset($oPurge);
                            echo '<p>' . i18n("Cleanup done") . '</p>';
                            flush();
                        }
                        
                        # Now install the client data
                        installClient($iClient, $sDomain, $sDesign, $sRepository, $sUrl, $sCode, $db);
                        echo $notification->returnNotification('info', sprintf(i18n("Design %s successfully installed"), $sDesign));
                        if (!$client) {
                            $client = $iClient;
                            $sql = 'SELECT l.idlang
                                    FROM ' . $cfg['tab']['lang'] . ' l INNER JOIN ' . $cfg['tab']['clients_lang'] . ' cl ON l.idlang = cl.idlang
                                    WHERE (idclient=' . intval($client) . ')
                                    ORDER BY l.idlang ASC';
                            $db->query($sql);
                            $db->next_record();
                            $lang = $db->f('idlang');
                            
                            if (!$perm->have_perm_client_lang($client, $lang)) {
                                $lang = '';
                                
                                while ($db->next_record() && ($lang == '')) {
                                    if ($perm->have_perm_client_lang($client, $db->f('idlang'))) {
                                        $lang = $db->f('idlang');
                                    }
                                }
                            }
                        }
                        echo str_replace('|1', $iClient, $sReloadScript);
                        flush();
                    }
                    break;
                default:
                    if (($perm->have_perm_area_action($plugin_name, $plugin_name)) && (($perm->have_perm_area_action($plugin_name, 'exchange')) || ($perm->have_perm_area_action($plugin_name, 'export')))) {
                        
                        # Action(s) allowed, show the first action
                        if (($perm->have_perm_area_action($plugin_name, 'exchange')) && ($_GET['what'] != 'export')) {
                            $sSearch = $_REQUEST['Search'];
                            $sFilter = $_REQUEST['Filter'];
                            
                            # Show the exchange page
                            $smarty->assign('textProperty', i18n("Property"));
                            $smarty->assign('textValue', i18n("Value"));
                            $smarty->assign('textClientName', i18n("Client name"));
                            $smarty->assign('textFrontendPath', i18n("Server path"));
                            $smarty->assign('textWebAddress', i18n("Web address"));
                            $smarty->assign('frontendPath', $cfg['path']['frontend']);
                            $smarty->assign('formAction', 'main.php?area=ds&frame=4&operation=clients&client=' . $iClient . '&what=exchange&contenido=' . $contenido);
                            $smarty->assign('textPreviewInfo', i18n("Screenshots were automatically generated and may look slightly different than the website pages"));
                            $smarty->assign('textSearch', i18n("Search"));
                            $smarty->assign('textTerms', i18n("Terms"));
                            $smarty->assign('textDoSearch', i18n("Go"));
                            $smarty->assign('textClearSearch', i18n("Clear search"));
                            $smarty->assign('textSearchDescription', i18n("(Search in Name and Description)"));
                            $smarty->assign('textFilter', i18n("Filter"));
                            $smarty->assign('textTags', i18n("Tags"));
                            $smarty->assign('textDesign', i18n("Design"));
                            $smarty->assign('textDescription', i18n("Description"));
                            $smarty->assign('textComment', i18n("Comment"));
                            $smarty->assign('textAuthor', i18n("Author"));
                            $smarty->assign('textCopyright', i18n("Copyright"));
                            $smarty->assign('textVersion', i18n("Version"));
                            $smarty->assign('textDate', i18n("Date"));
                            $smarty->assign('textPrice', i18n("Price"));
                            $smarty->assign('textPlugins', i18n("Used plugins"));
                            $smarty->assign('textRepository', i18n("Repository"));
                            $smarty->assign('textDoFilter', i18n("Use filter"));
                            $smarty->assign('textClearFilter', i18n("Clear filter"));
                            $smarty->assign('textFilterDescription', i18n("(Comma separated list of tags to search for)"));
                            $smarty->assign('infoBuy', sprintf(i18n("Paid designs can be purchased via the service provider %s (no account needed) by clicking on the golden coin symbol in the first column.<br>Purchased designs can only be used for one client and are linked to the client on first use (you can reinstall them later, but only for the same client).<br>Plugins used by the design can be installed after installing the design without problems."), '<a href="https://www.paypal.com/" target="_blank">PayPal</a>'));
                            $smarty->assign('textPaypalSuccess', i18n("Thank you for your order!<br><br>Your code for design |1 is |2.<br>It was saved for you for later use.<br><br>Have fun using DZIRCMS!"));
                            $smarty->assign('bought', i18n("Bought already"));
                            $smarty->assign('textClose', i18n("Close"));
                            $smarty->assign('installed', i18n("installed"));
                            $smarty->assign('notInstalled', i18n("currently not installed"));
                            $smarty->assign('search', $sSearch);
                            $smarty->assign('filter', $sFilter);
                            if ($iClient) {
                                $smarty->assign('warning', i18n("Caution! On installing a design, all data for the client will be replaced with the client data of the design package!"));
                            }
                            
                            # Get available designs from https://www.dzir.org/cms/designs/
                            $aParams = array();
                            if (strlen($sSearch)) {
                                $aParams[] = 'Search=' . urlencode($sSearch);
                            }
                            if (strlen($sFilter)) {
                                $aParams[] = 'Filter=' . urlencode($sFilter);
                            }
                            if ((strlen($sSearch)) || (strlen($sFilter))) {
                                $aParams[] = 'Language=' . urlencode($belang);
                            }
                            $sUrl = 'https://www.dzir.org/cms/designs/' . ((count($aParams)) ? '?' . implode('&', $aParams) : '');
                            $sDownloadUrl = 'https://www.dzir.org/cms/designs/';
                            $iErrNo = 0;
                            $sErrMsg = '';
                            $aDesigns = json_decode(getRemoteContent($sUrl, $iErrNo, $sErrMsg), true);
                            foreach ($aDesigns as $key => $value) {
                                $aDesigns[$key]['Url'] = $sDownloadUrl;
                                $aDesigns[$key]['Repo'] = 'DZIRCMS';
                                if ($key == $sCurrentDemo) {
                                    $aDemo = array();
                                    $aDemo[$key] = $aDesigns[$key];
                                    unset($aDesigns[$key]);
                                }
                            }
                            
                            # Get available designs from additionally added repositories
                            $bShowRepos = false;
                            $aReposInfo = getPropertiesByItemtype('design_repository', 'itemid');
                            $aRepos = array();
                            for ($i = 0, $n = count($aReposInfo); $i < $n; $i ++) {
                                $aRepos[$aReposInfo[$i]['itemid']][$aReposInfo[$i]['name']] = $aReposInfo[$i]['value'];
                            }
                            foreach ($aRepos as $itemid => $aServer) {
                                $bShowRepos = true;
                                $sUrl = $aServer['host'] . ((count($aParams)) ? '?' . implode('&', $aParams) : '');
                                $sDownloadUrl = $aServer['host'];
                                $sLogin = $aServer['login'];
                                $sPassword = $aServer['password'];
                                $iErrNo = 0;
                                $sErrMsg = '';
                                $aDesignsO = json_decode(getRemoteContent($sUrl, $iErrNo, $sErrMsg, $sLogin, $sPassword), true);
                                foreach ($aDesignsO as $key => $value) {
                                    $aDesigns[$key . '~~' . $itemid] = $value;
                                    $aDesigns[$key . '~~' . $itemid]['Url'] = $sDownloadUrl;
                                    $aDesigns[$key . '~~' . $itemid]['Repo'] = $itemid;
                                }
                            }
                            
                            # Sort the list
                            $aDesigns = array_csort($aDesigns, 'Date', SORT_STRING, SORT_DESC, 'name', SORT_STRING, SORT_ASC);
                            
                            # Move the demo client to the top of the list
                            if (isset($aDemo)) {
                                $aDesigns = array_merge($aDemo, $aDesigns);
                            }
                            
                            # List available designs
                            $aList = array();
                            foreach ($aDesigns as $key => $value) {
                                $bAdditional = (strpos($key, '~~') !== false);
                                $key = ((strpos($key, '~~') !== false) ? substr($key, 0, strpos($key, '~~')) : $key);
                                if (count($value['Screenshots'])) {
                                    $sImg = ((count($value['Screenshots']['Landscape'])) ? '<img class="landscape" src="' . $value['Url'] . '?Design=' . $key . '&Action=getImage&Image=' . $value['Screenshots']['Landscape'][0] . '" alt="' . i18n("Landscape") . '" data-count="' . count($value['Screenshots']['Landscape']) . '">' : '') . ((count($value['Screenshots']['Portrait'])) ? '<img class="portrait" src="' . $value['Url'] . '?Design=' . $key . '&Action=getImage&Image=' . $value['Screenshots']['Portrait'][0] . '" alt="' . i18n("Portrait") . '" data-count="' . count($value['Screenshots']['Portrait']) . '">' : '');
                                }
                                $sDescription = '';
                                if (strlen($value['Description'][$belang])) {
                                    $sDescription = $value['Description'][$belang];
                                }
                                else {
                                    foreach ($value['Description'] as $cult => $desc) {
                                        if (substr($sCult, 0, strpos($sCult, '_')) == $sLanguage) {
                                            $sDescription = $desc;
                                            break;
                                        }
                                    }
                                }
                                $sComment = '';
                                if (strlen($value['Comment'][$belang])) {
                                    $sComment = $value['Comment'][$belang];
                                }
                                else {
                                    foreach ($value['Comment'] as $cult => $desc) {
                                        if (substr($sCult, 0, strpos($sCult, '_')) == $sLanguage) {
                                            $sComment = $desc;
                                            break;
                                        }
                                    }
                                }
                                if ((version_compare($value['Requirements']['php'], phpversion(), '>')) && ((version_compare($value['Requirements']['dzircms'], $cfg['version'], '>')) || (version_compare($value['Requirements']['drugcms'], $cfg['version'], '>')))) {
                                    $sClass = 'old';
                                    $sInfo = sprintf(i18n("Old PHP version and old CMS version, installation not possible"), $value['Name']);
                                    $sImgClass = 'blocked';
                                    $sImage = 'icon_warning';
                                }
                                elseif (version_compare($value['Requirements']['php'], phpversion(), '>')) {
                                    $sClass = 'old';
                                    $sInfo = sprintf(i18n("Old PHP version, installation not possible"), $value['Name']);
                                    $sImgClass = 'blocked';
                                    $sImage = 'icon_warning';
                                }
                                elseif (version_compare($value['Requirements']['dzircms'], $cfg['version'], '>')) {
                                    $sClass = 'old';
                                    $sInfo = sprintf(i18n("Old CMS version, installation not possible"), $value['Name']);
                                    $sImgClass = 'blocked';
                                    $sImage = 'icon_warning';
                                }
                                else {
                                    $sClass = 'new';
                                    $sInfo = sprintf(i18n("Install %s"), $value['Name']);
                                    $sImgClass = 'install';
                                    $sImage = 'importieren';
                                }
                                $iDate = (int) date('Ymd', $value['Date']);
                                if (strlen($value['Date'])) {
                                    $sDate = formatDateLocale($value['Date'], $belang);
                                }
                                else {
                                    $sDate = '<span style="color: lightgray;">' . i18n("unknown") . '</span>';
                                }
                                $sBuy = $sCode = '';
                                $sPrice = (float) $value['Pricing']['Price'];
                                if ((float) $value['Pricing']['Price'] == 0) {
                                    $sPriceDisplay = i18n("free");
                                    $sClass = $sClass . ' free';
                                }
                                else {
                                    $sPriceDisplay = $value['Pricing']['Currency'] . ' ' . formatNumberLocale($sPrice, 2, $belang);
                                    if ($sClass == 'new') {
                                        $sBuy = sprintf(i18n("Buy %s"), mb_convert_encoding($value['Name'], $encoding[$lang], 'UTF-8'));
                                    }
                                    
                                    # Get all itemid's for this design
                                    $sql = 'SELECT `idclient`, `itemid`
                                            FROM `' . $cfg['tab']['properties'] . '`
                                            WHERE ((`itemtype`="designs")
                                               AND (`type`="design")
                                               AND (`name`="key")
                                               AND (`value`="' . $key . '"))
                                            ORDER BY `idclient` DESC';
                                    $db->query($sql);
                                    while ($db->next_record()) {
                                        if ((($db->f('idclient') > 0) && ($db->f('idclient') == $iClient)) || ($db->f('idclient') == 0)) {
                                            
                                            # Get the code for the design either for this client (preferred), or not used (idclient = 0)
                                            $sql = 'SELECT `value`
                                                    FROM `' . $cfg['tab']['properties'] . '`
                                                    WHERE ((`itemtype`="designs")
                                                       AND (`itemid`="' . $db->f('itemid') . '")
                                                       AND (`type`="design")
                                                       AND (`name`="code"))';
                                            $db->query($sql);
                                            if ($db->next_record()) {
                                                $sCode = $db->f('value');
                                                break;
                                            }
                                        }
                                    }
                                }
                                $aPlugins = array();
                                if (count($value['Plugins'])) {
                                    foreach ($value['Plugins'] as $pkey => $pvalue) {
                                        $aPlugins[] = array('name' => $pvalue, 'path' => $pkey, 'installed' => ((is_dir($cfg['path']['contenido'] . 'plugins/' . $pkey)) ? true : false));
                                    }
                                }
                                $aList[] = array(
                                    'key' => $key,
                                    'url' => $value['Url'],
                                    'firstImage' => $sImg,
                                    'name' => mb_convert_encoding($value['Name'], $encoding[$lang], 'UTF-8'),
                                    'description' => mb_convert_encoding(((strlen($sDescription)) ? $sDescription : $value['Description']['en_US']), $encoding[$lang], 'UTF-8'),
                                    'comment' => mb_convert_encoding(((strlen($sComment)) ? $sComment : $value['Comment']['en_US']), $encoding[$lang], 'UTF-8'),
                                    'tags' => mb_convert_encoding(implode(', ', $value['Tags'][$belang]), $encoding[$lang], 'UTF-8'),
                                    'author' => mb_convert_encoding($value['Author'], $encoding[$lang], 'UTF-8'),
                                    'copyright' => mb_convert_encoding($value['Copyright'], $encoding[$lang], 'UTF-8'),
                                    'version' => mb_convert_encoding($value['Version'], $encoding[$lang], 'UTF-8'),
                                    'iDate' => $iDate,
                                    'date' => $sDate,
                                    'price' => $sPrice,
                                    'priceDisplay' => $sPriceDisplay,
                                    'plugins' => $aPlugins,
                                    'class' => $sClass,
                                    'info' => $sInfo,
                                    'imgClass' => $sImgClass,
                                    'image' => $sImage,
                                    'buy' => $sBuy,
                                    'code' => $sCode,
                                    'amount' => $value['Pricing']['Price'],
                                    'currency' => $value['Pricing']['Currency'],
                                    'repository' => mb_convert_encoding($value['Repo'], $encoding[$lang], 'UTF-8'),
                                    'landscape' => $value['Screenshots']['Landscape'],
                                    'portrait' => $value['Screenshots']['Portrait']
                                );
                            }
                            
                            $smarty->assign('client', $iClient);
                            $smarty->assign('list', $aList);
                            $smarty->assign('showRepos', $bShowRepos);
                            $smarty->assign('paypalClientID', 'Aeh7YA1zJ51VPH8F8CHTX6hvoBxPCEST4IgcXgxiJw_DkmMKWdMucgGF1ZOD7-hPcaVKy9oj8lPXSh7g');
                            $smarty->display(realpath(dirname(__FILE__) . '/../templates/default/right-bottom-exchange.html'));
                        }
                        else {
                            session_start();
                            
                            $sClient = str_replace(' ', '_', capiStrReplaceDiacritics(getClientName($iClient)));
                            $sDir = $cfg['path']['contenido'] . 'data/temp/' . $sClient . '/';
                            $sDirData = $sDir . 'data/';
                            $sDirScreenshots = $sDir . 'screenshots/';
                            $sDirFiles = $sDir . 'files/';
                            $sDirFilesCss = $sDir . 'files/css/';
                            $sDirFilesImages = $sDir . 'files/images/';
                            $sDirFilesIncludes = $sDir . 'files/includes/';
                            $sDirFilesJs = $sDir . 'files/js/';
                            $sDirFilesTemplates = $sDir . 'files/templates/';
                            $sDirFilesUpload = $sDir . 'files/upload/';
                            
                            # Show the export page
                            $smarty->assign('contenido', $contenido);
                            $sDomain = str_replace(array('https://', 'http://'), '', $cfgClient[$iClient]['path']['htmlpath']);
                            $aDomainParts = explode('/', $sDomain);
                            $aDomain = explode('.', $aDomainParts[0]);
                            $sDomain = $aDomain[count($aDomain) - 2] . '.' . $aDomain[count($aDomain) - 1];
                            # Bugfix for local installations
                            if (substr($sDomain, 0, 1) == '.') {
                                $sDomain = substr($sDomain, 1);
                            }
                            $smarty->assign('documentDomain', $sDomain);
                            $smarty->assign('formAction', 'main.php?area=ds&frame=4&operation=clients&client=' . $iClient . '&what=export&contenido=' . $contenido);
                            $smarty->assign('waitInfo2', i18n("Converting images."));
                            
                            $aLandscape = array();
                            $aPortrait = array();
                            $aPages = array();
                            
                            # If the form has been sent, save the preview images and generate the client package
                            if ((isset($_REQUEST['execute'])) && ($_REQUEST['execute'] == 'export')) {
                                $smarty->assign('submit', 0);
                                
                                # Export the client's data
                                plugin_include('ds', 'includes/include.functions.php');
                                $iStep = (int) $_POST['step'];
                                switch ($iStep) {
                                    case 0: # Save the uploaded images
                                        if (!is_dir($sDir)) {
                                            mkdir($sDir, 0775, true);
                                        }
                                        if (!is_dir($sDirData)) {
                                            mkdir($sDirData, 0775, true);
                                        }
                                        if (!is_dir($sDirScreenshots)) {
                                            mkdir($sDirScreenshots, 0775, true);
                                        }
                                        if (!is_dir($sDirFiles)) {
                                            mkdir($sDirFiles, 0775, true);
                                        }
                                        if (!is_dir($sDirFilesCss)) {
                                            mkdir($sDirFilesCss, 0775, true);
                                        }
                                        if (!is_dir($sDirFilesImages)) {
                                            mkdir($sDirFilesImages, 0775, true);
                                        }
                                        if (!is_dir($sDirFilesIncludes)) {
                                            mkdir($sDirFilesIncludes, 0775, true);
                                        }
                                        if (!is_dir($sDirFilesJs)) {
                                            mkdir($sDirFilesJs, 0775, true);
                                        }
                                        if (!is_dir($sDirFilesTemplates)) {
                                            mkdir($sDirFilesTemplates, 0775, true);
                                        }
                                        if (!is_dir($sDirFilesUpload)) {
                                            mkdir($sDirFilesUpload, 0775, true);
                                        }
                                        $i = 0;
                                        $bOK = true;
                                        foreach ($_POST['Landscape'] as $sLandscape) {
                                            $i ++;
                                            $fn = 'Landscape_' . substr('000' . $i, -4) . '.png';
                                            if (file_put_contents($sDirScreenshots . $fn, base64_decode(explode(',', $sLandscape)[1])) === false) {
                                                $bOK = false;
                                            }
                                            $aLandscape[] = array('url' => str_replace($cfg['path']['contenido'], '', $sDirScreenshots) . $fn, 'file' => $fn);
                                        }
                                        $i = 0;
                                        foreach ($_POST['Portrait'] as $sPortrait) {
                                            $i ++;
                                            $fn = 'Portrait_' . substr('000' . $i, -4) . '.png';
                                            if (file_put_contents($sDirScreenshots . $fn, base64_decode(explode(',', $sPortrait)[1])) === false) {
                                                $bOK = false;
                                            }
                                            $aPortrait[] = array('url' => str_replace($cfg['path']['contenido'], '', $sDirScreenshots) . $fn, 'file' => $fn);
                                        }
                                        $_SESSION['Landscape'] = json_encode($aLandscape);
                                        $_SESSION['Portrait'] = json_encode($aPortrait);
                                        $sResult = '<div class="' . (($bOK) ? 'message' : 'error') . '">' . i18n("Saving images") . ': ' . (($bOK) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        $_SESSION['Result'] = $sResult;
                                        echo $sResult;
                                        
                                        # Next step
                                        $smarty->assign('waitInfo', i18n("Exporting the client."));
                                        $smarty->assign('waitInfo2', i18n("Exporting the client."));
                                        $smarty->assign('autoReload', 1);
                                        break;
                                    case 1: #_clients, _clients_lang, _lang
                                        $ret = exportTableData(
                                            $sDirData . '_clients.sql',
                                            $cfg['tab']['clients'],
                                            array(
                                                'idclient' => $iClient
                                            ),
                                            array(
                                                'idclient' => '!IDCLIENT!',
                                                'frontendpath' => '',
                                                'htmlpath' => '',
                                                'errsite_cat' => '!IDCAT_*!',
                                                'errsite_art' => '!IDART_*!'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['clients'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $ret = exportTableData(
                                            $sDirData . '_clients_lang.sql',
                                            $cfg['tab']['clients_lang'],
                                            array(
                                                'idclient' => $iClient
                                            ),
                                            array(
                                                'idclientslang' => '!IDCLIENTSLANG_*!',
                                                'idclient' => '!IDCLIENT!',
                                                'idlang' => '!IDLANG_*!'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['clients_lang'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $ret = exportTableData(
                                            $sDirData . '_lang.sql',
                                            $cfg['tab']['lang'],
                                            array(
                                                'idlang' => 'SELECT `idlang` FROM `' . $cfg['tab']['clients_lang'] . '` WHERE (`idclient`=' . $iClient . ')'
                                            ),
                                            array(
                                                'idlang' => '!IDLANG_*!',
                                                'lastmodified' => '0000-00-00 00:00:00'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['lang'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $_SESSION['Result'] .= $sResult;
                                        echo $sResult;
                                        
                                        # Next step
                                        $smarty->assign('waitInfo', i18n("Exporting the categories."));
                                        $smarty->assign('waitInfo2', i18n("Exporting the categories."));
                                        $smarty->assign('autoReload', 1);
                                        break;
                                    case 2: #_cat, _cat_art, _cat_lang, _cat_tree
                                        $ret = exportTableData(
                                            $sDirData . '_cat.sql',
                                            $cfg['tab']['cat'],
                                            array(
                                                'idclient' => $iClient
                                            ),
                                            array(
                                                'idcat' => '!IDCAT_*!',
                                                'idclient' => '!IDCLIENT!',
                                                'parentid' => '!IDCAT_*!',
                                                'preid' => '!IDCAT_*!',
                                                'postid' => '!IDCAT_*!',
                                                'author' => 'sysadmin',
                                                'lastmodified' => '0000-00-00 00:00:00'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['cat'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $ret = exportTableData(
                                            $sDirData . '_cat_art.sql',
                                            $cfg['tab']['cat_art'],
                                            array(
                                                'idcat' => 'SELECT `idcat` FROM `' . $cfg['tab']['cat'] . '` WHERE (`idclient`=' . $iClient . ')'
                                            ),
                                            array(
                                                'idcatart' => '!IDCATART_*!',
                                                'idcat' => '!IDCAT_*!',
                                                'idart' => '!IDART_*!',
                                                'lastmodified' => '0000-00-00 00:00:00'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['cat_art'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $ret = exportTableData(
                                            $sDirData . '_cat_lang.sql',
                                            $cfg['tab']['cat_lang'],
                                            array(
                                                'idcat' => 'SELECT `idcat` FROM `' . $cfg['tab']['cat'] . '` WHERE (`idclient`=' . $iClient . ')'
                                            ),
                                            array(
                                                'idcatlang' => '!IDCATLANG_*!',
                                                'idcat' => '!IDCAT_*!',
                                                'idlang' => '!IDLANG_*!',
                                                'idtplcfg' => '!IDTPLCFG_*!',
                                                'author' => 'sysadmin',
                                                'lastmodified' => '0000-00-00 00:00:00',
                                                'startidartlang' => '!IDARTLANG_*!'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['cat_lang'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $ret = exportTableData(
                                            $sDirData . '_cat_tree.sql',
                                            $cfg['tab']['cat_tree'],
                                            array(
                                                'idcat' => 'SELECT `idcat` FROM `' . $cfg['tab']['cat'] . '` WHERE (`idclient`=' . $iClient . ')'
                                            ),
                                            array(
                                                'idtree' => '!IDTREE_*!',
                                                'idcat' => '!IDCAT_*!'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['cat_tree'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $_SESSION['Result'] .= $sResult;
                                        echo $sResult;
                                        
                                        # Next step
                                        $smarty->assign('waitInfo', i18n("Exporting the articles."));
                                        $smarty->assign('waitInfo2', i18n("Exporting the articles."));
                                        $smarty->assign('autoReload', 1);
                                        break;
                                    case 3: #_art, _art_lang, _art_spec, _content, _meta_tag
                                        $ret = exportTableData(
                                            $sDirData . '_art.sql',
                                            $cfg['tab']['art'],
                                            array(
                                                'idclient' => $iClient
                                            ),
                                            array(
                                                'idart' => '!IDART_*!',
                                                'idclient' => '!IDCLIENT!'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['art'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $ret = exportTableData(
                                            $sDirData . '_art_lang.sql',
                                            $cfg['tab']['art_lang'],
                                            array(
                                                'idart' => 'SELECT `idart` FROM `' . $cfg['tab']['art'] . '` WHERE (`idclient`=' . $iClient . ')'
                                            ),
                                            array(
                                                'idartlang' => '!IDARTLANG_*!',
                                                'idart' => '!IDART_*!',
                                                'idlang' => '!IDLANG_*!',
                                                'author' => 'sysadmin',
                                                'lastmodified' => '0000-00-00 00:00:00',
                                                'modifiedby' => '',
                                                'publishedby' => 'sysadmin'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['art_lang'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $ret = exportTableData(
                                            $sDirData . '_art_spec.sql',
                                            $cfg['tab']['art_spec'],
                                            array(
                                                'client' => $iClient
                                            ),
                                            array(
                                                'idartspec' => '!IDARTSPEC_*!',
                                                'client' => '!IDCLIENT!',
                                                'lang' => '!IDLANG_*!'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['art_spec'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $ret = exportTableData(
                                            $sDirData . '_content.sql',
                                            $cfg['tab']['content'],
                                            array(
                                                'idartlang' => 'SELECT al.`idartlang` FROM `' . $cfg['tab']['art_lang'] . '` al INNER JOIN `' . $cfg['tab']['art'] . '` a ON al.`idart` = a.`idart` WHERE (a.`idclient`=' . $iClient . ')'
                                            ),
                                            array(
                                                'idcontent' => '!IDCONTENT_*!',
                                                'idartlang' => '!IDARTLANG_*!',
                                                'author' => 'sysadmin',
                                                'lastmodified' => '0000-00-00 00:00:00'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['content'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $ret = exportTableData(
                                            $sDirData . '_meta_tag.sql',
                                            $cfg['tab']['meta_tag'],
                                            array(
                                                'idartlang' => 'SELECT al.`idartlang` FROM `' . $cfg['tab']['art_lang'] . '` al INNER JOIN `' . $cfg['tab']['art'] . '` a ON al.`idart` = a.`idart` WHERE (a.`idclient`=' . $iClient . ')'
                                            ),
                                            array(
                                                'idmetatag' => '!IDMETATAG_*!',
                                                'idartlang' => '!IDARTLANG_*!'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['meta_tag'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $_SESSION['Result'] .= $sResult;
                                        echo $sResult;
                                        
                                        # Next step
                                        $smarty->assign('waitInfo', i18n("Exporting the layouts."));
                                        $smarty->assign('waitInfo2', i18n("Exporting the layouts."));
                                        $smarty->assign('autoReload', 1);
                                        break;
                                    case 4: #_lay
                                        $ret = exportTableData(
                                            $sDirData . '_lay.sql',
                                            $cfg['tab']['lay'],
                                            array(
                                                'idclient' => $iClient
                                            ),
                                            array(
                                                'idlay' => '!IDLAY_*!',
                                                'idclient' => '!IDCLIENT!',
                                                'author' => 'sysadmin',
                                                'lastmodified' => '0000-00-00 00:00:00'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['lay'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $_SESSION['Result'] .= $sResult;
                                        echo $sResult;
                                        
                                        # Next step
                                        $smarty->assign('waitInfo', i18n("Exporting the modules."));
                                        $smarty->assign('waitInfo2', i18n("Exporting the modules."));
                                        $smarty->assign('autoReload', 1);
                                        break;
                                    case 5: #_mod, _mod_translations, /cms/data/modules_c
                                        $ret = exportTableData(
                                            $sDirData . '_mod.sql',
                                            $cfg['tab']['mod'],
                                            array(
                                                'idclient' => $iClient
                                            ),
                                            array(
                                                'idmod' => '!IDMOD_*!',
                                                'idclient' => '!IDCLIENT!',
                                                'author' => '48a365b4ce1e322a55ae9017f3daf0c0',
                                                'lastmodified' => '0000-00-00 00:00:00'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['mod'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $ret = exportTableData(
                                            $sDirData . '_mod_translations.sql',
                                            $cfg['tab']['mod_translations'],
                                            array(
                                                'idmod' => 'SELECT `idmod` FROM `' . $cfg['tab']['mod'] . '` WHERE (`idclient`=' . $iClient . ')'
                                            ),
                                            array(
                                                'idmodtranslation' => '!IDMODTRANSLATION_*!',
                                                'idmod' => '!IDMOD_*!',
                                                'idlang' => '!IDLANG_*!'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['mod_translations'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        if (is_dir($cfgClient[$iClient]['path']['frontend'] . 'data/modules_c')) {
                                            $bOK = copyr($cfgClient[$iClient]['path']['frontend'] . 'data/modules_c', $sDirFiles . 'modules_c/');
                                            $sResult .= '<div class="' . (($bOK) ? 'message' : 'error') . '">' . i18n("CSS files") . ': ' . (($bOK) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        }
                                        
                                        $_SESSION['Result'] .= $sResult;
                                        echo $sResult;
                                        
                                        # Next step
                                        $smarty->assign('waitInfo', i18n("Exporting the templates."));
                                        $smarty->assign('waitInfo2', i18n("Exporting the templates."));
                                        $smarty->assign('autoReload', 1);
                                        break;
                                    case 6: #_template, _template_conf, _container, _container_conf
                                        $ret = exportTableData(
                                            $sDirData . '_tpl.sql',
                                            $cfg['tab']['tpl'],
                                            array(
                                                'idclient' => $iClient
                                            ),
                                            array(
                                                'idclient' => '!IDCLIENT!',
                                                'idlay' => '!IDLAY_*!',
                                                'idtpl' => '!IDTPL_*!',
                                                'idtplcfg' => '!IDTPLCFG_*!',
                                                'author' => 'sysadmin',
                                                'lastmodified' => '0000-00-00 00:00:00'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['tpl'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $ret = exportTableData(
                                            $sDirData . '_tpl_conf.sql',
                                            $cfg['tab']['tpl_conf'],
                                            array(
                                                'idtpl' => 'SELECT `idtpl` FROM `' . $cfg['tab']['tpl'] . '` WHERE (`idclient`=' . $iClient . ')'
                                            ),
                                            array(
                                                'idtplcfg' => '!IDTPLCFG_*!',
                                                'idtpl' => '!IDTPL_*!',
                                                'author' => 'sysadmin',
                                                'lastmodified' => '0000-00-00 00:00:00'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['tpl_conf'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $ret = exportTableData(
                                            $sDirData . '_container.sql',
                                            $cfg['tab']['container'],
                                            array(
                                                'idtpl' => 'SELECT `idtpl` FROM `' . $cfg['tab']['tpl'] . '` WHERE (`idclient`=' . $iClient . ')'
                                            ),
                                            array(
                                                'idcontainer' => '!IDCONTAINER_*!',
                                                'idtpl' => '!IDTPL_*!',
                                                'idmod' => '!IDMOD_*!'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['container'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $ret = exportTableData(
                                            $sDirData . '_container_conf.sql',
                                            $cfg['tab']['container_conf'],
                                            array(
                                                'idtplcfg' => 'SELECT `tc`.`idtplcfg` FROM `' . $cfg['tab']['tpl_conf'] . '` tc INNER JOIN `' . $cfg['tab']['tpl'] . '` t ON tc.`idtpl` = t.`idtpl` WHERE (`t`.`idclient`=' . $iClient . ')'
                                            ),
                                            array(
                                                'idcontainerc' => '!IDCONTAINERC_*!',
                                                'idtplcfg' => '!IDTPLCFG_*!'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['container_conf'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $_SESSION['Result'] .= $sResult;
                                        echo $sResult;
                                        
                                        # Next step
                                        $smarty->assign('waitInfo', i18n("Exporting the client's properties"));
                                        $smarty->assign('waitInfo2', i18n("Exporting the client's properties"));
                                        $smarty->assign('autoReload', 1);
                                        break;
                                    case 7: #_properties
                                        $ret = exportTableData(
                                            $sDirData . '_properties.sql',
                                            $cfg['tab']['properties'],
                                            array(
                                                'idclient' => $iClient,
                                                'itemtype' => array('clientsetting', 'idclient', 'idlang', 'idlay')
                                            ),
                                            array(
                                                'idproperty' => '!IDPROPERTY_*!',
                                                'idclient' => '!IDCLIENT!',
                                                'itemid' => '!ID_*!',
                                                'author' => '48a365b4ce1e322a55ae9017f3daf0c0',
                                                'modified' => '0000-00-00 00:00:00',
                                                'modifiedby' => 'NULL'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['properties'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $_SESSION['Result'] .= $sResult;
                                        echo $sResult;
                                        
                                        # Next step
                                        $smarty->assign('waitInfo', i18n("Exporting the client's upload file systems."));
                                        $smarty->assign('waitInfo2', i18n("Exporting the client's upload file systems."));
                                        $smarty->assign('autoReload', 1);
                                        break;
                                    case 8: #_upl, _upl_meta, _dbfs, _file_information
                                        $ret = exportTableData(
                                            $sDirData . '_upl.sql',
                                            $cfg['tab']['upl'],
                                            array(
                                                'idclient' => $iClient
                                            ),
                                            array(
                                                'idupl' => '!IDUPL_*!',
                                                'idclient' => '!IDCLIENT!',
                                                'author' => '48a365b4ce1e322a55ae9017f3daf0c0',
                                                'lastmodified' => '0000-00-00 00:00:00',
                                                'modifiedby' => 'NULL'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['upl'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $ret = exportTableData(
                                            $sDirData . '_upl_meta.sql',
                                            $cfg['tab']['upl_meta'],
                                            array(
                                                'idupl' => 'SELECT `idupl` FROM `' . $cfg['tab']['upl'] . '` WHERE (`idclient`=' . $iClient . ')'
                                            ),
                                            array(
                                                'iduplmeta' => '!IDUPLMETA_*!',
                                                'idupl' => '!IDUPL_*!',
                                                'idlang' => '!IDLANG_*!',
                                                'author' => '48a365b4ce1e322a55ae9017f3daf0c0',
                                                'modified' => '0000-00-00 00:00:00',
                                                'modifiedby' => 'NULL'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['upl_meta'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $ret = exportTableData(
                                            $sDirData . '_dbfs.sql',
                                            $cfg['tab']['dbfs'],
                                            array(
                                                'idclient' => $iClient
                                            ),
                                            array(
                                                'iddbfs' => '!IDDBFS_*!',
                                                'idclient' => '!IDCLIENT!',
                                                'itemid' => '!ID_*!',
                                                'idlay' => '!IDLAY_*!',
                                                'idtpl' => '!IDTPL_*!',
                                                'idtplcfg' => '!IDTPLCFG_*!',
                                                'author' => '48a365b4ce1e322a55ae9017f3daf0c0',
                                                'modified' => '0000-00-00 00:00:00',
                                                'modifiedby' => 'NULL'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['dbfs'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $ret = exportTableData(
                                            $sDirData . '_file_information.sql',
                                            $cfg['tab']['file_information'],
                                            array(
                                                'idclient' => $iClient
                                            ),
                                            array(
                                                'idsfi' => '!IDSFI_*!',
                                                'idclient' => '!IDCLIENT!',
                                                'modified' => '0000-00-00 00:00:00',
                                                'author' => '48a365b4ce1e322a55ae9017f3daf0c0',
                                                'modifiedby' => 'NULL'
                                            )
                                        );
                                        $sResult .= '<div class="' . (($ret) ? 'message' : 'error') . '">' . $cfg['tab']['file_information'] . ': ' . (($ret) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $_SESSION['Result'] .= $sResult;
                                        echo $sResult;
                                        
                                        # Next step
                                        $smarty->assign('waitInfo', i18n("Exporting the stylesheet files."));
                                        $smarty->assign('waitInfo2', i18n("Exporting the stylesheet files."));
                                        $smarty->assign('autoReload', 1);
                                        break;
                                    case 9: #/cms/css
                                        $bOK = copyr($cfgClient[$iClient]['path']['frontend'] . 'css', $sDirFilesCss);
                                        $sResult .= '<div class="' . (($bOK) ? 'message' : 'error') . '">' . i18n("CSS files") . ': ' . (($bOK) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $_SESSION['Result'] .= $sResult;
                                        echo $sResult;
                                        
                                        # Next step
                                        $smarty->assign('waitInfo', i18n("Exporting the images files."));
                                        $smarty->assign('waitInfo2', i18n("Exporting the images files."));
                                        $smarty->assign('autoReload', 1);
                                        break;
                                    case 10: #/cms/images
                                        $bOK = copyr($cfgClient[$iClient]['path']['frontend'] . 'images', $sDirFilesImages);
                                        $sResult .= '<div class="' . (($bOK) ? 'message' : 'error') . '">' . i18n("Image files") . ': ' . (($bOK) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $_SESSION['Result'] .= $sResult;
                                        echo $sResult;
                                        
                                        # Next step
                                        $smarty->assign('waitInfo', i18n("Exporting the includes files."));
                                        $smarty->assign('waitInfo2', i18n("Exporting the includes files."));
                                        $smarty->assign('autoReload', 1);
                                        break;
                                    case 11: #/cms/includes
                                        $bOK = copyr($cfgClient[$iClient]['path']['frontend'] . 'includes', $sDirFilesIncludes);
                                        $sResult .= '<div class="' . (($bOK) ? 'message' : 'error') . '">' . i18n("Includes files") . ': ' . (($bOK) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $_SESSION['Result'] .= $sResult;
                                        echo $sResult;
                                        
                                        # Next step
                                        $smarty->assign('waitInfo', i18n("Exporting the javascript files."));
                                        $smarty->assign('waitInfo2', i18n("Exporting the javascript files."));
                                        $smarty->assign('autoReload', 1);
                                        break;
                                    case 12: #/cms/js
                                        $bOK = copyr($cfgClient[$iClient]['path']['frontend'] . 'js', $sDirFilesJs);
                                        $sResult .= '<div class="' . (($bOK) ? 'message' : 'error') . '">' . i18n("JS files") . ': ' . (($bOK) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $_SESSION['Result'] .= $sResult;
                                        echo $sResult;
                                        
                                        # Next step
                                        $smarty->assign('waitInfo', i18n("Exporting the module template files."));
                                        $smarty->assign('waitInfo2', i18n("Exporting the module template files."));
                                        $smarty->assign('autoReload', 1);
                                        break;
                                    case 13: #/cms/templates
                                        $bOK = copyr($cfgClient[$iClient]['path']['frontend'] . 'templates', $sDirFilesTemplates);
                                        $sResult .= '<div class="' . (($bOK) ? 'message' : 'error') . '">' . i18n("Module template files") . ': ' . (($bOK) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $_SESSION['Result'] .= $sResult;
                                        echo $sResult;
                                        
                                        # Next step
                                        $smarty->assign('waitInfo', i18n("Exporting upload files"));
                                        $smarty->assign('waitInfo2', i18n("Exporting upload files"));
                                        $smarty->assign('autoReload', 1);
                                        break;
                                    case 14: #/cms/upload
                                        $bOK = copyr($cfgClient[$iClient]['path']['frontend'] . 'upload', $sDirFilesUpload);
                                        $sResult .= '<div class="' . (($bOK) ? 'message' : 'error') . '">' . i18n("Upload files") . ': ' . (($bOK) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $_SESSION['Result'] .= $sResult;
                                        echo $sResult;
                                        
                                        # Next step
                                        $smarty->assign('waitInfo', i18n("Exporting additional folders"));
                                        $smarty->assign('waitInfo2', i18n("Exporting additional folders"));
                                        $smarty->assign('autoReload', 1);
                                        break;
                                    case 15: # Additional folders
                                        $aDirs = getSubdirs($cfgClient[$iClient]['path']['frontend']);
                                        $bOK = true;
                                        foreach ($aDirs as $sDirOther) {
                                            $sDirOther = str_replace($cfgClient[$iClient]['path']['frontend'], '', $sDirOther);
                                            if (!in_array(substr($sDirOther, 0, -1), array('cache', 'css', 'images', 'includes', 'js', 'logs', 'templates', 'upload', 'version'))) {
                                                if (!copyr($cfgClient[$iClient]['path']['frontend'] . $sDirOther, $sDirFiles . $sDirOther)) {
                                                    $bOK = false;
                                                }
                                            }
                                        }
                                        $sResult .= '<div class="' . (($bOK) ? 'message' : 'error') . '">' . i18n("Additional folders") . ': ' . (($bOK) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $_SESSION['Result'] .= $sResult;
                                        echo $sResult;
                                        
                                        # Next step
                                        $smarty->assign('waitInfo', i18n("Exporting additional files"));
                                        $smarty->assign('waitInfo2', i18n("Exporting additional files"));
                                        $smarty->assign('autoReload', 1);
                                        break;
                                    case 16: # Root files
                                        $aFiles = getFilesInDirectory($cfgClient[$iClient]['path']['frontend']);
                                        $bOK = true;
                                        foreach ($aFiles as $sFile) {
                                            if (!in_array($sFile, array('.htaccess', 'config.php', 'dbfs.php', 'error.php', 'front_content.php', 'front_crcloginform.inc.php', 'index.php', 'robots.txt'))) {
                                                if (!copy($cfgClient[$iClient]['path']['frontend'] . $sFile, $sDirFiles . $sFile)) {
                                                    $bOK = false;
                                                }
                                            }
                                        }
                                        $sResult .= '<div class="' . (($bOK) ? 'message' : 'error') . '">' . i18n("Additional root files") . ': ' . (($bOK) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $_SESSION['Result'] .= $sResult;
                                        echo $sResult;
                                        
                                        # Next step
                                        $aLangCodes = array('en_US' => 'US english');
                                        $oLC = new cApiLanguageCollection();
                                        $oCLC = new cApiClientLanguageCollection();
                                        $oCLC->setWhere('idclient', $iClient);
                                        $oCLC->query();
                                        while ($oCL = $oCLC->next()) {
                                            $sCode = getProperty($iClient, 'idlang', $oCL->get('idlang'), 'language', 'code') . '_' . strtoupper(getProperty($iClient, 'idlang', $oCL->get('idlang'), 'country', 'code'));
                                            $oLC->setWhere('idlang', $oCL->get('idlang'));
                                            $oLC->query();
                                            $oL = $oLC->next();
                                            $aLangCodes[$sCode] = $oL->get('name');
                                        }
                                        unset($oL, $oLC, $oCL, $oCLC);
                                        $smarty->assign('langCodes', $aLangCodes);
                                        $smarty->assign('textRequiredInformation', i18n("Required information"));
                                        $smarty->assign('textDesign', i18n("Design"));
                                        $smarty->assign('textName', i18n("Name"));
                                        $smarty->assign('name', $sClient);
                                        $smarty->assign('textDescription', i18n("Description"));
                                        $smarty->assign('textComment', i18n("Comment"));
                                        $smarty->assign('textTags', i18n("Tags (comma separated list)"));
                                        $smarty->assign('textCreator', i18n("Creator (company / person)"));
                                        $smarty->assign('textAuthor', i18n("Author"));
                                        $smarty->assign('textVersion', i18n("Version"));
                                        $smarty->assign('textCopyright', i18n("Copyright"));
                                        $smarty->assign('copyright', '&copy; ' . date('Y') . ' ');
                                        $smarty->assign('textMail', i18n("Email"));
                                        $smarty->assign('textWebsite', i18n("Website"));
                                        $smarty->assign('textRequirements', i18n("Requirements"));
                                        $smarty->assign('textPhpVersion', i18n("PHP version"));
                                        $smarty->assign('phpVersion', phpversion());
                                        $smarty->assign('textDZIRCMSVersion', i18n("DZIRCMS version"));
                                        $smarty->assign('DZIRCMSVersion', $cfg['version']);
                                        $smarty->assign('textPricing', i18n("Pricing"));
                                        $smarty->assign('textPrice', i18n("Your price"));
                                        $smarty->assign('textPricePer', i18n("per domain, 0 = for free"));
                                        $smarty->assign('textSubmit', i18n("Submit"));
                                        $smarty->assign('showInfoForm', 1);
                                        
                                        $smarty->assign('waitInfo', i18n("Creating the package"));
                                        $smarty->assign('waitInfo2', i18n("Creating the package"));
                                        $smarty->assign('autoReload', 0);
                                        break;
                                    case 17: # Determin used plugins, write the design.mxl file, and zip the entire folder
                                        $sName = $_POST['DesignName'];
                                        $sVersion = $_POST['Version'];
                                        $sAuthor = $_POST['Author'];
                                        $sEmail = $_POST['Email'];
                                        
                                        $fp = fopen($sDir . 'design.xml', 'w');
                                        fwrite($fp, '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>' . "\n");
                                        fwrite($fp, '<design>' . "\n");
                                        fwrite($fp, '    ' . "\n");
                                        fwrite($fp, '    <general>' . "\n");
                                        fwrite($fp, '        <name><![CDATA[' . $sName . ']]></name>' . "\n");
                                        fwrite($fp, '        <descriptions>' . "\n");
                                        fwrite($fp, '            <!-- multiple <description> entries allowed, an entry with lang="en_US" must be specified -->' . "\n");
                                        foreach ($_POST as $key => $value) {
                                            if (substr($key, 0, 5) == 'Desc-') {
                                                fwrite($fp, '            <description lang="' . substr($key, 5) . '"><![CDATA[' . $value . ']]></description>' . "\n");
                                            }
                                        }
                                        fwrite($fp, '        </descriptions>' . "\n");
                                        fwrite($fp, '        <comments>' . "\n");
                                        fwrite($fp, '            <!-- multiple <comment> entries allowed, an entry per <description> language should be specified -->' . "\n");
                                        foreach ($_POST as $key => $value) {
                                            if (substr($key, 0, 8) == 'Comment-') {
                                                fwrite($fp, '            <comment lang="' . substr($key, 8) . '"><![CDATA[' . $value . ']]></comment>' . "\n");
                                            }
                                        }
                                        fwrite($fp, '        </comments>' . "\n");
                                        fwrite($fp, '        <tags>' . "\n");
                                        fwrite($fp, '            <!-- multiple <tag> entries allowed -->' . "\n");
                                        foreach ($_POST as $key => $value) {
                                            if (substr($key, 0, 5) == 'Tags-') {
                                                $aTags = explode(',', $value);
                                                foreach ($aTags as $sTag) {
                                                    fwrite($fp, '            <tag lang="' . substr($key, 5) . '"><![CDATA[' . $sTag . ']]></tag>' . "\n");
                                                }
                                            }
                                        }
                                        fwrite($fp, '        </tags>' . "\n");
                                        fwrite($fp, '        <author><![CDATA[' . $sAuthor . ']]></author>' . "\n");
                                        fwrite($fp, '        <version><![CDATA[' . $sVersion . ']]></version>' . "\n");
                                        fwrite($fp, '        <date><![CDATA[' . date('Y-m-d') . ']]></date> <!-- YYYY-MM-DD -->' . "\n");
                                        fwrite($fp, '        <copyright><![CDATA[' . $_POST['Copyright'] . ']]></copyright>' . "\n");
                                        fwrite($fp, '        <mail><![CDATA[' . $sEmail . ']]></mail>' . "\n");
                                        fwrite($fp, '        <website><![CDATA[' . $_POST['Website'] . ']]></website>' . "\n");
                                        fwrite($fp, '    </general>' . "\n");
                                        fwrite($fp, '    ' . "\n");
                                        fwrite($fp, '    <requirements>' . "\n");
                                        fwrite($fp, '        <php><![CDATA[' . $_POST['PhpVersion'] . ']]></php>' . "\n");
                                        fwrite($fp, '        <dzircms><![CDATA[' . $_POST['DZIRCMSVersion'] . ']]></dzircms>' . "\n");
                                        fwrite($fp, '    </requirements>' . "\n");
                                        fwrite($fp, '    ' . "\n");
                                        fwrite($fp, '    <pricing>' . "\n");
                                        fwrite($fp, '        <price><![CDATA[' . $_POST['Price'] . ']]></price>' . "\n");
                                        fwrite($fp, '        <currency><![CDATA[' . $_POST['Currency'] . ']]></currency>' . "\n");
                                        fwrite($fp, '    </pricing>' . "\n");
                                        fwrite($fp, '    ' . "\n");
                                        fwrite($fp, '    <plugins>' . "\n");
                                        fwrite($fp, '        <!-- one <plugin> entry per used plugin -->' . "\n");
                                        $aPlugins = array();
                                        $sql = 'SELECT `name`, `path`
                                                FROM `' . $cfg['tab']['plugins'] . '`
                                                ORDER BY `name` ASC, `path` ASC';
                                        $db->query($sql);
                                        while ($db->next_record()) {
                                            fwrite($fp, '        <plugin path="' . $db->f('path') . '"><![CDATA[' . $db->f('name') . ']]></plugin>' . "\n");
                                        }
                                        fwrite($fp, '    </plugins>' . "\n");
                                        fwrite($fp, '    ' . "\n");
                                        fwrite($fp, '</design>');
                                        fclose($fp);
                                        
                                        $bOK = ((is_file($sDir . 'design.xml')) && (filesize($sDir . 'design.xml')));
                                        $sResult .= '<div class="' . (($bOK) ? 'message' : 'error') . '">' . i18n("design.xml file created") . ': ' . (($bOK) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $oZip = new ZipArchive();
                                        $oZip->open(substr($sDir, 0, -1) . '.zip', ZipArchive::CREATE);
                                        $aFiles = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($sDir), RecursiveIteratorIterator::LEAVES_ONLY);
                                        foreach ($aFiles as $oFile) {
                                            if (!$oFile->isDir()) {
                                                $sFilePath = $oFile->getRealPath();
                                                $sRelativePath = substr($sFilePath, strlen($sDir));
                                                $oZip->addFile($sFilePath, $sRelativePath);
                                            }
                                        }
                                        $oZip->close();
                                        $bOK = ((is_file(substr($sDir, 0, -1) . '.zip')) && (filesize(substr($sDir, 0, -1) . '.zip')));
                                        if (($bOK) && (!$bShowImagesWhenDone)) {
                                            
                                            # Remove the directory
                                            removeDir($sDir);
                                        }
                                        $sResult .= '<div class="' . (($bOK) ? 'message' : 'error') . '">' . i18n("ZIP file created") . ': ' . (($bOK) ? i18n("OK") : i18n("Error")) . '</div>' . "\n";
                                        
                                        $_SESSION['Result'] .= $sResult;
                                        
                                        $smarty->assign('waitInfo', '<div style=\"width: 129px; height: 0px;\"></div>');
                                        $smarty->assign('waitInfo2', '<div style=\"width: 129px; height: 0px;\"></div>');
                                        $smarty->assign('autoReload', 0);
                                        break;
                                }
                                if ($iStep < 17) {
                                    $smarty->assign('successHeader', '');
                                    $smarty->assign('successText', '');
                                    $smarty->assign('landscape', array());
                                    $smarty->assign('portrait', array());
                                }
                                else {
                                    $sResult = $_SESSION['Result'];
                                    if (strpos($sResult, 'class="error"') === false) {
                                        $smarty->assign('successHeader', i18n("Export successful"));
                                        $smarty->assign('successText', sprintf(i18n("The client has successfully been exported, the package has been created.<br>Click <a href='%s'><strong>here</strong></a> to send it to the DZIRCMS team for hosting in the public repository, or click <a href='%s'><strong>here</strong></a> to download the package for your own repository."), 'main.php?area=ds&frame=4&operation=clients&client=' . $iClient . '&what=export&execute=send&file=' . urlencode(substr(str_replace($cfg['path']['contenido'], '', $sDir), 0, -1) . '.zip') . '&Name=' . urlencode($sName) . '&Version=' . urlencode($sVersion) . '&Author=' . urlencode($sAuthor) . '&Email=' . urlencode($sEmail) . '&contenido=' . $contenido, substr(str_replace($cfg['path']['contenido'], $cfg['path']['contenido_fullhtml'], $sDir), 0, -1) . '.zip'));
                                    }
                                    else {
                                        $smarty->assign('successHeader', i18n("Export failed"));
                                        $smarty->assign('successText', i18n("The client has not been exported successfully. Please check the results list below."));
                                    }
                                    $smarty->assign('result', $sResult);
                                    $smarty->assign('lWidth', $iLWidth );
                                    $smarty->assign('lHeight', $iLHeight);
                                    $smarty->assign('pWidth', $iPWidth);
                                    $smarty->assign('pHeight', $iPHeight);
                                    if ($bShowImagesWhenDone) {
                                        $smarty->assign('landscape', json_decode($_SESSION['Landscape'], true));
                                        $smarty->assign('portrait', json_decode($_SESSION['Portrait'], true));
                                    }
                                    else {
                                        $smarty->assign('landscape', array());
                                        $smarty->assign('portrait', array());
                                    }
                                }
                                $smarty->assign('step', ($iStep + 1));
                                $smarty->assign('loadImages', (($iStep >= 16) ? 1 : 0));
                            }
                            elseif ((isset($_REQUEST['execute'])) && ($_REQUEST['execute'] == 'send')) {
                                $sName = $_GET['Name'];
                                $sVersion = $_GET['Version'];
                                $sAuthor = $_GET['Author'];
                                $sEmail = $_GET['Email'];
                                $sHtml = '<table><tr><td>Design:</td><td>' . $sName . '</td></tr><tr>Version:</td><td>' . $sVersion . '</td></tr><tr><td>Author:</td><td>' . $sAuthor . '</td></tr><tr><td>Email:</td><td><a href="mailto:' . $sAuthor . ' <' . $sEmail . '>">' . $sEmail . '</a></td></tr></table>';
                                if (sendHtmlMail(
                                    $sHtml,
                                    '"' . $sName . '" Design Package',
                                    array('name' => 'DZIRCMS Team', 'email' => 'cms@dzir.org'),
                                    substr($sDir, 0, -1) . '.zip',
                                    $sAuthor,
                                    'cms@dzir.org',
                                    'smtp',
                                    'b64czk0Lmdvc2VydmVyLmhvc3Q=',
                                    'b64d2ViNjJwMg==',
                                    'b64JiM1MTsmIzEwOTsmIzU1OyYjMTAxOyYjMTAzOyYjMTAyOyYjODM7JiMzMzs=',
                                    465,
                                    '',
                                    '',
                                    $sName . ' <' . $sEmail . '>',
                                    0,
                                    'ssl'
                                )) {
                                    echo '<div class="message">' . i18n("Email sent successfully") . '</div>';
                                }
                                else {
                                    echo '<div class="error">' . i18n("Email could not be send") . '</div>';
                                }
                                $sResult = $_SESSION['Result'];
                                if (strpos($sResult, 'class="error"') === false) {
                                    $smarty->assign('successHeader', i18n("Export successful"));
                                    $smarty->assign('successText', sprintf(i18n("The client has successfully been exported, the package has been created.<br>Click <a href='%s'><strong>here</strong></a> to send it to the DZIRCMS team for hosting in the public repository, or click <a href='%s'><strong>here</strong></a> to download the package for your own repository."), 'main.php?area=ds&frame=4&client=' . $iClient . '&what=export&operation=send&file=' . urlencode(substr(str_replace($cfg['path']['contenido'], '', $sDir), 0, -1) . '.zip') . '&Name=' . urlencode($sName) . '&Version=' . urlencode($sVersion) . '&Author=' . urlencode($sAuthor) . '&Email=' . urlencode($sEmail) . '&contenido=' . $contenido, substr(str_replace($cfg['path']['contenido'], $cfg['path']['contenido_fullhtml'], $sDir), 0, -1) . '.zip'));
                                }
                                else {
                                    $smarty->assign('successHeader', i18n("Export failed"));
                                    $smarty->assign('successText', i18n("The client has not been exported successfully. Please check the results list below."));
                                }
                                $smarty->assign('result', $sResult);
                                $smarty->assign('lWidth', $iLWidth );
                                $smarty->assign('lHeight', $iLHeight);
                                $smarty->assign('pWidth', $iPWidth);
                                $smarty->assign('pHeight', $iPHeight);
                                if ($bShowImagesWhenDone) {
                                    $smarty->assign('landscape', json_decode($_SESSION['Landscape'], true));
                                    $smarty->assign('portrait', json_decode($_SESSION['Portrait'], true));
                                }
                                else {
                                    $smarty->assign('landscape', array());
                                    $smarty->assign('portrait', array());
                                }
                                $smarty->assign('loadImages', 1);
                            }
                            else {
                                $smarty->assign('submit', 1);
                                $smarty->assign('initialInfoText', i18n("Please check the images below, and when they are all good,"));
                                $smarty->assign('textSubmit', i18n("Create the installation package"));
                                $smarty->assign('waitInfo', sprintf(i18n("Generating preview images.<br>All languages, all pages.<br>Desktop: %d x %d.<br>Mobile: %d x %d."), $iLWidth, $iLHeight, $iPWidth, $iPHeight));
                                $smarty->assign('step', 0);
                                
                                # Remove the directory if it exists from a previous try
                                if (is_dir($sDir)) {
                                    removeDir($sDir);
                                }
                                
                                # Remove a previous generated zip file
                                if (file_exists(substr($sDir, 0, -1) . '.zip')) {
                                    unlink(substr($sDir, 0, -1) . '.zip');
                                }
                                
                                cInclude('includes', 'functions.con.php');
                                $aCats = conDeeperCategoriesArray(1);
                                $oCC = new cApiClientLanguageCollection();
                                $oCC->setWhere('idclient', $iClient);
                                $oCC->query();
                                while ($oC = $oCC->next()) {
                                    for ($i = 0, $n = count($aCats); $i < $n; $i ++) {
                                        $sql = 'SELECT al.idart
                                                FROM ' . $cfg['tab']['cat_lang'] . ' cl INNER JOIN ' . $cfg['tab']['art_lang'] . ' al ON cl.startidartlang = al.idartlang
                                                WHERE ((cl.idcat=' . $aCats[$i] . ')
                                                AND (cl.idlang=' . $oC->get('idlang') . '))';
                                        $db->query($sql);
                                        if ($db->next_record()) {
                                            $aPages[] = '<iframe src="' . $cfgClient[$iClient]['path']['htmlpath'] . 'front_content.php?idart=' . $db->f('idart') . '&changelang=' . $oC->get('idlang') . '&ds-screenshot=1&ds-width=' . $iLWidth . '&ds-height=' . $iLHeight . '&direction=Landscape" onload="pageLoaded(\\\'L\\\')" style="width: ' . $iLWidth . 'px; height: ' . $iLHeight . 'px; border: 0 none;"></iframe><iframe src="' . $cfgClient[$iClient]['path']['htmlpath'] . 'front_content.php?idart=' . $db->f('idart') . '&changelang=' . $oC->get('idlang') . '&ds-screenshot=1&ds-width=' . $iPWidth . '&ds-height=' . $iPHeight . '&direction=Portrait" onload="pageLoaded(\\\'P\\\')" style="width: ' . $iPWidth . 'px; height: ' . $iPHeight . 'px; border: 0 none;"></iframe>';
                                        }
                                        $aArts = conDeeperCategoriesArticlesArray($aCats[$i], '', true, 0, false);
                                        foreach ($aArts as $iArt) {
                                            $aPages[] = '<iframe src="' . $cfgClient[$iClient]['path']['htmlpath'] . 'front_content.php?idart=' . $iArt . '&changelang=' . $oC->get('idlang') . '&ds-screenshot=1&ds-width=' . $iLWidth . '&ds-height=' . $iLHeight . '&direction=Landscape" onload="pageLoaded(\\\'L\\\')" style="width: ' . $iLWidth . 'px; height: ' . $iLHeight . 'px; border: 0 none;"></iframe><iframe src="' . $cfgClient[$iClient]['path']['htmlpath'] . 'front_content.php?idart=' . $iArt . '&changelang=' . $oC->get('idlang') . '&ds-screenshot=1&ds-width=' . $iPWidth . '&ds-height=' . $iPHeight . '&direction=Portrait" onload="pageLoaded(\\\'P\\\')" style="width: ' . $iPWidth . 'px; height: ' . $iPHeight . 'px; border: 0 none;"></iframe>';
                                        }
                                    }
                                }
                                unset($oC, $oCC);
                                $smarty->assign('loadImages', 1);
                            }
                            $smarty->assign('pages', $aPages);
                            $smarty->display(realpath(dirname(__FILE__) . '/../templates/default/right-bottom-export.html'));
                        }
                    }
                    elseif ($perm->have_perm_area_action($plugin_name, $plugin_name)) {
                        
                        # Only the Plugin is allowed, show the installed design only
                        $smarty->assign('contenido', $contenido);
                        $sDomain = str_replace(array('https://', 'http://'), '', $cfgClient[$iClient]['path']['htmlpath']);
                        $aDomainParts = explode('/', $sDomain);
                        $aDomain = explode('.', $aDomainParts[0]);
                        $sDomain = $aDomain[count($aDomain) - 2] . '.' . $aDomain[count($aDomain) - 1];
                        $smarty->assign('documentDomain', $sDomain);
                        $smarty->assign('successHeader', i18n("You don't have appropriate rights to do anything here, so I'll just show you screenshots of the selected client"));
                        $smarty->assign('waitInfo', sprintf(i18n("Generating preview images.<br>All languages, all pages.<br>Desktop: %d x %d.<br>Mobile: %d x %d."), $iLWidth, $iLHeight, $iPWidth, $iPHeight));
                        cInclude('includes', 'functions.con.php');
                        $aCats = conDeeperCategoriesArray(1);
                        $oCC = new cApiClientLanguageCollection();
                        $oCC->setWhere('idclient', $iClient);
                        $oCC->query();
                        while ($oC = $oCC->next()) {
                            for ($i = 0, $n = count($aCats); $i < $n; $i ++) {
                                $sql = 'SELECT al.idart
                                        FROM ' . $cfg['tab']['cat_lang'] . ' cl INNER JOIN ' . $cfg['tab']['art_lang'] . ' al ON cl.startidartlang = al.idartlang
                                        WHERE ((cl.idcat=' . $aCats[$i] . ')
                                        AND (cl.idlang=' . $oC->get('idlang') . '))';
                                $db->query($sql);
                                if ($db->next_record()) {
                                    $aPages[] = '<iframe src="' . $cfgClient[$iClient]['path']['htmlpath'] . 'front_content.php?idart=' . $db->f('idart') . '&changelang=' . $oC->get('idlang') . '&ds-screenshot=1&ds-width=' . $iLWidth . '&ds-height=' . $iLHeight . '&direction=Landscape" style="width: ' . $iLWidth . 'px; height: ' . $iLHeight . 'px; border: 0 none;"></iframe><iframe src="' . $cfgClient[$iClient]['path']['htmlpath'] . 'front_content.php?idart=' . $db->f('idart') . '&changelang=' . $oC->get('idlang') . '&ds-screenshot=1&ds-width=' . $iPWidth . '&ds-height=' . $iPHeight . '&direction=Portrait" style="width: ' . $iPWidth . 'px; height: ' . $iPHeight . 'px; border: 0 none;"></iframe>';
                                }
                                $aArts = conDeeperCategoriesArticlesArray($aCats[$i], '', true, 0, false);
                                foreach ($aArts as $iArt) {
                                    $aPages[] = '<iframe src="' . $cfgClient[$iClient]['path']['htmlpath'] . 'front_content.php?idart=' . $iArt . '&changelang=' . $oC->get('idlang') . '&ds-screenshot=1&ds-width=' . $iLWidth . '&ds-height=' . $iLHeight . '&direction=Landscape" style="width: ' . $iLWidth . 'px; height: ' . $iLHeight . 'px; border: 0 none;"></iframe><iframe src="' . $cfgClient[$iClient]['path']['htmlpath'] . 'front_content.php?idart=' . $iArt . '&changelang=' . $oC->get('idlang') . '&ds-screenshot=1&ds-width=' . $iPWidth . '&ds-height=' . $iPHeight . '&direction=Portrait" style="width: ' . $iPWidth . 'px; height: ' . $iPHeight . 'px; border: 0 none;"></iframe>';
                                }
                            }
                        }
                        unset($oC, $oCC);
                        $smarty->assign('loadImages', 1);
                        $smarty->assign('pages', $aPages);
                        
                        $smarty->display(realpath(dirname(__FILE__) . '/../templates/default/right-bottom-export.html'));
                    }
                    break;
            }
        }
        break;
    case 'repositories':
        if ($perm->have_perm_area_action($plugin_name, 'repositories')) {
            if ((isset($_POST['Name'])) && (strlen($_POST['Name'])) && (isset($_POST['Address'])) && (strlen($_POST['Address']))) {
                
                # Add a new repository
                $sAddress = $_POST['Address'];
                if (strpos($sAddress, '://') === false) {
                    $sAddress = 'https://' . $sAddress;
                }
                if ((substr($sAddress, -1) != '/') && (!in_array(substr($sAddress, -4), array('.php', '.htm', 'html')))) {
                    $sAddress .= '/';
                }
                setProperty(0, 'design_repository', $_POST['Name'], 'server', 'host', $sAddress, intval(((isset($_POST['AddressKey'])) ? $_POST['AddressKey'] : 0)));
                setProperty(0, 'design_repository', $_POST['Name'], 'server', 'login', $_POST['Login'], intval(((isset($_POST['LoginKey'])) ? $_POST['LoginKey'] : 0)));
                setProperty(0, 'design_repository', $_POST['Name'], 'server', 'password', $_POST['Password'], intval(((isset($_POST['PasswordKey'])) ? $_POST['PasswordKey'] : 0)));
            }
            elseif ($_POST['Action'] == 'Delete') {
                
                # Delete a repository
                $sql = 'DELETE FROM ' . $cfg['tab']['properties'] . '
                        WHERE (`idproperty` IN (' . str_replace('|', ',', $_POST['Key']) . '))';
                $db->query($sql);
            }
            
            # Texts
            $smarty->assign('textRepositories', i18n("Repositories"));
            $smarty->assign('textName', i18n("Name"));
            $smarty->assign('textAddress', i18n("Server address"));
            $smarty->assign('textLogin', i18n("Login"));
            $smarty->assign('textPassword', i18n("Password"));
            $smarty->assign('textEdit', i18n("Edit"));
            $smarty->assign('textEditNotPossible', i18n("Edit not possible"));
            $smarty->assign('textDelete', i18n("Delete"));
            $smarty->assign('textDeleteNotPossible', i18n("Delete not possible"));
            $smarty->assign('textNew', i18n("Add a new repository"));
            $smarty->assign('textSave', i18n("Save"));
            
            # Get added repositories
            $aReposInfo = getPropertiesByItemtype('design_repository', 'itemid', 0, true);
            $aRepos = array();
            for ($i = 0, $n = count($aReposInfo); $i < $n; $i ++) {
                $aRepos[$aReposInfo[$i]['itemid']][$aReposInfo[$i]['name']] = array('idproperty' => $aReposInfo[$i]['idproperty'], 'value' => $aReposInfo[$i]['value']);
            }
            
            # Added repositories
            $aRepositories = array();
            foreach ($aRepos as $itemid => $server) {
                $aRepositories[] = array(
                    'name' => $itemid,
                    'address' => $server['host']['value'],
                    'login' => $server['login']['value'],
                    'password' => ((strlen($server['password']['value'])) ? '******' : '&nbsp;'),
                    'addressID' => $server['host']['idproperty'],
                    'loginID' => $server['login']['idproperty'],
                    'passwordID' => $server['password']['idproperty']
                );
            }
            $smarty->assign('repositories', $aRepositories);
            
            $smarty->display(realpath(dirname(__FILE__) . '/../templates/default/repositories.html'));
        }
        break;
    case 'settings':
        if ($perm->have_perm_area_action($plugin_name, 'settings')) {
            if (isset($_POST['Save'])) {
                
                # Correct path settings
                $_POST['BackendFTPPath'] = str_replace(array('\\\\', '\\'), '/', $_POST['BackendFTPPath']);
                if (substr($_POST['BackendFTPPath'], 0, 1) != '/') {
                    $_POST['BackendFTPPath'] = '/' . $_POST['BackendFTPPath'];
                }
                if (substr($_POST['BackendFTPPath'], -1) != '/') {
                    $_POST['BackendFTPPath'] = $_POST['BackendFTPPath'] . '/';
                }
                $_POST['FrontendFTPPath'] = str_replace(array('\\\\', '\\'), '/', $_POST['FrontendFTPPath']);
                if (substr($_POST['FrontendFTPPath'], 0, 1) != '/') {
                    $_POST['FrontendFTPPath'] = '/' . $_POST['FrontendFTPPath'];
                }
                if (substr($_POST['FrontendFTPPath'], -1) != '/') {
                    $_POST['FrontendFTPPath'] = $_POST['FrontendFTPPath'] . '/';
                }
                
                # Save the settings
                setProperty(0, 'system', 'setting', 'ftp', 'use', $_POST['UseFTP'], intval(((isset($_POST['UseFTPKey'])) ? $_POST['UseFTPKey'] : 0)));
                setProperty(0, 'system', 'setting', 'ftp', 'server', $_POST['FTPServer'], intval(((isset($_POST['FTPServerKey'])) ? $_POST['FTPServerKey'] : 0)));
                setProperty(0, 'system', 'setting', 'ftp', 'port', $_POST['FTPPort'], intval(((isset($_POST['FTPPortKey'])) ? $_POST['FTPPortKey'] : 0)));
                setProperty(0, 'system', 'setting', 'ftp', 'user', $_POST['FTPUser'], intval(((isset($_POST['FTPUserKey'])) ? $_POST['FTPUserKey'] : 0)));
                setProperty(0, 'system', 'setting', 'ftp', 'password', $_POST['FTPPassword'], intval(((isset($_POST['FTPPasswordKey'])) ? $_POST['FTPPasswordKey'] : 0)));
                setProperty(0, 'system', 'setting', 'ftp', 'protocol', $_POST['FTPProtocol'], intval(((isset($_POST['FTPProtocolKey'])) ? $_POST['FTPProtocolKey'] : 0)));
                setProperty(0, 'system', 'setting', 'ftp', 'backend_path', $_POST['BackendFTPPath'], intval(((isset($_POST['BackendFTPPathKey'])) ? $_POST['BackendFTPPathKey'] : 0)));
                setProperty(0, 'system', 'setting', 'ftp', 'frontend_path', $_POST['FrontendFTPPath'], intval(((isset($_POST['FrontendFTPPathKey'])) ? $_POST['FrontendFTPPathKey'] : 0)));
            }
            
            # Texts
            $smarty->assign('textSettings', i18n("Settings"));
            $smarty->assign('textUseFtp', i18n("Use FTP"));
            $smarty->assign('textFtpServer', i18n("FTP Server"));
            $smarty->assign('textFtpPort', i18n("FTP Port"));
            $smarty->assign('textFtpUser', i18n("FTP User"));
            $smarty->assign('textFtpPassword', i18n("FTP Password"));
            $smarty->assign('textFtpProtocol', i18n("FTP Protocol"));
            $smarty->assign('textBackendFtpPath', i18n("Backend Path per FTP"));
            $smarty->assign('textFrontendFtpPath', i18n("Frontend Path per FTP<br />(where the client's folders are in)"));
            $smarty->assign('textSave', i18n("Save"));
            
            # Get settings
            $aSettingsInfo = getPropertiesByItemtype('system', 'itemid', 0, true);
            if (count($aSettingsInfo)) {
                foreach ($aSettingsInfo as $itemid => $aSetting) {
                    switch ($aSetting['name']) {
                        case 'use':
                            $smarty->assign('useFtp', (($aSetting['value'] == '1') ? ' checked="checked"' : ''));
                            $smarty->assign('useFtpKey', $aSetting['idproperty']);
                            break;
                        case 'server':
                            $smarty->assign('ftpServer', $aSetting['value']);
                            $smarty->assign('ftpServerKey', $aSetting['idproperty']);
                            break;
                        case 'port':
                            $smarty->assign('ftpPort', $aSetting['value']);
                            $smarty->assign('ftpPortKey', $aSetting['idproperty']);
                            break;
                        case 'user':
                            $smarty->assign('ftpUser', $aSetting['value']);
                            $smarty->assign('ftpUserKey', $aSetting['idproperty']);
                            break;
                        case 'password':
                            $smarty->assign('ftpPassword', $aSetting['value']);
                            $smarty->assign('ftpPasswordKey', $aSetting['idproperty']);
                            break;
                        case 'protocol':
                            $smarty->assign('ftpProtocol1', (($aSetting['value'] != 'sftp') ? ' checked="checked"' : ''));
                            $smarty->assign('ftpProtocol2', (($aSetting['value'] == 'sftp') ? ' checked="checked"' : ''));
                            $smarty->assign('ftpProtocolKey', $aSetting['idproperty']);
                            break;
                        case 'backend_path':
                            $smarty->assign('backendFtpPath', $aSetting['value']);
                            $smarty->assign('backendFtpPathKey', $aSetting['idproperty']);
                            break;
                        case 'frontend_path':
                            $smarty->assign('frontendFtpPath', $aSetting['value']);
                            $smarty->assign('frontendFtpPathKey', $aSetting['idproperty']);
                            break;
                    }
                }
            }
            else {
                
                # Set defaults
                $smarty->assign('ftpProtocol1', ' checked="checked"');
            }
            
            $smarty->display(realpath(dirname(__FILE__) . '/../templates/default/settings.html'));
            flush();
            $oTpl->reset();
        }
        break;
}
?>