<?php
/**
 * Project:
 * DZIRCMS Content Management System
 *
 * Description:
 * Module Manager
 *
 * Requirements:
 * @con_php_req 5.4
 *
 *
 * @package    DZIRCMS Frontend modules
 * @version    1.0
 * @author     René Mansveld
 * @copyright  Spider IT Deutschland
 * @license    https://www.dzir.org/cms/license/LICENCE.txt
 * @link       https://www.dzir.org/cms/
 * @link       https://www.spider-it.de/
 * @since      file available since DZIRCMS release 2.2.0
 *
 * {@internal
 *   created  2022-01-06
 * }}
 *
 */

if (!defined('CON_FRAMEWORK')) {
    die('Illegal call');
}

$plugin_name = "mom";

if (!$perm->have_perm_area_action($plugin_name, $plugin_name)) {
    exit;
}
elseif (!$client) {
    die('<div style="margin-left: 246px;">' . $notification->returnNotification("error", sprintf(i18n("No client installed or selected"), $sFTPServer)) . '</div>');
}

function isWritablePath($sPath) {
    $sPath .= ((substr($sPath, -1) == DIRECTORY_SEPARATOR) ? '' : DIRECTORY_SEPARATOR);
    $sFile = 'sit_' . uniqid(mt_rand()) . '.writable';
    $bOK = false;
    $fp = @fopen($sPath . $sFile, 'w');
    if ($fp !== false) {
        $bOK = true;
        fclose($fp);
        unlink($sPath . $sFile);
    }
    return $bOK;
}
function copyr($src, $dst) {
    $src .= ((substr($src, -1) == '/') ? '' : '/');
    $dst .= ((substr($dst, -1) == '/') ? '' : '/');
    
    $dir = opendir($src);
    @mkdir($dst, 0775, true);
    while (($file = readdir($dir)) !== false) {
        if (($file != '.') && ($file != '..')) {
            if (is_dir($src . '/' . $file)) {
                copyr($src . $file, $dst . $file);
            }
            else {
                copy($src . $file, $dst . $file);
            }
        }
    }
    closedir($dir);
}
function installModule($sModule, $bUpdate) {
    global $db, $cfg, $cfgClient, $client;
    
    $oModule = new ModuleInfo($sModule);
    $aInstall = $oModule->getInstallInfo();
    $aInfo = $oModule->getInfo();
    if ($aInstall !== false) {
        if ($bUpdate) {
            
            # Update the module instances for the client (can be more than one)
            $sql = 'SELECT `idmod`, `name`, `modified`
                    FROM `' . $cfg['tab']['mod'] . '`
                    WHERE (`orig_name`="' . Contenido_Security::escapeDB($aInfo['Name'], $db) . '")
                    ORDER BY `name`';
            $db->query($sql);
            if ($db->num_rows()) {
                while ($db->next_record()) {
                    if ($db->f('modified')) {
                        echo '<p>' . sprintf(i18n("Module %s not updated, it was changed"), $db->f('name')) . '</p>';
                        flush();
                    }
                    else {
                        $oAM = new cApiModule($db->f('idmod'));
                        $oAM->import($cfgClient[$client]['path']['frontend'] . 'data/modules_c/' . $sModule . '/' . $aInstall['module']);
                        echo '<p>' . sprintf(i18n("Module %s updated"), $aInfo['Name']) . '</p>';
                        flush();

                        # Install the translations for the client
                        installModuleTranslations($aInstall, $oAM, $sModule);
                    }
                }
            }
            else {
                $bUpdate = false;
            }
        }
        if (!$bUpdate) {
            
            # Install the module for the client
            $oAMC = new cApiModuleCollection();
            $oAM = $oAMC->create($aInfo['Name']);
            $oAM->import($cfgClient[$client]['path']['frontend'] . 'data/modules_c/' . $sModule . '/' . $aInstall['module']);
            echo '<p>' . sprintf(i18n("Module %s installed"), $aInfo['Name']) . '</p>';
            flush();

            # Install the translations for the client
            installModuleTranslations($aInstall, $oAM, $sModule);
        }
        
        return true;
    }
    else {
        return false;
    }
}
function installModuleTranslations($aInstall, $oAM, $sModule) {
    global $cfgClient, $client;

    if (count($aInstall['languages'])) {
        $aLangs = array();
        $oCLC = new cApiClientLanguageCollection();
        $oCLC->setWhere('idclient', $client);
        $oCLC->query();
        
        while ($oItem = $oCLC->next()) {
            $oLang = new cApiLanguage($oItem->get("idlang"));
            $aLangs[$oItem->get("idlang")] = array('name' => $oLang->get('name'), 'code' => $oLang->getProperty('language', 'code') . '_' . strtoupper($oLang->getProperty('country', 'code')));
        }
        unset($oCLC, $oItem, $oLang);
        
        $oTranslations = new cApiModuleTranslationCollection();
        foreach ($aInstall['languages'] as $sLangCode => $sLangFile) {
            
            # Install the translations only if this language is present
            foreach ($aLangs as $idlang => $aLang) {
                if ($aLang['code'] == $sLangCode) {
                    $oTranslations->import($oAM->get('idmod'), $idlang, $cfgClient[$client]['path']['frontend'] . 'data/modules_c/' . $sModule . '/' . $sLangFile);
                    echo '<p>' . sprintf(i18n("Module translations installed for language %s"), $aLang['name']) . '</p>';
                    flush();
                }
            }
        }
        unset($oTranslations);
    }
}
function addDirSep($sPath) {
    return $sPath . ((substr($sPath, -1) == '/') ? '' : '/');
}

# Make the pages show faster, even with slow backend internet connections
@ini_set('zlib.output_compression', 0);
@ini_set('implicit_flush', true);
@ini_set('output_buffering', 0);
ob_implicit_flush(1);
while (ob_get_level()) ob_end_flush();

# Include needed files
plugin_include($plugin_name, 'includes/config.plugin.php');
plugin_include($plugin_name, 'classes/class.moduleinfo.php');

$oTpl = new Template();

$sWhat = ((isset($_REQUEST['what'])) ? $_REQUEST['what'] : (($perm->have_perm_area_action($plugin_name, 'modules')) ? 'modules' : 'repositories'));
$sAction = ((isset($_REQUEST['Action'])) ? $_REQUEST['Action'] : 'List');
$sFilter = ((isset($_REQUEST['Filter'])) ? $_REQUEST['Filter'] : '');
if (strlen($sFilter)) {
    $aFilter = explode(',', $sFilter);
    $sFilter = '';
    for ($i = 0, $n = count($aFilter); $i < $n; $i ++) {
        if (strlen(trim($aFilter[$i]))) {
            $sFilter .= strtolower(trim($aFilter[$i])) . ',';
        }
    }
    $sFilter = substr($sFilter, 0, -1);
}

switch ($sWhat) {
    case 'modules':
        if ($perm->have_perm_area_action($plugin_name, 'modules')) {
            $sScrollScript = '
<script>
    setTimeout(function() {
        $("html, body").animate({scrollTop: $(document).height() + "px"}, 2000);
    }, 100);
</script>';
            $sReloadScript = '
<p>' . i18n("(auto redirect in 5 seconds)") . '</p>
<script>
    setTimeout(function() {
        document.location.href = document.location.href' . ((strlen($_REQUEST['Filter'])) ? ' + "&Filter=' . $_REQUEST['Filter'] . '"' : '') . ';
    }, 5000);
</script>';

            switch ($sAction) {
                case 'List':

                    # Output the top HTML first
                    $sHTML = file_get_contents($cfg['path']['contenido'] . $cfg['path']['plugins'] . 'mom/templates/default/page.html');
                    $p1 = strpos($sHTML, '{CONTENT}');
                    echo substr($sHTML, 0, $p1) . str_repeat(" ", (65537 * 2)) . "\n";
                    flush();

                    # Set fixed informations
                    $oTpl->set('s', 'FORM_ACTION', 'main.php?area=mom&frame=4&contenido=' . $contenido);
                    $oTpl->set('s', 'TEXT_INSTALLED', i18n("Installed modules"));
                    $oTpl->set('s', 'TEXT_FILTER', i18n("Filter"));
                    $oTpl->set('s', 'TEXT_FILTER_DESCRIPTION', i18n("(Comma separated list of tags to search for)"));
                    $oTpl->set('s', 'TEXT_DO_FILTER', i18n("Use filter"));
                    $oTpl->set('s', 'TEXT_CLEAR_FILTER', i18n("Clear filter"));
                    $oTpl->set('s', 'TEXT_AVAILABLE', i18n("Available modules"));
                    $oTpl->set('s', 'TEXT_UNINSTALL', i18n("Uninstall"));
                    $oTpl->set('s', 'TEXT_DUPLICATE', i18n("Duplicate"));
                    $oTpl->set('s', 'TEXT_NAME', i18n("Module"));
                    $oTpl->set('s', 'TEXT_DESCRIPTION', i18n("Description"));
                    $oTpl->set('s', 'TEXT_TAGS', i18n("Tags"));
                    $oTpl->set('s', 'TEXT_AUTHOR', i18n("Author"));
                    $oTpl->set('s', 'TEXT_COPYRIGHT', i18n("Copyright"));
                    $oTpl->set('s', 'TEXT_VERSION', i18n("Version"));
                    $oTpl->set('s', 'TEXT_DATE', i18n("Date"));
                    
                    $oTpl->set('s', 'FILTER', $_REQUEST['Filter']);
                    
                    # Get installed modules
                    $aModulesI = array();
                    
                    # First list all modules for the client from the DB table
                    $sql = 'SELECT `idmod`, `name`, `description`, `install_dir`, `modified`
                            FROM `' . $cfg['tab']['mod'] . '`
                            WHERE (`idclient`=' . $client . ')
                            ORDER BY `name` ASC';
                    $db->query($sql);
                    while ($db->next_record()) {
                        $sDescription = $db->f('description');
                        $p1 = strpos($sDescription, "\n");
                        if ($p1 !== false) {
                            $sDescription = substr($sDescription, 0, $p1);
                        }
                        $oModule = new cApiModule($db->f('idmod'));
                        $bInUse = $oModule->moduleInUse($db->f('idmod'));
                        unset($oModule);
                        $aModulesI[$db->f('name')] = array('ID' => $db->f('idmod'), 'Name' => $db->f('name'), 'Description' => array('en_US' => $sDescription), 'InstallDir' => $db->f('install_dir'), 'InUse' => $bInUse, 'Modified' => (bool) $db->f('modified'));
                        if (strlen($db->f('install_dir'))) {
                            $aModulesIKeys[] = $db->f('install_dir');
                        }
                    }
                    
                    # Then get the module info's from their module.xml files
                    $oModule = new ModuleInfo();
                    foreach ($aModulesI as $key => $value) {
                        $oModule->moduleName($value['InstallDir']);
                        if (($aInfo = $oModule->getInfo()) !== false) {
                            $sName = $value['Name'];
                            $aModulesI[$key] = array_merge($value, $aInfo);
                            $aModulesI[$key]['Name'] = $sName;
                        }
                    }
                    
                    # List installed modules
                    $sClass = 'odd';
                    foreach ($aModulesI as $key => $value) {
                        if (count($value)) {
                            $oTpl->set('d', 'CLASS', $sClass);
                            $oTpl->set('d', 'KEY', $value['InstallDir']);
                            $oTpl->set('d', 'ID', $value['ID']);
                            $oTpl->set('d', 'NAME', mb_convert_encoding($value['Name'], $encoding[$lang], 'UTF-8'));
                            $oTpl->set('d', 'CLEARNAME', mb_convert_encoding($key, $encoding[$lang], 'UTF-8'));
                            $sDescription = '';
                            if (strlen($value['Description'][$belang])) {
                                $sDescription = $value['Description'][$belang];
                            }
                            else {
                                foreach ($value['Description'] as $cult => $desc) {
                                    if (substr($sCult, 0, strpos($sCult, '_')) == $sLanguage) {
                                        $sDescription = $desc;
                                        break;
                                    }
                                }
                            }
                            $oTpl->set('d', 'DESCRIPTION', htmlentities(mb_convert_encoding(((strlen($sDescription)) ? $sDescription : $value['Description']['en_US']), $encoding[$lang], 'UTF-8'), ENT_QUOTES, $encoding[$lang]));
                            $aTags = array();
                            if (count($value['Tags'][$belang])) {
                                $aTags = $value['Tags'][$belang];
                            }
                            else {
                                foreach ($value['Tags'] as $cult => $desc) {
                                    if (substr($cult, 0, strpos($cult, '_')) == $sLanguage) {
                                        $aTags[] = $desc;
                                    }
                                }
                            }
                            if (is_array($aTags[0])) {
                                $aTags = $aTags[0];
                            }
                            $oTpl->set('d', 'TAGS', mb_convert_encoding(implode(', ', $aTags), $encoding[$lang], 'UTF-8'));
                            if (strlen($value['Author'])) {
                                $oTpl->set('d', 'AUTHOR', mb_convert_encoding($value['Author'], $encoding[$lang], 'UTF-8'));
                            }
                            else {
                                $oTpl->set('d', 'AUTHOR', '<span style="color: lightgray;">' . i18n("unknown") . '</span>');
                            }
                            if (strlen($value['Copyright'])) {
                                $oTpl->set('d', 'COPYRIGHT', mb_convert_encoding($value['Copyright'], $encoding[$lang], 'UTF-8'));
                            }
                            else {
                                $oTpl->set('d', 'COPYRIGHT', '<span style="color: lightgray;">' . i18n("unknown") . '</span>');
                            }
                            if (strlen($value['Version'])) {
                                $oTpl->set('d', 'VERSION', $value['Version']);
                            }
                            else {
                                $oTpl->set('d', 'VERSION', '<span style="color: lightgray;">' . i18n("unknown") . '</span>');
                            }
                            if (strlen($value['Date'])) {
                                $oTpl->set('d', 'DATE', date('Y-m-d', strtotime(mb_convert_encoding($value['Date'], $encoding[$lang], 'UTF-8'))));
                            }
                            else {
                                $oTpl->set('d', 'DATE', '<span style="color: lightgray;">' . i18n("unknown") . '</span>');
                            }
                            if ($value['InUse']) {
                                $oTpl->set('d', 'IMG_CLASS', '');
                                $oTpl->set('d', 'IMAGE', 'exclamation');
                                $oTpl->set('d', 'TEXT_UNINSTALL', sprintf(i18n("%s can't be uninstalled because it is used"), $value['Name']));
                            }
                            else {
                                $oTpl->set('d', 'IMG_CLASS', 'remove');
                                $oTpl->set('d', 'IMAGE', 'but_delete');
                                $oTpl->set('d', 'TEXT_UNINSTALL', sprintf(((strlen($sModules)) ? i18n("Uninstall %s including it's modules (all clients)") : i18n("Uninstall %s")), $key));
                            }
                            $sClass = (($sClass == 'odd') ? 'even' : 'odd');
                            $oTpl->next();
                        }
                    }
                    $oTpl->set('s', 'SHOW_MODULES', 'false');
                    
                    # Generate list template
                    echo $oTpl->generate($cfg['path']['contenido'] . $cfg['path']['plugins'] . $plugin_name . '/templates/default/list.html', true);
                    flush();
                    $oTpl->reset();
                    
                    # Set fixed informations
                    $oTpl->set('s', 'FORM_ACTION', 'main.php?area=mom&frame=4&contenido=' . $contenido);
                    $oTpl->set('s', 'TEXT_INSTALL', i18n("Install"));
                    $oTpl->set('s', 'TEXT_NAME', i18n("Module"));
                    $oTpl->set('s', 'TEXT_DESCRIPTION', i18n("Description"));
                    $oTpl->set('s', 'TEXT_TAGS', i18n("Tags"));
                    $oTpl->set('s', 'TEXT_AUTHOR', i18n("Author"));
                    $oTpl->set('s', 'TEXT_COPYRIGHT', i18n("Copyright"));
                    $oTpl->set('s', 'TEXT_VERSION', i18n("Version"));
                    $oTpl->set('s', 'TEXT_DATE', i18n("Date", $plugin_nam));
                    $oTpl->set('s', 'TEXT_DEPENDENCIES', i18n("Dependencies"));
                    $oTpl->set('s', 'TEXT_REPOSITORY', i18n("Repository"));
                    
                    $oTpl->set('s', 'FILTER', $_REQUEST['Filter']);
                    
                    # Get available modules from https://www.dzir.org/cms/modules/
                    $sUrl = 'https://www.dzir.org/cms/modules/' . ((strlen($sFilter)) ? '?Filter=' . urlencode($sFilter) : '');
                    $sDownloadUrl = 'https://www.dzir.org/cms/modules/';
                    $iErrNo = 0;
                    $sErrMsg = '';
                    $aModulesA = json_decode(getRemoteContent($sUrl, $iErrNo, $sErrMsg), true);
                    foreach ($aModulesA as $key => $value) {
                        $aModulesA[$key]['Url'] = $sDownloadUrl;
                        $aModulesA[$key]['Repo'] = 'DZIRCMS';
                    }
                    
                    # Get available modules from additionally added repositories
                    $aReposInfo = getPropertiesByItemtype('module_repository', 'itemid');
                    $aRepos = array();
                    for ($i = 0, $n = count($aReposInfo); $i < $n; $i ++) {
                        $aRepos[$aReposInfo[$i]['itemid']][$aReposInfo[$i]['name']] = $aReposInfo[$i]['value'];
                    }
                    foreach ($aRepos as $itemid => $server) {
                        $oTpl->set('s', 'SHOW_REPOSITORY', '1');
                        $sUrl = $server['host'] . ((strlen($sFilter)) ? '?Filter=' . urlencode($sFilter) : '');
                        $sDownloadUrl = $server['host'];
                        $sLogin = $server['login'];
                        $sPassword = $server['password'];
                        $iErrNo = 0;
                        $sErrMsg = '';
                        $aModulesO = json_decode(getRemoteContent($sUrl, $iErrNo, $sErrMsg, $sLogin, $sPassword), true);
                        foreach ($aModulesO as $key => $value) {
                            $aModulesA[$key . '~~' . $itemid] = $value;
                            $aModulesA[$key . '~~' . $itemid]['Url'] = $sDownloadUrl;
                            $aModulesA[$key . '~~' . $itemid]['Repo'] = $itemid;
                        }
                    }
                    $aModulesA = array_csort($aModulesA, 'Name', SORT_STRING, SORT_ASC);
                    $oTpl->set('s', 'SHOW_REPOSITORY', '');
                    
                    # List available modules
                    $sClass = 'odd';
                    foreach ($aModulesA as $key => $value) {
                        $bAdditional = (strpos($key, '~~') !== false);
                        $key = ((strpos($key, '~~') !== false) ? substr($key, 0, strpos($key, '~~')) : $key);
                        $bInArray = in_array($key, $aModulesIKeys);
                        $sVersion = '';
                        if ($bInArray) {
                            $oModule->moduleName($key);
                            if (($aInfo = $oModule->getInfo()) !== false) {
                                $sVersion = $aInfo['Version'];
                            }
                        }
                        if ((!$bInArray) || (version_compare($value['Version'], $sVersion, '>'))) {
                            $oTpl->set('d1', 'CLASS', $sClass);
                            $oTpl->set('d1', 'KEY', $key);
                            $oTpl->set('d1', 'URL', $value['Url']);
                            $oTpl->set('d1', 'NAME', mb_convert_encoding($value['Name'], $encoding[$lang], 'UTF-8'));
                            $sDescription = '';
                            if (strlen($value['Description'][$belang])) {
                                $sDescription = htmlentities($value['Description'][$belang], ENT_QUOTES, 'UTF-8');
                            }
                            else {
                                foreach ($value['Description'] as $cult => $desc) {
                                    if (substr($sCult, 0, strpos($sCult, '_')) == $sLanguage) {
                                        $sDescription = htmlentities($desc, ENT_QUOTES, 'UTF-8');
                                        break;
                                    }
                                }
                            }
                            $oTpl->set('d1', 'DESCRIPTION', mb_convert_encoding(((strlen($sDescription)) ? $sDescription : $value['Description']['en_US']), $encoding[$lang], 'UTF-8'));
                            $aTags = array();
                            if (count($value['Tags'][$belang])) {
                                $aTags = $value['Tags'][$belang];
                            }
                            else {
                                foreach ($value['Tags'] as $cult => $desc) {
                                    if (substr($cult, 0, strpos($cult, '_')) == $sLanguage) {
                                        $aTags[] = $desc;
                                    }
                                }
                            }
                            if (is_array($aTags[0])) {
                                $aTags = $aTags[0];
                            }
                            $oTpl->set('d1', 'TAGS', mb_convert_encoding(implode(', ', $aTags), $encoding[$lang], 'UTF-8'));
                            $oTpl->set('d1', 'AUTHOR', mb_convert_encoding($value['Author'], $encoding[$lang], 'UTF-8'));
                            $oTpl->set('d1', 'COPYRIGHT', mb_convert_encoding($value['Copyright'], $encoding[$lang], 'UTF-8'));
                            $oTpl->set('d1', 'VERSION', mb_convert_encoding($value['Version'], $encoding[$lang], 'UTF-8'));
                            if (strlen($value['Date'])) {
                                $oTpl->set('d1', 'DATE', date('Y-m-d', strtotime(mb_convert_encoding($value['Date'], $encoding[$lang], 'UTF-8'))));
                            }
                            else {
                                $oTpl->set('d1', 'DATE', '<span style="color: lightgray;">' . i18n("unknown") . '</span>');
                            }
                            if ((version_compare($value['Requirements']['php'], phpversion(), '>')) && ((version_compare($value['Requirements']['dzircms'], $cfg['version'], '>')) || (version_compare($value['Requirements']['drugcms'], $cfg['version'], '>')))) {
                                $oTpl->set('d1', 'CLASS', 'old');
                                $oTpl->set('d1', 'TEXT_INSTALL', sprintf(i18n("Old PHP version and old CMS version, installation not possible"), $value['Name']));
                                $oTpl->set('d1', 'IMG_CLASS', 'blocked');
                                $oTpl->set('d1', 'IMAGE', 'icon_warning');
                            }
                            elseif (version_compare($value['Requirements']['php'], phpversion(), '>')) {
                                $oTpl->set('d1', 'CLASS', 'old');
                                $oTpl->set('d1', 'TEXT_INSTALL', sprintf(i18n("Old PHP version, installation not possible"), $value['Name']));
                                $oTpl->set('d1', 'IMG_CLASS', 'blocked');
                                $oTpl->set('d1', 'IMAGE', 'icon_warning');
                            }
                            elseif (version_compare($value['Requirements']['dzircms'], $cfg['version'], '>')) {
                                $oTpl->set('d1', 'CLASS', 'old');
                                $oTpl->set('d1', 'TEXT_INSTALL', sprintf(i18n("Old CMS version, installation not possible"), $value['Name']));
                                $oTpl->set('d1', 'IMG_CLASS', 'blocked');
                                $oTpl->set('d1', 'IMAGE', 'icon_warning');
                            }
                            elseif (version_compare($value['Requirements']['drugcms'], $cfg['version'], '>')) {
                                $oTpl->set('d1', 'CLASS', 'old');
                                $oTpl->set('d1', 'TEXT_INSTALL', sprintf(i18n("Old CMS version, installation not possible"), $value['Name']));
                                $oTpl->set('d1', 'IMG_CLASS', 'blocked');
                                $oTpl->set('d1', 'IMAGE', 'icon_warning');
                            }
                            elseif (!array_key_exists($key, $aModulesI)) {
                                $oTpl->set('d1', 'CLASS', $sClass);
                                $oTpl->set('d1', 'TEXT_INSTALL', sprintf(i18n("Install %s"), $value['Name']));
                                $oTpl->set('d1', 'IMG_CLASS', 'install');
                                $oTpl->set('d1', 'IMAGE', 'importieren');
                            }
                            elseif (version_compare($value['Version'], $aModulesI[$key]['Version'], '>')) {
                                $oTpl->set('d1', 'CLASS', 'new');
                                $oTpl->set('d1', 'TEXT_INSTALL', sprintf(i18n("Update %s"), $value['Name']));
                                $oTpl->set('d1', 'IMG_CLASS', 'install');
                                $oTpl->set('d1', 'IMAGE', 'importieren');
                            }
                            $oTpl->set('d1', 'REPOSITORY', mb_convert_encoding($value['Repo'], $encoding[$lang], 'UTF-8'));
                            $sClass = (($sClass == 'odd') ? 'even' : 'odd');
                            $oTpl->next(1);
                        }
                    }
                    
                    # Generate list template
                    echo $oTpl->generate($cfg['path']['contenido'] . $cfg['path']['plugins'] . $plugin_name . '/templates/default/list_available.html', true);
                    flush();
                    $oTpl->reset();
                    break;
                
                case 'Install':
                    $sModule = $_POST['Key'];
                    $sUrl = $_POST['Url'];
                    $sPath = $cfgClient[$client]['path']['frontend'] . 'data/modules_c/' . $sModule . '/';
                    $bUpdate = is_dir($sPath);
                    #echo '<pre>$sModule = ' . $sModule . '</pre>';
                    #echo '<pre>$sPath = ' . $sPath . '</pre>';
                    
                    switch ((int) $_POST['getStep']) {
                        case 0:

                            # Output the top HTML first
                            $sHTML = file_get_contents($cfg['path']['contenido'] . $cfg['path']['plugins'] . 'mom/templates/default/page.html');
                            $p1 = strpos($sHTML, '{CONTENT}');
                            echo substr($sHTML, 0, $p1) . str_repeat(" ", (65537 * 2)) . "\n";
                            flush();

                            echo $sScrollScript;
                            echo '
</div>
<p id="Wait">&bull;....</p>
<script>
    var step = 0;
    var tmr = setInterval(function() {
        switch (step) {
            case 0:
                cnt = ".&bull;...";
                step = 1;
                break;
            case 1:
                cnt = "..&bull;..";
                step = 2;
                break;
            case 2:
                cnt = "...&bull;.";
                step = 3;
                break;
            case 3:
                cnt = "....&bull;";
                step = 4;
                break;
            case 4:
                cnt = "...&bull;.";
                step = 5;
                break;
            case 5:
                cnt = "..&bull;..";
                step = 6;
                break;
            case 6:
                cnt = ".&bull;...";
                step = 7;
                break;
            case 7:
                cnt = "&bull;....";
                step = 0;
                break;
        }
        $("#Wait").html(cnt);
    }, 150);
    function getStep(i) {
        $.post(
            "' . $_SERVER['REQUEST_URI'] . '",
            {';
                            foreach ($_POST as $key => $value) {
                                if ($key != 'getStep') {
                                    echo '
                ' . $key . ': "' . $value . '",';
                                }
                            }
                            echo '
                getStep: i
            },
            function(data) {
                $("#Content").append(data);
            }
        );
    }
    getStep(1);
</script>
<body>
</html>';
                            break;
                        case 1:

                            # Download the module
                            $bOK = false;
                            if (intval(getProperty(0, 'system', 'setting', 'ftp', 'use'))) {
                                $bOK = true;
                                
                                # Get the FTP access data
                                $sFTPServer         = getProperty(0, 'system', 'setting', 'ftp', 'server');
                                $iFTPPort           = intval(getProperty(0, 'system', 'setting', 'ftp', 'port'));
                                $sFTPProtocol       = getProperty(0, 'system', 'setting', 'ftp', 'protocol');
                                $iFTPPort           = (($iFTPPort) ? $iFTPPort : (($sFTPProtocol == 'sftp') ? 22 : 21));
                                $sFTPUser           = getProperty(0, 'system', 'setting', 'ftp', 'user');
                                $sFTPPassword       = getProperty(0, 'system', 'setting', 'ftp', 'password');
                                $sBackendFTPPath    = getProperty(0, 'system', 'setting', 'ftp', 'backend_path');
                                $sFrontendFTPPath   = getProperty(0, 'system', 'setting', 'ftp', 'frontend_path');
                                
                                # Establish the FTP connection
                                switch ($sFTPProtocol) {
                                    case 'ftp':
                                        $ftp = new ftp($sFTPServer, $iFTPPort, $sFTPUser, $sFTPPassword, $sFrontendFTPPath);
                                        break;
                                    case 'sftp':
                                        $ftp = new sftp($sFTPServer, $iFTPPort, $sFTPUser, $sFTPPassword, $sFrontendFTPPath);
                                        break;
                                }
                                if (!$ftp) {
                                    echo $notification->returnNotification("error", sprintf(i18n("Unable to create a (S)FTP connection to %s"), $sFTPServer));
                                    flush();
                                    $bOK = false;
                                    break;
                                }
                                echo '<p>' . i18n("FTP connection established") . '</p>';
                                flush();
                                
                                # Determin the modules path under the FTP root path
                                $sPath = $sFrontendFTPPath . str_replace($cfg['path']['frontend'], '', $cfgClient[$client]['path']['frontend']) . 'data/modules_c/' . $sModule . '/';
                                
                                # Create the folder for the new module
                                if (!$ftp->mkdir($sPath, 0775, true)) {
                                    echo $notification->returnNotification("error", sprintf(i18n("Unable to create the module's folder"), $sFTPServer));
                                    flush();
                                    $bOK = false;
                                    unset($ftp);
                                    break;
                                }
                                
                                # Download the module
                                if (class_exists('ZipArchive')) {
                                    if (getRemoteContentToFile($sUrl . '?Module=' . $sModule . '&Action=GetZip', $cfg['path']['contenido'] . 'data/cache/' . $sModule . '.zip', $iErrNo, $sErrMsg, $sLogin, $sPassword)) {
                                        echo '<p>' . i18n("Zip file downloaded") . '</p>';
                                        
                                        # Extract the zip file to it's folder
                                        $zip = new ZipArchive();
                                        if ($zip->open($cfg['path']['contenido'] . 'data/cache/' . $sModule . '.zip')) {
                                            mkdir($cfg['path']['contenido'] . 'data/cache/' . $sModule, 0775, true);
                                            $zip->extractTo($cfg['path']['contenido'] . 'data/cache/' . $sModule);
                                            $zip->close();
                                        }
                                        echo '<p>' . i18n("Zip file extracted") . '</p>';
                                        flush();
                                        $bOK = true;
                                        
                                        # Delete the zip file
                                        unlink($cfg['path']['contenido'] . 'data/cache/' . $sModule . '.zip');
                                        
                                        # Upload all files per FTP
                                        if ($ftp->putAll($cfg['path']['contenido'] . 'data/cache/' . $sModule . '/', $sPath)) {
                                            echo '<p>' . i18n("Files placed per FTP") . '</p>';
                                            flush();
                                            $bOK = true;
                                        }
                                        else {
                                            echo $notification->returnNotification("error", i18n("Unable to write files per FTP"));
                                            flush();
                                            removeDir($cfg['path']['contenido'] . 'data/cache/' . $sModule);
                                            $ftp->rmdir($sPath, true);
                                            $bOK = false;
                                            break;
                                        }
                                    }
                                    else {
                                        echo $notification->returnNotification("error", i18n("Failed to download zip file")) . '</pre>';
                                        flush();
                                    }
                                }
                                if (!$bOK) {
                                    # Get the folders and files
                                    $aFoldersAndFiles = json_decode(getRemoteContent($sUrl . '?Module=' . $sModule . '&Action=Files', $iErrNo, $sErrMsg, $sLogin, $sPassword));
                                    $iDone = 0;
                                    for ($i = 0, $n = count($aFoldersAndFiles); $i < $n; $i ++) {
                                        if (substr($aFoldersAndFiles[$i], -1) == '/') {
                                            if ($ftp->mkdir($sPath . $aFoldersAndFiles[$i], 0775, true)) {
                                                echo '<p>' . sprintf(i18n("Folder %s created"), $aFoldersAndFiles[$i]) . '</p>';
                                                flush();
                                            }
                                            else {
                                                echo $notification->returnNotification("error", sprintf(i18n("Unable to create folder %s"), $aFoldersAndFiles[$i]));
                                                flush();
                                                $bOK = false;
                                                break;
                                            }
                                        }
                                        else {
                                            if (getRemoteContentToFile($sUrl . '?Module=' . $sModule . '&Action=GetFile&File=' . urlencode($aFoldersAndFiles[$i]), $cfg['path']['contenido'] . 'data/cache/' . basename($aFoldersAndFiles[$i]), $iErrNo, $sErrMsg, $sLogin, $sPassword)) {
                                                if (!is_file($cfg['path']['contenido'] . 'data/cache/' . basename($aFoldersAndFiles[$i]))) {
                                                    echo $notification->returnNotification("error", sprintf(i18n("Unable to write temporary file %s in the DZIRCMS backend cache folder (data/cache/)"), $aFoldersAndFiles[$i]));
                                                    flush();
                                                    $bOK = false;
                                                    break;
                                                }
                                                if (!$ftp->put($sPath . $aFoldersAndFiles[$i], $cfg['path']['contenido'] . 'data/cache/' . basename($aFoldersAndFiles[$i]))) {
                                                    echo $notification->returnNotification("error", sprintf(i18n("Unable to write file %s per FTP"), $aFoldersAndFiles[$i]));
                                                    flush();
                                                    unlink($cfg['path']['contenido'] . 'data/cache/' . basename($aFoldersAndFiles[$i]));
                                                    $bOK = false;
                                                    break;
                                                }
                                                else {
                                                    unlink($cfg['path']['contenido'] . 'data/cache/' . basename($aFoldersAndFiles[$i]));
                                                }
                                                echo '<p>' . sprintf(i18n("File %s downloaded"), $aFoldersAndFiles[$i]) . '</p>';
                                                flush();
                                            }
                                            else {
                                                echo $notification->returnNotification("error", $sErrMsg);
                                                flush();
                                                $bOK = false;
                                                break;
                                            }
                                        }
                                        $iDone ++;
                                    }
                                }
                                if (!$bOK) {
                                    
                                    # Rollback the already done actions (create folders, put files)
                                    echo '<p>' . i18n("Rolling back already transfered folders and files") . '</p>';
                                    flush();
                                    if (!$ftp->rmdir($sPath, true)) {
                                        echo $notification->returnNotification("error", sprintf(i18n("Unable to remove the folder for %s"), $sModule));
                                        flush();
                                    }
                                }
                                
                                # Close the FTP connection
                                $ftp->quit();
                            }
                            elseif (($bUpdate) || (mkdir($sPath, 0775, true))) {
                                if (class_exists('ZipArchive')) {
                                    if (getRemoteContentToFile($sUrl . '?Module=' . $sModule . '&Action=GetZip', $sPath . $sModule . '.zip', $iErrNo, $sErrMsg, $sLogin, $sPassword)) {
                                        echo '<p>' . i18n("Zip file downloaded") . '</p>';
                                        flush();
                                        
                                        # Extract the zip file to it's folder
                                        $zip = new ZipArchive();
                                        if ($zip->open($sPath . $sModule . '.zip')) {
                                            $zip->extractTo($sPath);
                                            $zip->close();
                                        }
                                        echo '<p>' . i18n("Zip file extracted") . '</p>';
                                        flush();
                                        $bOK = true;
                                        
                                        # Delete the zip file
                                        unlink($sPath . $sModule . '.zip');
                                    }
                                    else {
                                        echo $notification->returnNotification("error", i18n("Failed to download zip file")) . '</p>';
                                        flush();
                                    }
                                }
                                if (!$bOK) {
                                    $bOK = true;
                                    $aFoldersAndFiles = json_decode(getRemoteContent($sUrl . '?Module=' . $sModule . '&Action=Files', $iErrNo, $sErrMsg, $sLogin, $sPassword));
                                    for ($i = 0, $n = count($aFoldersAndFiles); $i < $n; $i ++) {
                                        if (substr($aFoldersAndFiles[$i], -1) == '/') {
                                            if (mkdir($sPath . $aFoldersAndFiles[$i], 0775, true)) {
                                                echo '<p>' . sprintf(i18n("Folder %s created"), $aFoldersAndFiles[$i]) . '</p>';
                                                flush();
                                            }
                                            else {
                                                echo $notification->returnNotification("error", sprintf(i18n("Unable to create folder %s"), $aFoldersAndFiles[$i]));
                                                flush();
                                                $bOK = false;
                                                break;
                                            }
                                        }
                                        else {
                                            if (getRemoteContentToFile($sUrl . '?Module=' . $sModule . '&Action=GetFile&File=' . urlencode($aFoldersAndFiles[$i]), $sPath . $aFoldersAndFiles[$i], $iErrNo, $sErrMsg, $sLogin, $sPassword)) {
                                                echo '<p>' . sprintf(i18n("File %s downloaded"), $aFoldersAndFiles[$i]) . '</p>';
                                                flush();
                                            }
                                            else {
                                                echo $notification->returnNotification("error", $sErrMsg);
                                                flush();
                                                $bOK = false;
                                                break;
                                            }
                                        }
                                    }
                                }
                            }
                            if ($bOK) {
                                
                                # Check if the module.xml file is present (if not, file and folder rights permit PHP to write files)
                                if (!is_file($sPath . 'module.xml')) {
                                    echo $notification->returnNotification("error", i18n("Module manager is not allowed to write files and folders directly, please switch to the Settings tab and enter your data to use FTP instead."));
                                    flush();
                                    $bOK = false;
                                }
                            }
                            if (!(($bOK) || ($bUpdate))) {
                                
                                # Delete the created folder, as it would show the non-installed module as update
                                removeDir($sPath);
                            }
                            if ($bOK) {
                                echo '<script>getStep(2);</script>';
                            }
                            break;
                        case 2:

                            # Install the module
                            if (installModule($sModule, $bUpdate)) {
                                echo $notification->returnNotification('info', sprintf(i18n("Module %s successfully installed"), $sModule));
                                echo '<script>getStep(3);</script>';
                            }
                            else {
                                echo '<p>' . i18n("Rolling back already transfered folders and files") . '</p>';
                                flush();
                                if (intval(getProperty(0, 'system', 'setting', 'ftp', 'use'))) {
                                    
                                    # Get the FTP access data
                                    $sFTPServer         = getProperty(0, 'system', 'setting', 'ftp', 'server');
                                    $iFTPPort           = intval(getProperty(0, 'system', 'setting', 'ftp', 'port'));
                                    $sFTPProtocol       = getProperty(0, 'system', 'setting', 'ftp', 'protocol');
                                    $iFTPPort           = (($iFTPPort) ? $iFTPPort : (($sFTPProtocol == 'sftp') ? 22 : 21));
                                    $sFTPUser           = getProperty(0, 'system', 'setting', 'ftp', 'user');
                                    $sFTPPassword       = getProperty(0, 'system', 'setting', 'ftp', 'password');
                                    $sBackendFTPPath    = getProperty(0, 'system', 'setting', 'ftp', 'backend_path');
                                    $sFrontendFTPPath   = getProperty(0, 'system', 'setting', 'ftp', 'frontend_path');
                                    
                                    # Establish the FTP connection
                                    switch ($sFTPProtocol) {
                                        case 'ftp':
                                            $ftp = new ftp($sFTPServer, $iFTPPort, $sFTPUser, $sFTPPassword, $sFrontendFTPPath);
                                            break;
                                        case 'sftp':
                                            $ftp = new sftp($sFTPServer, $iFTPPort, $sFTPUser, $sFTPPassword, $sFrontendFTPPath);
                                            break;
                                    }
                                    if (!$ftp) {
                                        echo $notification->returnNotification("error", sprintf(i18n("Unable to create a (S)FTP connection to %s"), $sFTPServer));
                                    }
                                    elseif (!$ftp->rmdir($sPath, true)) {
                                        echo $notification->returnNotification("error", sprintf(i18n("Unable to remove the folder for %s"), $sModule));

                                        # Close the FTP connection
                                        $ftp->quit();
                                    }
                                }
                                else {
                                    removeDir($sPath);
                                }
                                echo $notification->returnNotification("error", sprintf(i18n("Unable to install %s, missing install information"), $sModule));
                                flush();
                            }
                            break;
                        case 3:
                            
                            # Copy additional files
                            $oModule = new ModuleInfo($sModule);
                            $aFoldersAndFiles = $oModule->getAdditionalFoldersAndFiles();
                            if (count($aFoldersAndFiles)) {
                                echo '<p>' . i18n("Copying additional files") . '</p>';
                                if (($ret = array_search('php/', $aFoldersAndFiles)) !== false) {

                                    # New version module, all files and folders belong in the client's /data/modules/<module> folder
                                    $sModulePath = $cfgClient[$client]['path']['frontend'] . 'data/modules/' . $sModule . '/';
                                    mkdir($sModulePath, 0775);
                                    foreach ($aFoldersAndFiles as $entry) {
                                        if (substr($entry, -1) == '/') {
                                            
                                            # Entry is a folder, create it
                                            mkdir($sModulePath . $entry, 0775, true);
                                        }
                                        else {
                                            
                                            # Entry is a file, copy it
                                            copy($sPath . $entry, $sModulePath . $entry);
                                        }
                                    }
                                }
                                else {

                                    # Old version module, additional files and folders belong in the client's folder
                                    foreach ($aFoldersAndFiles as $entry) {
                                        if (substr($entry, -1) == '/') {
                                            
                                            # Entry is a folder, create it
                                            mkdir(addDirSep($cfgClient[$client]['path']['frontend']) . $entry, 0775, true);
                                        }
                                        else {
                                            
                                            # Entry is a file, copy it
                                            copy($sPath . $entry, addDirSep($cfgClient[$client]['path']['frontend']) . $entry);
                                        }
                                    }
                                }
                            }
                            echo $sReloadScript;
                            echo '
<script>
    clearInterval(tmr);
    $("#Wait").html("");
</script>';
                            break;
                    }
                    break;
                
                case 'Duplicate':

                    # Output the top HTML first
                    $sHTML = file_get_contents($cfg['path']['contenido'] . $cfg['path']['plugins'] . 'mom/templates/default/page.html');
                    $p1 = strpos($sHTML, '{CONTENT}');
                    echo substr($sHTML, 0, $p1) . str_repeat(" ", (65537 * 2)) . "\n";
                    flush();

                    $oMod1 = new cApiModule((int) $_POST['ID']);
                    $oMC = new cApiModuleCollection();
                    $sName = $_POST['Name'] . ' (x)';
                    $iCopyNo = 0;
                    for ($i = 1; $i < 1000; $i ++) {
                        $sql = 'SELECT `idmod`
                                FROM `' . $cfg['tab']['mod'] . '`
                                WHERE ((`idclient`=' . $client . ')
                                   AND (`name`="' . $_POST['Name'] . ' (' . $i . ')"))';
                        $db->query($sql);
                        if ($db->num_rows() == 0) {
                            $sName = $_POST['Name'] . ' (' . $i . ')';
                            $iCopyNo = $i;
                            break;
                        }
                    }
                    $oMod2 = $oMC->create($sName);
                    $oMod2->set('type', $oMod1->get('type'));
                    $oMod2->set('error', $oMod1->get('error'));
                    $oMod2->set('description', $oMod1->get('description'));
                    $oMod2->set('input', $oMod1->get('input'));
                    $oMod2->set('output', $oMod1->get('output'));
                    $oMod2->set('template', $oMod1->get('template'));
                    $oMod2->set('static', $oMod1->get('static'));
                    $oMod2->set('package_guid', $oMod1->get('package_guid'));
                    $oMod2->set('package_data', $oMod1->get('package_data'));
                    $oMod2->set('author', $oMod1->get('author'));
                    $oMod2->set('orig_name', $oMod1->get('orig_name'));
                    $oMod2->set('install_dir', $oMod1->get('install_dir'));
                    $oMod2->set('modified', $oMod1->get('modified'));
                    $oMod2->set('hide_empty_translations', $oMod1->get('hide_empty_translations'));
                    $oMod2->store(false, true);

                    # Copy all module files if they are in the module's folder
                    $sModule = $_POST['Key'];
                    cInclude('includes', 'functions.upl.php');
                    if (is_dir($cfgClient[$client]['path']['frontend'] . 'data/modules/' . $sModule)) {
                        $sNewModule = uplCreateFriendlyName($sName);
                        mkdir($cfgClient[$client]['path']['frontend'] . 'data/modules/' . $sNewModule, 0775, true);
                        copyr($cfgClient[$client]['path']['frontend'] . 'data/modules/' . $sModule, $cfgClient[$client]['path']['frontend'] . 'data/modules/' . $sNewModule);

                        # Rename the php files to fit the new name of the copied module
                        rename($cfgClient[$client]['path']['frontend'] . 'data/modules/' . $sNewModule . '/php/' . $sModule . '-input.php', $cfgClient[$client]['path']['frontend'] . 'data/modules/' . $sNewModule . '/php/' . $sModule . '_' . $iCopyNo . '-input.php');
                        rename($cfgClient[$client]['path']['frontend'] . 'data/modules/' . $sNewModule . '/php/' . $sModule . '-output.php', $cfgClient[$client]['path']['frontend'] . 'data/modules/' . $sNewModule . '/php/' . $sModule . '_' . $iCopyNo . '-output.php');
                    }
                    echo $notification->returnNotification('info', sprintf(i18n("Module %s successfully duplicated to %s"), $_POST['Name'], $sName));
                    echo $sReloadScript;
                    break;
                    
                case 'Uninstall':

                    # Output the top HTML first
                    $sHTML = file_get_contents($cfg['path']['contenido'] . $cfg['path']['plugins'] . 'mom/templates/default/page.html');
                    $p1 = strpos($sHTML, '{CONTENT}');
                    echo substr($sHTML, 0, $p1) . str_repeat(" ", (65537 * 2)) . "\n";
                    flush();

                    echo $sScrollScript;
                    flush();
                    $sModule = $_POST['Key'];
                    $iID = (int) $_POST['ID'];
                    $sName = $_POST['Name'];
                    
                    # Delete the module with it's properties and cache from the system
                    $oMod = new cApiModuleCollection();
                    $oMod->delete($iID);
                    cInclude('includes', 'functions.upl.php');
                    $sModulePath = uplCreateFriendlyName($sName);
                    if (is_dir($cfgClient[$client]['path']['frontend'] . 'data/modules/' . $sModulePath)) {
                        removeDir($cfgClient[$client]['path']['frontend'] . 'data/modules/' . $sModulePath);
                    }
                    echo '<p>' . sprintf(i18n("Module %s deleted successfully"), $sName) . '</p>';
                    
                    if (strlen($sModule)) {
                        $oModule = new ModuleInfo($sModule);
                        
                        # Get the module info
                        $aInfo = $oModule->getInfo();
                        
                        # Only if there are no more modules of the same package
                        $sql = 'SELECT COUNT(`idmod`) AS num
                                FROM `' . $cfg['tab']['mod'] . '`
                                WHERE (`orig_name`="' . $aInfo['Name'] . '")';
                        $db->query($sql);
                        $db->next_record();
                        if ($db->f('num') == 0) {
                            
                            # Establish an (S)FTP connection if nessecary
                            $bUseFTP = false;
                            if (intval(getProperty(0, 'system', 'setting', 'ftp', 'use'))) {
                                $bUseFTP = true;
                                
                                # Get the FTP access data
                                $sFTPServer         = getProperty(0, 'system', 'setting', 'ftp', 'server');
                                $iFTPPort           = intval(getProperty(0, 'system', 'setting', 'ftp', 'port'));
                                $sFTPProtocol       = getProperty(0, 'system', 'setting', 'ftp', 'protocol');
                                $iFTPPort           = (($iFTPPort) ? $iFTPPort : (($sFTPProtocol == 'sftp') ? 22 : 21));
                                $sFTPUser           = getProperty(0, 'system', 'setting', 'ftp', 'user');
                                $sFTPPassword       = getProperty(0, 'system', 'setting', 'ftp', 'password');
                                $sBackendFTPPath    = getProperty(0, 'system', 'setting', 'ftp', 'backend_path');
                                $sFrontendFTPPath   = getProperty(0, 'system', 'setting', 'ftp', 'frontend_path');
                                
                                # Establish the FTP connection
                                switch ($sFTPProtocol) {
                                    case 'ftp':
                                        $ftp = new ftp($sFTPServer, $iFTPPort, $sFTPUser, $sFTPPassword, $sFrontendFTPPath);
                                        break;
                                    case 'sftp':
                                        $ftp = new sftp($sFTPServer, $iFTPPort, $sFTPUser, $sFTPPassword, $sFrontendFTPPath);
                                        break;
                                }
                                if (!$ftp) {
                                    echo $notification->returnNotification("error", sprintf(i18n("Unable to create a (S)FTP connection to %s"), $sFTPServer));
                                    flush();
                                    $bOK = false;
                                    break;
                                }
                                echo '<p>' . i18n("FTP connection established") . '</p>';
                                flush();
                            }
                            
                            # If there are additional folders and files in the module's folder, delete them in the client's folders
                            $aFoldersAndFiles = $oModule->getAdditionalFoldersAndFiles();
                            if (($ret = array_search('php/', $aFoldersAndFiles)) === false) {
                                if (count($aFoldersAndFiles)) {
                                    echo '<p>' . sprintf(i18n("Deleting additional folders and files for %s"), $aInfo['Name']) . '</p>';
                                    flush();
                                    $aFoldersAndFiles = array_reverse($aFoldersAndFiles);
                                    if ($bUseFTP) {
                                        
                                        # Determin the client's path under the FTP frontend path
                                        $aPath = explode('/', $cfgClient[$client]['path']['frontend']);
                                        while ((count($aPath)) && (!$ftp->isdir($sFrontendFTPPath . str_replace($cfg['path']['frontend'], '', $cfgClient[$client]['path']['frontend']) . implode('/', $aPath)))) {
                                            array_shift($aPath);
                                        }
                                        $sClientPath = $sFrontendFTPPath . str_replace($cfg['path']['frontend'], '', $cfgClient[$client]['path']['frontend']) . implode('/', $aPath);
                                    }
                                    foreach ($aFoldersAndFiles as $entry) {
                                        if (substr($entry, -1) == '/') {
                                            
                                            # Entry is a folder, delete it if it's empty and not a system folder
                                            if (!in_array(substr($entry, 0, -1), array('cache', 'css', 'images', 'includes', 'js', 'logs', 'templates', 'upload', 'version'))) {
                                                if ($bUseFTP) {
                                                    $ftp->rmdir($sClientPath . $entry);
                                                }
                                                else {
                                                    @rmdir($cfgClient[$client]['path']['frontend'] . $entry);
                                                }
                                            }
                                        }
                                        else {
                                            
                                            # Entry is a file, delete it
                                            if ($bUseFTP) {
                                                $ftp->delete($sClientPath . $entry);
                                            }
                                            else {
                                                unlink($cfgClient[$client]['path']['frontend'] . $entry);
                                            }
                                        }
                                    }
                                }
                            }
                            
                            # Get additional database tables to delete
                            $aDbTables = $oModule->getAdditionalTablesToDelete();
                            if (is_array($aDbTables)) {
                                echo '<p>' . sprintf(i18n("Deleting additional database tables for %s"), $aInfo['Name']) . '</p>';
                                flush();
                                foreach ($aDbTables as $DbTable) {
                                    if (strpos($DbTable, '*') !== false) {
                                        $aTables = array();
                                        $sql = 'SELECT TABLE_NAME
                                                FROM information_schema.TABLES
                                                WHERE (TABLE_NAME LIKE "' . str_replace(array('!PREFIX!', '*'), array($cfg['sql']['sqlprefix'], '%'), $DbTable) . '")';
                                        $db->query($sql);
                                        while ($db->next_record()) {
                                            $aTables[] = $db->f(0);
                                        }
                                        for ($i = 0, $n = count($aTables); $i < $n; $i ++) {
                                            $sql = 'DROP TABLE IF EXISTS `' . $aTables[$i] . '`';
                                            $db->query($sql);
                                        }
                                    }
                                    else {
                                        $sql = 'DROP TABLE IF EXISTS `' . str_replace('!PREFIX!', $cfg['sql']['sqlprefix'], $DbTable) . '`';
                                        $db->query($sql);
                                    }
                                }
                            }
                            
                            # Delete the install folder
                            echo '<p>' . sprintf(i18n("Removing folders and files for %s"), $aInfo['Name']) . '</p>';
                            flush();
                            if ($bUseFTP) {
                                $ftp->rmdir($sFrontendFTPPath . str_replace($cfg['path']['frontend'], '', $cfgClient[$client]['path']['frontend']) . 'data/modules_c/' . $sModule . '/', true);
                            }
                            else {
                                removeDir($cfgClient[$client]['path']['frontend'] . 'data/modules_c/' . $sModule . '/');
                            }
                            
                            if ($bUseFTP) {
                                $ftp->quit();
                                unset($ftp);
                            }
                        }
                    }
                    
                    echo $notification->returnNotification('info', sprintf(i18n("Module %s successfully uninstalled and removed"), $sName));
                    echo $sReloadScript;
                    flush();
                    break;
            }
        }
        break;
    
    case 'repositories':
        if ($perm->have_perm_area_action($plugin_name, 'repositories')) {
            if ((isset($_POST['Name'])) && (strlen($_POST['Name'])) && (isset($_POST['Address'])) && (strlen($_POST['Address']))) {
                
                # Add a new repository
                $sAddress = $_POST['Address'];
                if (strpos($sAddress, '://') === false) {
                    $sAddress = 'http://' . $sAddress;
                }
                if ((substr($sAddress, -1) != '/') && (!in_array(substr($sAddress, -4), array('.php', '.htm', 'html')))) {
                    $sAddress .= '/';
                }
                setProperty(0, 'module_repository', $_POST['Name'], 'server', 'host', $sAddress, intval(((isset($_POST['AddressKey'])) ? $_POST['AddressKey'] : 0)));
                setProperty(0, 'module_repository', $_POST['Name'], 'server', 'login', $_POST['Login'], intval(((isset($_POST['LoginKey'])) ? $_POST['LoginKey'] : 0)));
                setProperty(0, 'module_repository', $_POST['Name'], 'server', 'password', $_POST['Password'], intval(((isset($_POST['PasswordKey'])) ? $_POST['PasswordKey'] : 0)));
            }
            elseif ($sAction == 'Delete') {
                
                # Delete a repository
                $sql = 'DELETE FROM ' . $cfg['tab']['properties'] . '
                        WHERE (`idproperty` IN (' . str_replace('|', ',', $_POST['Key']) . '))';
                $db->query($sql);
            }
            
            $oTpl->set('s', 'FORM_ACTION', 'main.php?area=mom&what=repositories&frame=4&contenido=' . $contenido);
            
            # Texts
            $oTpl->set('s', 'TEXT_REPOSITORIES', i18n("Repositories"));
            $oTpl->set('s', 'TEXT_NAME', i18n("Name"));
            $oTpl->set('s', 'TEXT_ADDRESS', i18n("Server address"));
            $oTpl->set('s', 'TEXT_LOGIN', i18n("Login"));
            $oTpl->set('s', 'TEXT_PASSWORD', i18n("Password"));
            $oTpl->set('s', 'TEXT_EDIT', i18n("Edit"));
            $oTpl->set('s', 'TEXT_EDIT_NOT_POSSIBLE', i18n("Edit not possible"));
            $oTpl->set('s', 'TEXT_DELETE', i18n("Delete"));
            $oTpl->set('s', 'TEXT_DELETE_NOT_POSSIBLE', i18n("Delete not possible"));
            $oTpl->set('s', 'TEXT_NEW', i18n("Add a new repository"));
            $oTpl->set('s', 'TEXT_SAVE', i18n("Save"));
            
            # Get added repositories
            $aReposInfo = getPropertiesByItemtype('module_repository', 'itemid', 0, true);
            $aRepos = array();
            for ($i = 0, $n = count($aReposInfo); $i < $n; $i ++) {
                $aRepos[$aReposInfo[$i]['itemid']][$aReposInfo[$i]['name']] = array('idproperty' => $aReposInfo[$i]['idproperty'], 'value' => $aReposInfo[$i]['value']);
            }
            
            # Added repositories
            $sClass = 'even';
            foreach ($aRepos as $itemid => $server) {
                $oTpl->set('d', 'CLASS', $sClass);
                $oTpl->set('d', 'NAME', $itemid);
                $oTpl->set('d', 'ADDRESS', $server['host']['value']);
                $oTpl->set('d', 'LOGIN', $server['login']['value']);
                $oTpl->set('d', 'PASSWORD', ((strlen($server['password']['value'])) ? '******' : '&nbsp;'));
                $oTpl->set('d', 'ADDRESS_ID', $server['host']['idproperty']);
                $oTpl->set('d', 'LOGIN_ID', $server['login']['idproperty']);
                $oTpl->set('d', 'PASSWORD_ID', $server['password']['idproperty']);
                $oTpl->next();
                $sClass = (($sClass == 'odd') ? 'even' : 'odd');
            }
            
            echo $oTpl->generate($cfg['path']['contenido'] . $cfg['path']['plugins'] . $plugin_name . '/templates/default/repositories.html', true);
            flush();
            $oTpl->reset();
        }
        break;
    
    case 'settings':
        if ($perm->have_perm_area_action($plugin_name, 'settings')) {
            if (isset($_POST['Save'])) {
                
                # Correct path settings
                $_POST['BackendFTPPath'] = str_replace(array('\\\\', '\\'), '/', $_POST['BackendFTPPath']);
                if (substr($_POST['BackendFTPPath'], 0, 1) != '/') {
                    $_POST['BackendFTPPath'] = '/' . $_POST['BackendFTPPath'];
                }
                if (substr($_POST['BackendFTPPath'], -1) != '/') {
                    $_POST['BackendFTPPath'] = $_POST['BackendFTPPath'] . '/';
                }
                $_POST['FrontendFTPPath'] = str_replace(array('\\\\', '\\'), '/', $_POST['FrontendFTPPath']);
                if (substr($_POST['FrontendFTPPath'], 0, 1) != '/') {
                    $_POST['FrontendFTPPath'] = '/' . $_POST['FrontendFTPPath'];
                }
                if (substr($_POST['FrontendFTPPath'], -1) != '/') {
                    $_POST['FrontendFTPPath'] = $_POST['FrontendFTPPath'] . '/';
                }
                
                # Save the settings
                setProperty(0, 'system', 'setting', 'ftp', 'use', $_POST['UseFTP'], intval(((isset($_POST['UseFTPKey'])) ? $_POST['UseFTPKey'] : 0)));
                setProperty(0, 'system', 'setting', 'ftp', 'server', $_POST['FTPServer'], intval(((isset($_POST['FTPServerKey'])) ? $_POST['FTPServerKey'] : 0)));
                setProperty(0, 'system', 'setting', 'ftp', 'port', $_POST['FTPPort'], intval(((isset($_POST['FTPPortKey'])) ? $_POST['FTPPortKey'] : 0)));
                setProperty(0, 'system', 'setting', 'ftp', 'user', $_POST['FTPUser'], intval(((isset($_POST['FTPUserKey'])) ? $_POST['FTPUserKey'] : 0)));
                setProperty(0, 'system', 'setting', 'ftp', 'password', $_POST['FTPPassword'], intval(((isset($_POST['FTPPasswordKey'])) ? $_POST['FTPPasswordKey'] : 0)));
                setProperty(0, 'system', 'setting', 'ftp', 'protocol', $_POST['FTPProtocol'], intval(((isset($_POST['FTPProtocolKey'])) ? $_POST['FTPProtocolKey'] : 0)));
                setProperty(0, 'system', 'setting', 'ftp', 'backend_path', $_POST['BackendFTPPath'], intval(((isset($_POST['BackendFTPPathKey'])) ? $_POST['BackendFTPPathKey'] : 0)));
                setProperty(0, 'system', 'setting', 'ftp', 'frontend_path', $_POST['FrontendFTPPath'], intval(((isset($_POST['FrontendFTPPathKey'])) ? $_POST['FrontendFTPPathKey'] : 0)));
            }
            
            $oTpl->set('s', 'FORM_ACTION', 'main.php?area=mom&what=settings&frame=4&contenido=' . $contenido);
            
            # Texts
            $oTpl->set('s', 'TEXT_SETTINGS', i18n("Settings"));
            $oTpl->set('s', 'TEXT_USE_FTP', i18n("Use FTP"));
            $oTpl->set('s', 'TEXT_FTP_SERVER', i18n("FTP Server"));
            $oTpl->set('s', 'TEXT_FTP_PORT', i18n("FTP Port"));
            $oTpl->set('s', 'TEXT_FTP_USER', i18n("FTP User"));
            $oTpl->set('s', 'TEXT_FTP_PASSWORD', i18n("FTP Password"));
            $oTpl->set('s', 'TEXT_FTP_PROTOCOL', i18n("FTP Protocol"));
            $oTpl->set('s', 'TEXT_BACKEND_FTP_PATH', i18n("Backend Path per FTP"));
            $oTpl->set('s', 'TEXT_FRONTEND_FTP_PATH', i18n("Frontend Path per FTP<br />(where the client's folders are in)"));
            $oTpl->set('s', 'TEXT_SAVE', i18n("Save"));
            
            # Get settings
            $aSettingsInfo = getPropertiesByItemtype('system', 'itemid', 0, true);
            if (count($aSettingsInfo)) {
                foreach ($aSettingsInfo as $itemid => $aSetting) {
                    switch ($aSetting['name']) {
                        case 'use':
                            $oTpl->set('s', 'USE_FTP', (($aSetting['value'] == '1') ? ' checked="checked"' : ''));
                            $oTpl->set('s', 'USE_FTP_KEY', $aSetting['idproperty']);
                            break;
                        case 'server':
                            $oTpl->set('s', 'FTP_SERVER', $aSetting['value']);
                            $oTpl->set('s', 'FTP_SERVER_KEY', $aSetting['idproperty']);
                            break;
                        case 'port':
                            $oTpl->set('s', 'FTP_PORT', $aSetting['value']);
                            $oTpl->set('s', 'FTP_PORT_KEY', $aSetting['idproperty']);
                            break;
                        case 'user':
                            $oTpl->set('s', 'FTP_USER', $aSetting['value']);
                            $oTpl->set('s', 'FTP_USER_KEY', $aSetting['idproperty']);
                            break;
                        case 'password':
                            $oTpl->set('s', 'FTP_PASSWORD', $aSetting['value']);
                            $oTpl->set('s', 'FTP_PASSWORD_KEY', $aSetting['idproperty']);
                            break;
                        case 'protocol':
                            $oTpl->set('s', 'FTP_PROTOCOL1', (($aSetting['value'] != 'sftp') ? ' checked="checked"' : ''));
                            $oTpl->set('s', 'FTP_PROTOCOL2', (($aSetting['value'] == 'sftp') ? ' checked="checked"' : ''));
                            $oTpl->set('s', 'FTP_PROTOCOL_KEY', $aSetting['idproperty']);
                            break;
                        case 'backend_path':
                            $oTpl->set('s', 'BACKEND_FTP_PATH', $aSetting['value']);
                            $oTpl->set('s', 'BACKEND_FTP_PATH_KEY', $aSetting['idproperty']);
                            break;
                        case 'frontend_path':
                            $oTpl->set('s', 'FRONTEND_FTP_PATH', $aSetting['value']);
                            $oTpl->set('s', 'FRONTEND_FTP_PATH_KEY', $aSetting['idproperty']);
                            break;
                    }
                }
            }
            
            # Set unset template variables
            $oTpl->set('s', 'USE_FTP', '');
            $oTpl->set('s', 'USE_FTP_KEY', '');
            $oTpl->set('s', 'FTP_SERVER', '');
            $oTpl->set('s', 'FTP_SERVER_KEY', '');
            $oTpl->set('s', 'FTP_PORT', '21');
            $oTpl->set('s', 'FTP_PORT_KEY', '');
            $oTpl->set('s', 'FTP_USER', '');
            $oTpl->set('s', 'FTP_USER_KEY', '');
            $oTpl->set('s', 'FTP_PASSWORD', '');
            $oTpl->set('s', 'FTP_PASSWORD_KEY', '');
            $oTpl->set('s', 'FTP_PROTOCOL1', ' checked="checked"');
            $oTpl->set('s', 'FTP_PROTOCOL2', '');
            $oTpl->set('s', 'FTP_PROTOCOL_KEY', '');
            $oTpl->set('s', 'BACKEND_FTP_PATH', '');
            $oTpl->set('s', 'BACKEND_FTP_PATH_KEY', '');
            $oTpl->set('s', 'FRONTEND_FTP_PATH', '');
            $oTpl->set('s', 'FRONTEND_FTP_PATH_KEY', '');
            
            echo $oTpl->generate($cfg['path']['contenido'] . $cfg['path']['plugins'] . $plugin_name . '/templates/default/settings.html', true);
            flush();
            $oTpl->reset();
        }
        break;
}

echo '
</body>
</html>';
unset($plugin_name);
?>