<?php
 /**
 * Project:
 * Contenido Content Management System
 *
 * Description:
 * Creates/Updates the database tables and fills them with entries (depending on 
 * selected options during setup process)
 *
 * @package    Contenido setup
 * @version    0.2.2
 * @author     unknown
 * @copyright  four for business AG <www.4fb.de>
 * @license    http://www.contenido.org/license/LIZENZ.txt
 * @link       http://www.4fb.de
 * @link       http://www.contenido.org
 *
 *   $Id$:
 */

if (!defined('CON_FRAMEWORK')) {
    define('CON_FRAMEWORK', true);
}
define('C_CONTENIDO_PATH', '../admin/');

include_once('lib/startup.php');


checkAndInclude(C_CONTENIDO_PATH . 'includes/functions.database.php');
checkAndInclude(C_CONTENIDO_PATH . 'classes/class.version.php');

function removeDir($sDir, $leaveDir = false) {
    if (!$dh = @opendir($sDir)) {
        return;
    }
    while (($obj = readdir($dh)) !== false) {
        if (($obj == '.') || ($obj == '..')) {
            continue;
        }
        if (!@unlink($sDir . '/' . $obj)) {
            removeDir($sDir . '/' . $obj);
        }
    }
    closedir($dh);
    if (!$leaveDir){
        @rmdir($sDir);
    }
}

if (!hasMySQLiExtension()) {
    die("Can't detect MySQLi extension");
} else {
    $cfg['database_extension'] = $_SESSION['dbmode'];
}

checkAndInclude('../conlib/prepend.php');

$db = getSetupMySQLDBConnection(false);

if (checkMySQLDatabaseCreation($db, $_SESSION['dbname'])) {
    $db = getSetupMySQLDBConnection();
}

$currentstep = intval($_GET['step']);

if ($currentstep == 0) {
    $currentstep = 1;
}

if (($_SESSION["setuptype"] == 'setup') && (strlen($_SESSION["dbencoding"])) && ($currentstep == 1)) {
    $db->query('ALTER DATABASE `' . $_SESSION['dbname'] . '` CHARACTER SET ' . substr($_SESSION['dbencoding'], 0, strpos($_SESSION['dbencoding'], '_')) . ' COLLATE ' . $_SESSION['dbencoding']);
}

// Count DB Chunks
$file = fopen('data/tables.txt', 'r');
$step = 1;
while (($data = fgetcsv($file, 4000, ';')) !== false) {
    if ($count == 50) {
        $count = 0;
        $step ++;
    }
    
    if ($currentstep == $step) {
        if ($data[7] == '1') {
            $drop = true;
        } else {
            $drop = false;
        }
        dbUpgradeTable($db, $_SESSION['dbprefix'].'_'.$data[0], $data[1], $data[2], $data[3], $data[4], $data[5], $data[6], '', $drop);
        
        if ($db->errno != 0) {
            $_SESSION['install_failedupgradetable'] = true;
        }
    }
    
    $count ++;
    $fullcount ++;
}

$baseChunks = explode("\n", file_get_contents('data/base.txt'));
if ($_SESSION['setuptype'] == 'setup') {
    $sysadminChunk = explode("\n", file_get_contents('data/sysadmin.txt'));
    $fullChunks = array_merge($baseChunks, $sysadminChunk);
} elseif ($_SESSION['setuptype'] == 'upgrade') {
    $upgradeChunks = explode("\n", file_get_contents('data/upgrade.txt'));
    $fullChunks = array_merge($baseChunks, $upgradeChunks);
} else {
    $fullChunks = $baseChunks;
}

list($root_path, $root_http_path) = getSystemDirectories();

$totalsteps = ceil($fullcount / 50) + count($fullChunks) + 1;
foreach ($fullChunks as $fullChunk) {
    $step ++;
    if ($step == $currentstep) {
        $failedChunks = array();

        $replacements = array(
            '<!--{contenido_root}-->' => addslashes(str_replace('\\', '/', $root_path)),
            '<!--{contenido_web}-->' => addslashes(str_replace('\\', '/', $root_http_path))
        );

        #echo '<div style="font-family: Arial, Verdana, sans-serif; font-size: 12px;">do insertSQL for file ' . $fullChunk . '</div>';
        injectSQL($db, $_SESSION['dbprefix'], 'data/' . $fullChunk, $replacements, $failedChunks);

        if (count($failedChunks) > 0) {
            if (@$fp = fopen('../admin/data/logs/setuplog.txt', 'a')) {
                foreach ($failedChunks as $failedChunk) {
                    fwrite($fp, sprintf("Setup was unable to execute SQL. MySQL-Error: %s, MySQL-Message: %s, SQL-Statements:\n%s", $failedChunk['errno'], $failedChunk['error'], $failedChunk['sql']) . "\n\n");
                }
                fclose($fp);
            }
            else {
                echo sprintf("Setup was unable to execute SQL. MySQL-Error: %s, MySQL-Message: %s, SQL-Statements:\n%s", $failedChunk['errno'], $failedChunk['error'], $failedChunk['sql']);
                echo '<script type="text/javascript">alert("' . sprintf("Setup was unable to execute SQL. MySQL-Error: %s, MySQL-Message: %s, SQL-Statements:\n%s", $failedChunk['errno'], $failedChunk['error'], $failedChunk['sql']) . '");</script>';
            }
            $_SESSION['install_failedchunks'] = true;
        }
    }
}

$percent = intval((100 / $totalsteps) * ($currentstep));

echo '<script type="text/javascript">parent.updateProgressbar(' . $percent . ');</script>';
if ($currentstep < $totalsteps) {
    printf('<script type="text/javascript">window.setTimeout("nextStep()", 5); function nextStep() { window.location.href="dbupdate.php?step=%s"; }</script>', $currentstep + 1);
} else {
    $tables = array();
    $sql = 'SHOW TABLES';
    $db->query($sql);
    while ($db->next_record()) {
        $tables[] = $db->f(0);
    }

    foreach ($tables as $table) {
        dbUpdateSequence($_SESSION['dbprefix'].'_sequence', $table, $db);
    }

    updateContenidoVersion($db, $_SESSION['dbprefix'].'_system_prop', C_SETUP_VERSION);
    updateSystemProperties($db, $_SESSION['dbprefix'].'_system_prop');

    if (isset($_SESSION['sysadminpass']) && $_SESSION['sysadminpass'] != '') {
        updateSysadminPassword($db, $_SESSION['dbprefix'].'_phplib_auth_user_md5', 'sysadmin');
    }

    $sql = 'DELETE FROM ' . $_SESSION['dbprefix'].'_code';
    $db->query($sql);

    // As con_code has been emptied, force code creation (on update)
    $sql = 'UPDATE ' . $_SESSION['dbprefix'] .'_cat_art SET createcode = 1';
    $db->query($sql);

    if ($_SESSION['setuptype'] == 'migration') {
        $aClients = listClients($db, $_SESSION['dbprefix'].'_clients');

        foreach ($aClients as $iIdClient => $aInfo) {
            updateClientPath($db, $_SESSION['dbprefix'].'_clients', $iIdClient, str_replace('\\', '/', $_SESSION['frontendpath'][$iIdClient]), str_replace('\\', '/', $_SESSION['htmlpath'][$iIdClient]));
        }
    }

    $_SESSION['start_compatible'] = false;

    if ($_SESSION['setuptype'] == 'upgrade') {
        $sql = 'SELECT is_start FROM ' . $_SESSION['dbprefix'].'_cat_art WHERE is_start = 1';
        $db->query($sql);
        if ($db->next_record()) {
            $_SESSION['start_compatible'] = true;
        }
    }

    // Update Keys
    $aNothing = array();

    injectSQL($db, $_SESSION['dbprefix'], 'data/indexes.sql', array(), $aNothing);

    if ($_SESSION['setuptype'] == 'upgrade') {
        /**
         * For upgrades from pre 2.1.0 we need to eventually move the plugins to the new admin folder.
         * We can't just upgrade them, as they may be individually reworked.
         * Also, if the backend was in another folder, we need to redirect the users.
         */
        $sPath = '';
        if (file_exists($root_path . "/admin/data/config/config.php")) {
            $sPath = 'admin';
        }
        elseif (file_exists($root_path . "/admin/includes/config.php")) {
            $sPath = 'admin';
        }
        elseif (file_exists($root_path . "/drugcms/includes/config.php")) {
            $sPath = 'drugcms';
            checkAndInclude($root_path . "/drugcms/includes/config.php");
        }
        else {
            $sPath = 'contenido';
            checkAndInclude($root_path . "/contenido/includes/config.php");
        }
        switch ($sPath) {
            case 'drugcms':
            case 'contenido':
                
                # Copy the plugins (except system plugins) to the admin plugins folder
                $sSource = $cfg['path']['contenido'] . 'plugins/';
                $aPlugins = getSubdirs($sSource);
                foreach ($aPlugins as $sPlugin) {
                    if (!in_array($sPlugin, array('chains', 'ds', 'frontendlogic', 'frontendusers', 'mom', 'pim', 'repository'))) {
                        recursive_copy($sSource . $sPlugin, C_CONTENIDO_PATH . 'plugins/' . $sPlugin);
                    }
                }
                
                # Create a new file 'index.php' in the old backend folder to redirect to the new one
                $fp = fopen($cfg['path']['contenido'] . 'index.php', 'w');
                fwrite($fp, '<?php header("Location: ../admin/index.php"); ?>');
                fclose($fp);
                break;
        }
        /**
         * Do some cleanup in the conlib directory
         */
        foreach (glob($root_path . '/conlib/*.inc') as $sFile) {
            unlink($sFile);
        }
        /**
         * For upgrades from pre 2.2.0 we must delete some old config files (they are now in admin/data/config/).
         */
        unlink($root_path . '/' . $sPath . '/includes/cfg_language_de.inc.php');
        unlink($root_path . '/' . $sPath . '/includes/cfg_sgl.inc.php');
        unlink($root_path . '/' . $sPath . '/includes/config.chains.php');
        unlink($root_path . '/' . $sPath . '/includes/config.colors.php');
        unlink($root_path . '/' . $sPath . '/includes/config.http_check.php');
        unlink($root_path . '/' . $sPath . '/includes/config.misc.php');
        unlink($root_path . '/' . $sPath . '/includes/config.path.php');
        unlink($root_path . '/' . $sPath . '/includes/config.templates.php');
        /**
         * For upgrades from pre 2.2.0 we need to eventually move module files into the new structure.
         */
        $aClients = listClients($db, $_SESSION['dbprefix'].'_clients');
        foreach ($aClients as $iIdClient => $aInfo) {
            $sPath = $aInfo['frontendpath'];
            $sModPath = $sPath . 'data/modules/';
            $aMods = getSubdirs($sModPath);
            foreach ($aMods as $sMod) {
                $sMod = basename($sMod);
                if (((file_exists($sModPath . $sMod . '/' . $sMod . '-input.php')) || (file_exists($sModPath . $sMod . '/' . $sMod . '-output.php')) || (file_exists($sModPath . $sMod . '/input.php')) || (file_exists($sModPath . $sMod . '/output.php'))) && (!is_dir($sModPath . $sMod . '/php/'))) {
                    mkdir($sModPath . $sMod . '/css/', 0755, true);
                    mkdir($sModPath . $sMod . '/images/', 0755, true);
                    mkdir($sModPath . $sMod . '/js/', 0755, true);
                    mkdir($sModPath . $sMod . '/php/', 0755, true);
                    mkdir($sModPath . $sMod . '/templates/', 0755, true);
                    if (file_exists($sModPath . $sMod . '/' . $sMod . '-input.php')) {
                        rename($sModPath . $sMod . '/' . $sMod . '-input.php', $sModPath . $sMod . '/php/' . $sMod . '-input.php');
                    }
                    elseif (file_exists($sModPath . $sMod . '/input.php')) {
                        rename($sModPath . $sMod . '/input.php', $sModPath . $sMod . '/php/' . $sMod . '-input.php');
                    }
                    if (file_exists($sModPath . $sMod . '/' . $sMod . '-output.php')) {
                        rename($sModPath . $sMod . '/' . $sMod . '-output.php', $sModPath . $sMod . '/php/' . $sMod . '-output.php');
                    }
                    elseif (file_exists($sModPath . $sMod . '/output.php')) {
                        rename($sModPath . $sMod . '/output.php', $sModPath . $sMod . '/php/' . $sMod . '-output.php');
                    }
                }
            }
        }
        /**
         * Modules not exported to files need to get so right now.
         */
        $sql = 'SELECT `idclient`, `name`, `input`, `output`
                FROM `' . $_SESSION['dbprefix'] . '_mod`
                ORDER BY `idmod`';
        $db->query($sql);
        while ($db->next_record()) {
            $sPath = $aClients[$db->f('idclient')]['frontendpath'];
            $sModPath = $sPath . 'data/modules/';
            $sMod = uplCreateFriendlyName($db->f('name'));
            if (!is_dir($sModPath . $sMod)) {
                mkdir($sModPath . $sMod . '/css/', 0755, true);
                mkdir($sModPath . $sMod . '/images/', 0755, true);
                mkdir($sModPath . $sMod . '/js/', 0755, true);
                mkdir($sModPath . $sMod . '/php/', 0755, true);
                mkdir($sModPath . $sMod . '/templates/', 0755, true);
                file_put_contents($sModPath . $sMod . '/php/' . $sMod . '-input.php', str_replace('\\', '\\\\', $db->f('input')));
                file_put_contents($sModPath . $sMod . '/php/' . $sMod . '-output.php', str_replace('\\', '\\\\', $db->f('output')));
            }
        }
        /**
         * Layouts not exported to files need to get so right now.
         */
        $sql = 'SELECT `idclient`, `name`, `code`
                FROM `' . $_SESSION['dbprefix'] . '_lay`
                ORDER BY `idlay`';
        $db->query($sql);
        while ($db->next_record()) {
            $sPath = $aClients[$db->f('idclient')]['frontendpath'];
            $sLayPath = $sPath . 'data/layouts/';
            if (!is_dir($sLayPath)) {
                mkdir($sLayPath, 0755, true);
            }
            $sLay = uplCreateFriendlyName($db->f('name'));
            if (!file_exists($sLayPath . $sLay . '.php')) {
                file_put_contents($sLayPath . $sLay . '.php', str_replace('\\', '\\\\', $db->f('code')));
            }
        }
        /**
         * The front_content.php might have changed, copy it from 
         * admin/external/frontend/front_content.php to each client directory
         */
        foreach ($aClients as $aClient) {
            if (!file_exists($aClient['frontendpath'] . 'front_content.php.sic')) {
                rename($aClient['frontendpath'] . 'front_content.php', $aClient['frontendpath'] . 'front_content.php.sic');
            }
            elseif (!file_exists($aClient['frontendpath'] . 'front_content.php.s01')) {
                rename($aClient['frontendpath'] . 'front_content.php', $aClient['frontendpath'] . 'front_content.php.s01');
            }
            elseif (!file_exists($aClient['frontendpath'] . 'front_content.php.s02')) {
                rename($aClient['frontendpath'] . 'front_content.php', $aClient['frontendpath'] . 'front_content.php.s02');
            }
            elseif (!file_exists($aClient['frontendpath'] . 'front_content.php.s03')) {
                rename($aClient['frontendpath'] . 'front_content.php', $aClient['frontendpath'] . 'front_content.php.s03');
            }
            elseif (!file_exists($aClient['frontendpath'] . 'front_content.php.s04')) {
                rename($aClient['frontendpath'] . 'front_content.php', $aClient['frontendpath'] . 'front_content.php.s04');
            }
            copy(C_CONTENIDO_PATH . 'external/frontend/front_content.php', $aClient['frontendpath'] . 'front_content.php');
        }
        /**
         * Switch $force to true for all clients that have modules_in_files switched on
         */
        foreach ($aClients as $iClient => $aClient) {
            $sql = 'SELECT `value`
                    FROM `' . $_SESSION['dbprefix'] . '_properties`
                    WHERE ((`idclient`=' . $iClient . ')
                       AND (`itemtype`="clientsetting")
                       AND (`type`="modules_in_files")
                       AND (`name`="use"))';
            $db->query($sql);
            if ($db->next_record()) {
                if (in_array($db->f('value'), array('true', '1'))) {
                    file_put_contents($aClient['frontendpath'] . 'config.php', str_replace('$force = 0;', '$force = 1', file_get_contents($aClient['frontendpath'] . 'config.php')));
                }
            }
        }
    }
    
    printf('<script>
        setTimeout(function() {
            setTimeout(function() {
                parent.document.getElementById("installing").style.display="none";
                parent.document.getElementById("installingdone").style.display="block";
                parent.document.getElementById("next").style.visibility="visible";
            }, 1000);
            parent.document.getElementById("Next").click();
        }, 1000);
        </script>');
}
?>