<?php
 /**
 * Project:
 * Contenido Content Management System
 *
 * Description:
 * Generates the configuration file and saves it into contenido folder or
 * outputs the for download (depending on selected option during setup)
 *
 * Requirements:
 * @con_php_req 5
 *
 * @package    Contenido setup
 * @version    0.2.1
 * @author     unknown
 * @copyright  four for business AG <www.4fb.de>
 * @license    http://www.contenido.org/license/LIZENZ.txt
 * @link       http://www.4fb.de
 * @link       http://www.contenido.org
 * 
 *
 * {@internal
 *   created  unknown
 *   modified 2008-07-07, bilal arslan, added security fix
 *   modified 2011-02-28, Murat Purc, normalized setup startup process and some cleanup/formatting
 *
 *   $Id$:
 * }}
 *
 */

if (!defined('CON_FRAMEWORK')) {
    define('CON_FRAMEWORK', true);
}

define('C_CONTENIDO_PATH', '../admin/');
include_once('lib/startup.php');

class cConfigInstaller extends cSetupMask {
    function __construct() {
        cSetupMask::__construct("templates/makeconfig.tpl", 6);
        $this->setHeader(i18n_setup("System Installation"));
        $this->_oStepTemplate->set("s", "TITLE", i18n_setup("System Installation"));
        switch ($_SESSION["setuptype"]) {
            case "setup":
                $this->setNavigation("", "setup" . C_SETUP_STEPS);
                break;
            case "upgrade":
                $this->setNavigation("", "upgrade" . C_SETUP_STEPS);
                break;
            case "migration":
                $this->setNavigation("", "migration" . C_SETUP_STEPS);
                break;
        }
    }
    function setContent($saved) {
        if ($saved) {
            $this->_oStepTemplate->set("s", "DESCRIPTION", i18n_setup("Your config.php file has been saved successfully"));
            $this->_oStepTemplate->set("s", "DOWNLOADLINK", '');
        }
        else {
            $this->_oStepTemplate->set("s", "DESCRIPTION", i18n_setup("Please download your config.php file and place it in your admin/includes folder"));
            $this->_oStepTemplate->set("s", "DOWNLOADLINK", '<a href="makeconfig.php?file=' . urlencode('config.php') . '">config.php</a>');
        }
    }
    function addScript($script) {
        $this->_oStepTemplate->set("s", "SCRIPT", $script);
    }
    function makeConfig($save = false) {
        list($root_path, $root_http_path) = getSystemDirectories();

        $tpl = new Template();
        $tpl->set('s', 'CONTENIDO_ROOT', str_replace('\\', '/', $root_path));
        $tpl->set('s', 'CONTENIDO_WEB', str_replace('\\', '/', $root_http_path));
        $tpl->set('s', 'MYSQL_HOST', $_SESSION['dbhost']);
        $tpl->set('s', 'MYSQL_DB', $_SESSION['dbname']);
        $tpl->set('s', 'MYSQL_USER', $_SESSION['dbuser']);
        $tpl->set('s', 'MYSQL_PASS', $_SESSION['dbpass']);
        $tpl->set('s', 'MYSQL_PREFIX', $_SESSION['dbprefix']);
        $tpl->set('s', 'DB_ENCODING', $_SESSION['dbencoding']);
        $tpl->set('s', 'DB_EXTENSION', $_SESSION['dbmode']);

        $tpl->set('s', 'START_COMPATIBLE', (($_SESSION['start_compatible']) ? 'true' : 'false'));

        $tpl->set('s', 'NOLOCK', $_SESSION['nolock']);

        if ($save) {
            @unlink($root_path . '/admin/data/config/config.php');
            @unlink($root_path . '/admin/includes/config.php');

            @$handle = fopen($root_path . '/admin/data/config/config.php', 'wb');
            @fwrite($handle, $tpl->generate('templates/config.php.tpl', true, false));
            @fclose($handle);

            return file_exists($root_path . '/admin/data/config/config.php');
        }
        else {
            return $tpl->generate('templates/config.php.tpl', true, false);
        }
    }
}

$oCI = new cConfigInstaller();
if ((isset($_GET['file'])) && ($_GET['file'] == 'config.php')) {
    header('Content-Type: application/octet-stream');
    header('Etag: ' . md5(mt_rand()));
    header('Content-Disposition: attachment;filename=config.php');
    die($oCI->makeConfig(false));
}
else {
    $ret = false;
    $sScript = '';
    if ($_SESSION['configmode'] == 'save') {
        $ret = $oCI->makeConfig(true);
    }
    $oCI->setContent($ret);
    if ($ret) {
        $sScript = '<script>
            setTimeout(function() {
                $("#Step").val("' . $_SESSION["setuptype"] . C_SETUP_STEPS . '");
                $("#Next").trigger("click");
            }, 10);
        </script>';
    }
    $oCI->addScript($sScript);
    $oCI->render();
}
?>